@@ -9,6 +9,7 @@ typedef struct
    double from_val, to_val;
    Ecore_Animator *anim;
    Ecore_Timer *retry_timer;
+   int ddc_max;
    int retries;
 } Backlight_Device;
 
@@ -20,6 +21,8 @@ static Ecore_Job *zone_change_job = NULL;
 static Eina_List *_devices = NULL;
 static int        _devices_pending_ops = 0;
 static Eina_Bool  _devices_zones_update = EINA_FALSE;
+static Eina_Bool  _own_vt = EINA_TRUE;
+static Eina_Bool  _e_bl_suspend = EINA_FALSE;
 
 static void _backlight_devices_device_set(Backlight_Device *bd, double val);
 static void _backlight_devices_device_update(Backlight_Device *bd);
@@ -45,7 +48,8 @@ _backlight_mismatch_retry(Backlight_Device *bd)
        // and the delta between expected and val >= 0.05
        (fabs(bd->expected_val - bd->val) >= 0.05) &&
        // and we retried < 20 times
-       (bd->retries < 10))
+       (bd->retries < 10) &&
+       (_own_vt))
      { // try again
         printf("RETRY backlight set as %1.2f != %1.2f (expected) try=%i\n",
                bd->val, bd->expected_val, bd->retries);
@@ -81,17 +85,18 @@ static void
 _backlight_system_ddc_get_cb(void *data, const char *params)
 {
    char edid[257];
-   int id = -1, val = -1;
+   int id = -1, val = -1, max = -1;
    double fval;
    Backlight_Device *bd = data;
 
    if (!params) return;
-   if (sscanf(params, "%256s %i %i", edid, &id, &val) != 3) return;
+   if (sscanf(params, "%256s %i %i %i", edid, &id, &val, &max) != 4) return;
    if (!bd->edid) return;
    if (!!strncmp(bd->edid, edid, strlen(edid))) return;
    e_system_handler_del("ddc-val-get", _backlight_system_ddc_get_cb, bd);
    if (val < 0) fval = -1.0;
    else fval = (double)val / 100.0;
+   bd->ddc_max = max;
    if ((fabs(fval - bd->val) >= DBL_EPSILON) || (val == -1))
      {
         bd->val = fval;
@@ -225,6 +230,7 @@ _backlight_devices_device_set(Backlight_Device *bd, double val)
 {
    if (fabs(bd->expected_val - val) > DBL_EPSILON) bd->retries = 0;
    bd->val = bd->expected_val = val;
+   if (!_own_vt) return;
 #ifndef HAVE_WAYLAND_ONLY
    if (!strcmp(bd->dev, "randr"))
      {
@@ -233,6 +239,7 @@ _backlight_devices_device_set(Backlight_Device *bd, double val)
              Ecore_X_Randr_Output o = _backlight_devices_randr_output_get(e_comp->root, bd->output, bd->edid);
              if (o)
                {
+//                  fprintf(stderr, "BL: randr bklight %1.3f @ %1.3f\n", bd->val, ecore_time_get());
                   ecore_x_randr_output_backlight_level_set(e_comp->root, o, bd->val);
                   ecore_event_add(E_EVENT_BACKLIGHT_CHANGE, NULL, NULL, NULL);
                }
@@ -242,11 +249,18 @@ _backlight_devices_device_set(Backlight_Device *bd, double val)
 #endif
    if (!strncmp(bd->dev, "ddc:", 4))
      {
-        e_system_send("ddc-val-set", "%s %i %i", bd->dev + 4, 0x10, (int)(bd->val * 100.0)); // backlight val in e_system_ddc.c
+        double fval;
+
+//        fprintf(stderr, "BL: ddc bklight %1.3f @ %1.3f\n", bd->val, ecore_time_get());
+        if (bd->ddc_max) fval = bd->val * (double)bd->ddc_max;
+        else fval = bd->val * 100.0;
+        if (e_config->backlight.ddc)
+          e_system_send("ddc-val-set", "%s %i %i", bd->dev + 4, 0x10, (int)(fval)); // backlight val in e_system_ddc.c
         ecore_event_add(E_EVENT_BACKLIGHT_CHANGE, NULL, NULL, NULL);
      }
    else
      {
+//        fprintf(stderr, "BL: internal bklight %1.3f @ %1.3f\n", bd->val, ecore_time_get());
         e_system_send("bklight-set", "%s %i", bd->dev, (int)(bd->val * 1000.0));
         ecore_event_add(E_EVENT_BACKLIGHT_CHANGE, NULL, NULL, NULL);
      }
@@ -263,18 +277,15 @@ _backlight_devices_device_zone_get(Backlight_Device *bd)
    EINA_LIST_FOREACH(e_comp->zones, l, zone)
      {
         const char *id = zone->randr2_id;
-        fprintf(stderr, "look at %p %s\n", zone, id);
         if (!id)
           {
              const char *id2 = bd->edid;
              if (!id2) id2 = "";
              id = "";
-             fprintf(stderr, "cmp1 [%s] == [%s]\n", id, id2);
              if (!strcmp(id, id2)) return zone;
           }
         else
           {
-             fprintf(stderr, "cmp2 [%s] == [%s]\n", id, buf);
              if (!strcmp(id, buf)) return zone;
           }
      }
@@ -350,6 +361,10 @@ _backlight_devices_screen_lid_get(void)
      {
         if (sc->info.is_lid) return sc;
      }
+   EINA_LIST_FOREACH(e_randr2->screens, l, sc)
+     {
+        if (sc->info.backlight) return sc;
+     }
    return NULL;
 }
 
@@ -516,7 +531,9 @@ _backlight_system_ddc_list_cb(void *data EINA_UNUSED, const char *params)
 {
    const char *p = params;
    char dev[257], buf[343];
+   Eina_Hash *tmphash;
 
+   tmphash = eina_hash_string_superfast_new(NULL);
    e_system_handler_del("ddc-list", _backlight_system_ddc_list_cb, NULL);
    while ((p) && (*p))
      {
@@ -527,10 +544,14 @@ _backlight_system_ddc_list_cb(void *data EINA_UNUSED, const char *params)
              bl_devs = eina_list_append
                (bl_devs, eina_stringshare_add(buf));
              _backlight_devices_edid_register(buf, dev);
+             if (eina_hash_find(tmphash, dev))
+               printf("BL: DDC ERROR: You have multiple DDC screens with the same EDID [%s] - this will lead to weirdness.\n", dev);
+             eina_hash_add(tmphash, dev, dev);
              if (*p != ' ') break;
           }
         else break;
      }
+   eina_hash_free(tmphash);
    _backlight_devices_pending_done();
 }
 
@@ -638,6 +659,52 @@ _cb_handler_zone_change(void *data EINA_UNUSED, int type EINA_UNUSED, void *info
    return EINA_TRUE;
 }
 
+#ifndef HAVE_WAYLAND_ONLY
+static Eina_Bool
+_cb_handler_x_window_property(void *data EINA_UNUSED, int type EINA_UNUSED, void *event)
+{
+   Ecore_X_Event_Window_Property *ev = event;
+
+   if ((e_comp) && (e_comp->comp_type == E_PIXMAP_TYPE_X))
+     {
+        static Ecore_X_Atom atom_xfree86_has_vt = 0;
+
+        if (!atom_xfree86_has_vt)
+          atom_xfree86_has_vt = ecore_x_atom_get("XFree86_has_VT");
+        if ((ev->win) && (e_comp->root) && (ev->win == e_comp->root) &&
+            (ev->atom == atom_xfree86_has_vt))
+          {
+             int *val = NULL, num = 0;
+
+             if (ecore_x_window_prop_property_get(ev->win, atom_xfree86_has_vt,
+                                                  ECORE_X_ATOM_INTEGER, 32,
+                                                  (unsigned char **)&val,
+                                                  &num))
+               {
+                  if (num > 0)
+                    {
+                       if ((*val == 1) && (!_own_vt)) // have vt
+                         {
+                            printf("BL: gained VT\n");
+                            _own_vt = EINA_TRUE;
+                            // we just go back to normal backlight if we gain
+                            // the vt again
+                            e_backlight_mode_set(NULL, E_BACKLIGHT_MODE_NORMAL);
+                         }
+                       else if (_own_vt)
+                         {
+                            printf("BL: lost VT\n");
+                            _own_vt = EINA_FALSE;
+                         }
+                    }
+                  free(data);
+               }
+          }
+     }
+   return ECORE_CALLBACK_PASS_ON;
+}
+#endif
+
 EINTERN int
 e_backlight_init(void)
 {
@@ -651,6 +718,9 @@ e_backlight_init(void)
    H(E_EVENT_ZONE_MOVE_RESIZE, _cb_handler_zone_change);
    H(E_EVENT_ZONE_STOW, _cb_handler_zone_change);
    H(E_EVENT_ZONE_UNSTOW, _cb_handler_zone_change);
+#ifndef HAVE_WAYLAND_ONLY
+   H(ECORE_X_EVENT_WINDOW_PROPERTY, _cb_handler_x_window_property);
+#endif
    e_backlight_update();
    if (!getenv("E_RESTART"))
      e_backlight_level_set(NULL, e_config->backlight.normal, -1.0);
@@ -808,3 +878,17 @@ e_backlight_devices_get(void)
 {
    return bl_devs;
 }
+
+E_API void
+e_backlight_suspend_set(Eina_Bool suspend)
+{
+   if (_e_bl_suspend == suspend) return;
+   _e_bl_suspend = suspend;
+   _own_vt = !_e_bl_suspend;
+   if (_own_vt)
+     {
+        // we just go back to normal backlight if we gain
+        // the vt again
+        e_backlight_mode_set(NULL, E_BACKLIGHT_MODE_NORMAL);
+     }
+}
