@@ -236,10 +236,13 @@ struct E_Client
    } pre_cb;
    Eina_Rectangle client; //client geom
    Evas_Object *frame; //comp object
+   Evas_Object *frame_object; //frame border comp object
    Evas_Object *agent; //resize agent;
    E_Zone *zone;
    E_Desk *desk;
 
+   const char *restore_zone_id;
+
    Ecore_Poller              *ping_poller;
    Ecore_Timer               *kill_timer;
 
@@ -569,6 +572,10 @@ struct E_Client
       unsigned char                  vkbd E_BITFIELD;
    } vkbd;
 
+   struct {
+      unsigned int steam_game_id;
+   } steam;
+
    struct
    {
       unsigned char visible E_BITFIELD;
@@ -606,6 +613,7 @@ struct E_Client
    unsigned int       need_shape_merge E_BITFIELD;
    unsigned int       need_shape_export E_BITFIELD;
    unsigned int       fullscreen E_BITFIELD;
+   unsigned int       unfullscreen_forced E_BITFIELD;
    unsigned int       need_fullscreen E_BITFIELD;
    unsigned int       already_unparented E_BITFIELD;
    unsigned int       need_reparent E_BITFIELD;
@@ -648,6 +656,8 @@ struct E_Client
 
    unsigned int       stolen E_BITFIELD;
 
+   unsigned int       frozen E_BITFIELD;
+
    unsigned int       internal E_BITFIELD;
    unsigned int       internal_no_remember E_BITFIELD;
    unsigned int       internal_no_reopen E_BITFIELD;
@@ -805,6 +815,7 @@ E_API Eina_Bool e_client_focus_track_enabled(void);
 E_API void e_client_focus_track_freeze(void);
 E_API void e_client_focus_track_thaw(void);
 E_API void e_client_refocus(void);
+E_API void e_client_revert_focus(E_Client *ec);
 E_API void e_client_focus_set_with_pointer(E_Client *ec);
 E_API void e_client_activate(E_Client *ec, Eina_Bool just_do_it);
 E_API E_Client *e_client_focused_get(void);
