@@ -34,8 +34,6 @@ static Ecore_Timer *action_timeout = NULL;
 static Eina_Bool gl_avail = EINA_FALSE;
 static Eina_Bool shape_debug = EINA_FALSE;
 
-static double ecore_frametime = 0;
-
 static int _e_comp_log_dom = -1;
 
 E_API int E_EVENT_COMPOSITOR_UPDATE = -1;
@@ -192,10 +190,11 @@ _e_comp_fps_update(void)
         if (e_comp->canvas->fps_bg) return;
 
         e_comp->canvas->fps_bg = evas_object_rectangle_add(e_comp->evas);
-        evas_object_color_set(e_comp->canvas->fps_bg, 0, 0, 0, 128);
+        evas_object_color_set(e_comp->canvas->fps_bg, 0, 0, 0, 192);
         evas_object_layer_set(e_comp->canvas->fps_bg, E_LAYER_MAX);
         evas_object_name_set(e_comp->canvas->fps_bg, "e_comp->canvas->fps_bg");
         evas_object_lower(e_comp->canvas->fps_bg);
+        evas_object_pass_events_set(e_comp->canvas->fps_bg, EINA_TRUE);
         evas_object_show(e_comp->canvas->fps_bg);
 
         e_comp->canvas->fps_fg = evas_object_text_add(e_comp->evas);
@@ -203,14 +202,27 @@ _e_comp_fps_update(void)
         evas_object_text_text_set(e_comp->canvas->fps_fg, "???");
         evas_object_color_set(e_comp->canvas->fps_fg, 255, 255, 255, 255);
         evas_object_layer_set(e_comp->canvas->fps_fg, E_LAYER_MAX);
-        evas_object_name_set(e_comp->canvas->fps_bg, "e_comp->canvas->fps_fg");
+        evas_object_name_set(e_comp->canvas->fps_fg, "e_comp->canvas->fps_fg");
         evas_object_stack_above(e_comp->canvas->fps_fg, e_comp->canvas->fps_bg);
+        evas_object_pass_events_set(e_comp->canvas->fps_fg, EINA_TRUE);
         evas_object_show(e_comp->canvas->fps_fg);
+
+        e_comp->canvas->fps_gr = evas_object_image_filled_add(e_comp->evas);
+        evas_object_image_smooth_scale_set(e_comp->canvas->fps_gr, EINA_FALSE);
+        evas_object_image_alpha_set(e_comp->canvas->fps_gr, EINA_TRUE);
+        evas_object_image_size_set(e_comp->canvas->fps_gr, 1, 1);
+        evas_object_color_set(e_comp->canvas->fps_gr, 255, 255, 255, 255);
+        evas_object_layer_set(e_comp->canvas->fps_gr, E_LAYER_MAX);
+        evas_object_name_set(e_comp->canvas->fps_gr, "e_comp->canvas->fps_gr");
+        evas_object_stack_above(e_comp->canvas->fps_gr, e_comp->canvas->fps_fg);
+        evas_object_pass_events_set(e_comp->canvas->fps_gr, EINA_TRUE);
+        evas_object_show(e_comp->canvas->fps_gr);
      }
    else
      {
         E_FREE_FUNC(e_comp->canvas->fps_fg, evas_object_del);
         E_FREE_FUNC(e_comp->canvas->fps_bg, evas_object_del);
+        E_FREE_FUNC(e_comp->canvas->fps_gr, evas_object_del);
      }
 }
 
@@ -258,6 +270,7 @@ _e_comp_cb_nocomp_begin(void)
    e_comp_render_queue();
    e_comp_shape_queue_block(1);
    ecore_event_add(E_EVENT_COMPOSITOR_DISABLE, NULL, NULL, NULL);
+   printf("COMP: suspend\n");
 }
 
 static void
@@ -267,6 +280,7 @@ _e_comp_cb_nocomp_end(void)
 
    if (!e_comp->nocomp) return;
 
+   printf("COMP: resume\n");
    INF("COMP RESUME!");
    //ecore_evas_manual_render_set(e_comp->ee, EINA_FALSE);
    ecore_evas_show(e_comp->ee);
@@ -356,8 +370,6 @@ _e_comp_nocomp_end(void)
           {
              if (!e_config->allow_above_fullscreen)
                layer = E_LAYER_CLIENT_FULLSCREEN;
-             else if (e_config->mode.presentation)
-               layer = E_LAYER_CLIENT_TOP;
           }
         evas_object_layer_set(e_comp->nocomp_ec->frame, layer);
         e_comp->nocomp_ec->fullscreen = fs;
@@ -365,6 +377,263 @@ _e_comp_nocomp_end(void)
    E_FREE_FUNC(e_comp->nocomp_ec, e_object_unref);
 }
 
+static double
+_e_comp_frame_event_fps_calc(int info)
+{
+   int i, count = 0, iind, start, end, end2;
+   Eina_Bool first = EINA_FALSE;
+   double t0 = 0.0, dt;
+
+   start = e_comp->frame_event_now - 1 + E_COMP_FRAME_EVENT_COUNT;
+   end = e_comp->frame_event_now + E_COMP_FRAME_EVENT_COUNT ;
+   end2 = e_comp->frame_event_now + E_COMP_FRAME_EVENT_COUNT + 1;
+   for (i = start;;)
+     {
+        iind = i % E_COMP_FRAME_EVENT_COUNT;
+        i--;
+        if (iind == (end % E_COMP_FRAME_EVENT_COUNT)) break;
+        if (e_comp->frame_events[iind].info[0] != info) continue;
+        count++;
+        if (!first)
+          {
+             t0 =  e_comp->frame_events[iind].t;
+             first = EINA_TRUE;
+          }
+        else
+          {
+             dt = t0 - e_comp->frame_events[iind].t;
+             if ((dt > 0.5) || (iind == (end2 % E_COMP_FRAME_EVENT_COUNT)))
+               {
+                  if (dt > 0.0) return ((double)(count - 1)) / dt;
+               }
+          }
+     }
+   return 0.0;
+}
+
+E_API void
+e_comp_frame_event_add(int info[4], double t)
+{
+   e_comp->frame_events[e_comp->frame_event_now].info[0] = info[0];
+   e_comp->frame_events[e_comp->frame_event_now].info[1] = info[1];
+   e_comp->frame_events[e_comp->frame_event_now].info[2] = info[2];
+   e_comp->frame_events[e_comp->frame_event_now].info[3] = info[3];
+   e_comp->frame_events[e_comp->frame_event_now].t = t;
+   e_comp->frame_event_now++;
+   if (e_comp->frame_event_now >= E_COMP_FRAME_EVENT_COUNT)
+     e_comp->frame_event_now = 0;
+}
+
+E_API void
+e_comp_client_frame_add(Evas_Object *obj EINA_UNUSED)
+{
+   if (conf->fps_show)
+     {
+        int info[4] = { E_COMP_FRAME_EVENT_CLIENT_DAMAGE, 0, 0, 0 };
+        e_comp_frame_event_add(info, ecore_time_get());
+     }
+}
+
+static inline void
+_e_comp_fps_draw_point(unsigned int *pix, int stride, int w, int y, int col, int x)
+{
+   unsigned int *pixrow = pix + (y * (stride / 4));
+   // out of bounds abort draw
+   if (x <  0) return;
+   if (x >= w) return;
+   pixrow[x] = col;
+}
+
+/* for future
+static inline void
+_e_comp_fps_draw_span(unsigned int *pix, int stride, int w, int y, int col, int x1, int x2)
+{
+   unsigned int *pixrow = pix + (y * (stride / 4));
+   int x;
+
+   if (x2 < x1) // swap so x2 >= x1
+     {
+        x = x2;
+        x2 = x1;
+        x1 = x;
+     }
+   // out of bounds abort draw
+   if (x2 < 0) return;
+   if (x1 >= w) return;
+   // clip to span area
+   if (x1 < 0) x1 = 0;
+   if (x2 >= w) x2 = w - 1;
+   for (x = x1; x <= x2; x++)
+     pixrow[x] = col;
+}
+*/
+
+E_API void
+e_comp_fps_update(void)
+{
+   char buf[128];
+   double fps, comp_fps;
+   Evas_Coord x = 0, y = 0, w = 0, h = 0;
+   Evas_Coord gx = 0, gy = 0, gw = 0, gh = 0;
+   Evas_Coord bx = 0, by = 0, bw = 0, bh = 0;
+   E_Zone *z;
+
+   e_comp->frameskip++;
+   if (e_comp->frameskip >= 20)
+     {
+        unsigned int *pix;
+        int i, pixw, pixh, pixsz, pixstride, px, pixscale, info0, start, end, iind;
+        double t;
+
+        t = ecore_time_get();
+        e_comp->frameskip = 0;
+
+        pixh = 12;
+        pixsz = 2;
+
+        fps = _e_comp_frame_event_fps_calc(E_COMP_FRAME_EVENT_HANDLE_DAMAGE);
+        comp_fps = _e_comp_frame_event_fps_calc(E_COMP_FRAME_EVENT_RENDER_END);
+
+        snprintf(buf, sizeof(buf), "FPS: (in) %1.1f (out) %1.1f", fps, comp_fps);
+        evas_object_text_text_set(e_comp->canvas->fps_fg, buf);
+
+        evas_object_geometry_get(e_comp->canvas->fps_fg, NULL, NULL, &w, &h);
+        w += 16;
+        h += 16;
+        z = e_zone_current_get();
+        pixscale = 1000;
+        if (z)
+          {
+             if (z->w >= 2048) pixw = 2000;
+             else if (z->w >= 1600) pixw = 1500;
+             else if (z->w >= 1024) pixw = 1000;
+             else if (z->w >= 512) pixw = 500;
+             else if (z->w >= 256) pixw = 250;
+             else pixw = 125;
+
+             switch (conf->fps_corner)
+               {
+                case 3: // bottom-right
+                  x = z->x + z->w - w;
+                  y = z->y + z->h - h;
+
+                  gw = pixw;
+                  gh = pixh * pixsz;
+                  gx = x + w - gw;
+                  gy = y - gh - 8;
+
+                  bw = (gw > bw) ? gw : bw;
+                  bh = h + gh + 16;
+                  bx = (x > gx) ? gx: x;
+                  by = gy - 8;
+                  break;
+                case 2: // bottom-left
+                  x = z->x;
+                  y = z->y + z->h - h;
+
+                  gw = pixw;
+                  gh = pixh * pixsz;
+                  gx = x;
+                  gy = y - gh - 8;
+
+                  bw = (gw > bw) ? gw : bw;
+                  bh = h + gh + 16;
+                  bx = x;
+                  by = gy - 8;
+                  break;
+                case 1: // top-right
+                  x = z->x + z->w - w;
+                  y = z->y;
+
+                  gw = pixw;
+                  gh = pixh * pixsz;
+                  gx = x + w - gw;
+                  gy = y + h;
+
+                  bw = (gw > bw) ? gw : bw;
+                  bh = h + gh + 16;
+                  bx = (x > gx) ? gx: x;
+                  by = y;
+                  break;
+                case 0: // top-left
+                default:
+                  x = z->x;
+                  y = z->y;
+
+                  gw = pixw;
+                  gh = pixh * pixsz;
+                  gx = x;
+                  gy = y + h;
+
+                  bw = (gw > bw) ? gw : bw;
+                  bh = h + gh + 16;
+                  bx = x;
+                  by = y;
+                  break;
+               }
+          }
+        else pixw = 500;
+
+        evas_object_image_size_set(e_comp->canvas->fps_gr, pixw, pixh);
+        evas_object_image_alpha_set(e_comp->canvas->fps_gr, EINA_TRUE);
+        pixstride = evas_object_image_stride_get(e_comp->canvas->fps_gr);
+        pix = evas_object_image_data_get(e_comp->canvas->fps_gr, EINA_TRUE);
+        if (pix)
+          {
+             memset(pix, 0, pixstride * pixh);
+
+             // go backwards from newest to oldest
+             start = e_comp->frame_event_now + E_COMP_FRAME_EVENT_COUNT - 1;
+             end = e_comp->frame_event_now + E_COMP_FRAME_EVENT_COUNT ;
+             for (i = start;;)
+               {
+                  iind = i % E_COMP_FRAME_EVENT_COUNT;
+                  i--;
+                  if (iind == (end % E_COMP_FRAME_EVENT_COUNT)) break;
+
+                  info0 = e_comp->frame_events[iind].info[0];
+                  px = (t - e_comp->frame_events[iind].t) * pixscale;
+                  px = pixw - px - 1;
+                  if (px < 0) break;
+                  if (info0 == E_COMP_FRAME_EVENT_RENDER2_END)
+                    _e_comp_fps_draw_point(pix, pixstride, pixw, 0, 0xffffffff, px);
+                  else if (info0 == E_COMP_FRAME_EVENT_RENDER2_BEGIN)
+                    _e_comp_fps_draw_point(pix, pixstride, pixw, 1, 0xffffee88, px);
+                  else if (info0 == E_COMP_FRAME_EVENT_RENDER_END)
+                    _e_comp_fps_draw_point(pix, pixstride, pixw, 2, 0xffff9944, px);
+                  else if (info0 == E_COMP_FRAME_EVENT_RENDER_BEGIN)
+                    _e_comp_fps_draw_point(pix, pixstride, pixw, 3, 0xffff4433, px);
+                  else if (info0 == E_COMP_FRAME_EVENT_IDLE_ENTER)
+                    _e_comp_fps_draw_point(pix, pixstride, pixw, 4, 0xffff88ff, px);
+                  else if (info0 == E_COMP_FRAME_EVENT_HANDLE_DAMAGE)
+                    _e_comp_fps_draw_point(pix, pixstride, pixw, 5, 0xff44ff22, px);
+                  else if (info0 == E_COMP_FRAME_EVENT_CLIENT_DAMAGE)
+                    _e_comp_fps_draw_point(pix, pixstride, pixw, 6, 0xff4466ff, px);
+                  else if (info0 == E_COMP_FRAME_EVENT_IDLE_EXIT)
+                    _e_comp_fps_draw_point(pix, pixstride, pixw, 7, 0xff994499, px);
+               }
+             for (t = 0.0; t < 10.0; t += (1.0 / 60.0))
+               {
+                  px = t * pixscale;
+                  px = pixw - px - 1;
+                  if (px < 0) break;
+                  _e_comp_fps_draw_point(pix, pixstride, pixw, 10, 0xffffffff, px);
+                  _e_comp_fps_draw_point(pix, pixstride, pixw, 11, 0xffffffff, px - 1);
+                  _e_comp_fps_draw_point(pix, pixstride, pixw, 11, 0xffffffff, px);
+                  _e_comp_fps_draw_point(pix, pixstride, pixw, 11, 0xffffffff, px + 1);
+               }
+             evas_object_image_data_set(e_comp->canvas->fps_gr, pix);
+             evas_object_image_data_update_add(e_comp->canvas->fps_gr,
+                                               0, 0, pixw, pixh);
+          }
+
+        evas_object_color_set(e_comp->canvas->fps_bg, 0, 0, 0, 192);
+        evas_object_geometry_set(e_comp->canvas->fps_bg, bx, by, bw, bh);
+        evas_object_geometry_set(e_comp->canvas->fps_gr, gx, gy, gw, gh);
+        evas_object_move(e_comp->canvas->fps_fg, x + 8, y + 8);
+     }
+}
+
 static Eina_Bool
 _e_comp_cb_update(void)
 {
@@ -373,10 +642,6 @@ _e_comp_cb_update(void)
    //   static int doframeinfo = -1;
 
    if (!e_comp) return EINA_FALSE;
-   if (e_comp->update_job)
-     e_comp->update_job = NULL;
-   else
-     ecore_animator_freeze(e_comp->render_animator);
    DBG("UPDATE ALL");
    if (e_comp->nocomp) goto nocomp;
 //   if (conf->grab && (!e_comp->grabbed))
@@ -397,95 +662,10 @@ _e_comp_cb_update(void)
    _e_comp_fps_update();
    if (conf->fps_show)
      {
-        char buf[128];
-        double fps = 0.0, t, dt;
-        int i;
-        Evas_Coord x = 0, y = 0, w = 0, h = 0;
-        E_Zone *z;
-
-        t = ecore_loop_time_get();
-        if (conf->fps_average_range < 1)
-          conf->fps_average_range = 30;
-        else if (conf->fps_average_range > 120)
-          conf->fps_average_range = 120;
-        dt = t - e_comp->frametimes[conf->fps_average_range - 1];
-        if (dt > 0.0) fps = (double)conf->fps_average_range / dt;
-        else fps = 0.0;
-        if (fps > 0.0) snprintf(buf, sizeof(buf), "FPS: %1.1f", fps);
-        else snprintf(buf, sizeof(buf), "N/A");
-        for (i = 121; i >= 1; i--)
-          e_comp->frametimes[i] = e_comp->frametimes[i - 1];
-        e_comp->frametimes[0] = t;
-        e_comp->frameskip++;
-        if (e_comp->frameskip >= conf->fps_average_range)
-          {
-             e_comp->frameskip = 0;
-             evas_object_text_text_set(e_comp->canvas->fps_fg, buf);
-          }
-        evas_object_geometry_get(e_comp->canvas->fps_fg, NULL, NULL, &w, &h);
-        w += 8;
-        h += 8;
-        z = e_zone_current_get();
-        if (z)
-          {
-             switch (conf->fps_corner)
-               {
-                case 3: // bottom-right
-                  x = z->x + z->w - w;
-                  y = z->y + z->h - h;
-                  break;
-
-                case 2: // bottom-left
-                  x = z->x;
-                  y = z->y + z->h - h;
-                  break;
-
-                case 1: // top-right
-                  x = z->x + z->w - w;
-                  y = z->y;
-                  break;
-                default: // 0 // top-left
-                  x = z->x;
-                  y = z->y;
-                  break;
-               }
-          }
-        evas_object_move(e_comp->canvas->fps_bg, x, y);
-        evas_object_resize(e_comp->canvas->fps_bg, w, h);
-        evas_object_move(e_comp->canvas->fps_fg, x + 4, y + 4);
+        int info[4] = { E_COMP_FRAME_EVENT_HANDLE_DAMAGE, 0, 0, 0 };
+        e_comp_frame_event_add(info, ecore_loop_time_get());
+        e_comp_fps_update();
      }
-   if (conf->lock_fps)
-     {
-        DBG("MANUAL RENDER...");
-        //        if (!e_comp->nocomp) ecore_evas_manual_render(e_comp->ee);
-     }
-
-   if (e_comp->updates && (!e_comp->update_job))
-     ecore_animator_thaw(e_comp->render_animator);
-   /*
-      if (doframeinfo == -1)
-      {
-      doframeinfo = 0;
-      if (getenv("DFI")) doframeinfo = 1;
-      }
-      if (doframeinfo)
-      {
-      static double t0 = 0.0;
-      double td, t;
-
-      t = ecore_time_get();
-      td = t - t0;
-      if (td > 0.0)
-      {
-      int fps, i;
-
-      fps = 1.0 / td;
-      for (i = 0; i < fps; i+= 2) putchar('=');
-      printf(" : %3.3f", 1.0 / td);
-      }
-      t0 = t;
-      }
-    */
 nocomp:
    ec = _e_comp_fullscreen_check();
    if (ec)
@@ -504,7 +684,7 @@ nocomp:
                        if (!ec->visible) continue;
                        if (evas_object_data_get(ec->frame, "comp_skip")) continue;
                        if (e_object_is_del(E_OBJECT(ec)) || (!e_client_util_desk_visible(ec, e_desk_current_get(ec->zone)))) continue;
-                       if (ec->override || (e_config->allow_above_fullscreen && (!e_config->mode.presentation)))
+                       if (ec->override || (e_config->allow_above_fullscreen))
                          {
                             _e_comp_nocomp_end();
                             break;
@@ -530,15 +710,10 @@ static void
 _e_comp_cb_job(void *data EINA_UNUSED)
 {
    DBG("UPDATE ALL JOB...");
+   if (e_comp->update_job) e_comp->update_job = NULL;
    _e_comp_cb_update();
 }
 
-static Eina_Bool
-_e_comp_cb_animator(void *data EINA_UNUSED)
-{
-   return _e_comp_cb_update();
-}
-
 //////////////////////////////////////////////////////////////////////////
 
 
@@ -846,7 +1021,6 @@ _e_comp_free(E_Comp *c)
 
    ecore_evas_free(c->ee);
 
-   if (c->render_animator) ecore_animator_del(c->render_animator);
    if (c->update_job) ecore_job_del(c->update_job);
    if (c->nocomp_delay_timer) ecore_timer_del(c->nocomp_delay_timer);
    if (c->nocomp_override_timer) ecore_timer_del(c->nocomp_override_timer);
@@ -888,14 +1062,11 @@ _e_comp_screensaver_on(void *data EINA_UNUSED, int type EINA_UNUSED, void *event
    Eina_List *l;
    E_Zone *zone;
 
-   ecore_frametime = ecore_animator_frametime_get();
    if (e_comp->saver) return ECORE_CALLBACK_RENEW;
    e_comp_override_add();
    e_comp->saver = EINA_TRUE;
    // XXX: this is not quite right - need to wait for signals from theme
    // before freezing render animator
-   if (e_comp->render_animator)
-     ecore_animator_freeze(e_comp->render_animator);
    EINA_LIST_FOREACH(e_comp->zones, l, zone)
      {
         e_zone_fade_handle(zone, 1, 3.0);
@@ -913,7 +1084,6 @@ _e_comp_screensaver_off(void *data EINA_UNUSED, int type EINA_UNUSED, void *even
    E_Zone *zone;
    E_Client *ec;
 
-   ecore_animator_frametime_set(ecore_frametime);
    if (!e_comp->saver) return ECORE_CALLBACK_RENEW;
    e_comp_override_del();
    e_comp->saver = EINA_FALSE;
@@ -1018,7 +1188,6 @@ e_comp_init(void)
 {
    _e_comp_log_dom = eina_log_domain_register("e_comp", EINA_COLOR_YELLOW);
 
-   ecore_frametime = ecore_animator_frametime_get();
    shape_debug = !!getenv("E_SHAPE_DEBUG");
 
    E_EVENT_COMPOSITOR_UPDATE = ecore_event_type_new();
@@ -1056,6 +1225,11 @@ e_comp_init(void)
         conf->version = E_COMP_VERSION;
      }
 
+   if (conf->no_dither)
+     e_util_env_set("EVAS_GL_RENDER_DISABLE_DITHER", "1");
+   else
+     e_util_env_set("EVAS_GL_RENDER_DISABLE_DITHER", NULL);
+
    {
       E_Action *act;
 
@@ -1406,9 +1580,6 @@ e_comp_new(void)
    e_comp = E_OBJECT_ALLOC(E_Comp, E_COMP_TYPE, _e_comp_free);
    if (!e_comp) return NULL;
    e_comp->canvas = E_NEW(E_Comp_Canvas, 1);
-
-   e_comp->render_animator = ecore_animator_add(_e_comp_cb_animator, NULL);
-   ecore_animator_freeze(e_comp->render_animator);
    return e_comp;
 }
 
@@ -1453,20 +1624,13 @@ e_comp_shutdown(void)
 E_API void
 e_comp_render_queue(void)
 {
-   if (conf->lock_fps)
+   if (e_comp->update_job)
      {
-        ecore_animator_thaw(e_comp->render_animator);
-     }
-   else
-     {
-        if (e_comp->update_job)
-          {
-             DBG("UPDATE JOB DEL...");
-             E_FREE_FUNC(e_comp->update_job, ecore_job_del);
-          }
-        DBG("UPDATE JOB ADD...");
-        e_comp->update_job = ecore_job_add(_e_comp_cb_job, e_comp);
+        DBG("UPDATE JOB DEL...");
+        E_FREE_FUNC(e_comp->update_job, ecore_job_del);
      }
+   DBG("UPDATE JOB ADD...");
+   e_comp->update_job = ecore_job_add(_e_comp_cb_job, e_comp);
 }
 
 E_API void
@@ -1601,34 +1765,6 @@ e_comp_override_add(void)
    if ((e_comp->nocomp_override > 0) && (e_comp->nocomp)) _e_comp_nocomp_end();
 }
 
-#if 0
-FIXME
-E_API void
-e_comp_block_window_add(void)
-{
-   e_comp->block_count++;
-   if (e_comp->block_win) return;
-   e_comp->block_win = ecore_x_window_new(e_comp->root, 0, 0, e_comp->w, e_comp->h);
-   INF("BLOCK WIN: %x", e_comp->block_win);
-   ecore_x_window_background_color_set(e_comp->block_win, 0, 0, 0);
-   e_comp_ignore_win_add(e_comp->block_win);
-   ecore_x_window_configure(e_comp->block_win,
-     ECORE_X_WINDOW_CONFIGURE_MASK_SIBLING | ECORE_X_WINDOW_CONFIGURE_MASK_STACK_MODE,
-     0, 0, 0, 0, 0, ((E_Comp_Win*)e_comp->wins)->win, ECORE_X_WINDOW_STACK_ABOVE);
-   ecore_x_window_show(e_comp->block_win);
-}
-
-E_API void
-e_comp_block_window_del(void)
-{
-   if (!e_comp->block_count) return;
-   e_comp->block_count--;
-   if (e_comp->block_count) return;
-   if (e_comp->block_win) ecore_x_window_free(e_comp->block_win);
-   e_comp->block_win = 0;
-}
-#endif
-
 E_API E_Comp *
 e_comp_find_by_window(Ecore_Window win)
 {
