@@ -35,8 +35,8 @@ typedef enum _E_Layer
    E_LAYER_CLIENT_EDGE = 300,
    E_LAYER_CLIENT_FULLSCREEN = 350,
    E_LAYER_CLIENT_EDGE_FULLSCREEN = 400,
-   E_LAYER_CLIENT_POPUP = 450,
-   E_LAYER_CLIENT_TOP = 500,
+   E_LAYER_CLIENT_TOP = 450,
+   E_LAYER_CLIENT_POPUP = 500,
    E_LAYER_CLIENT_DRAG = 550,
    E_LAYER_CLIENT_PRIO = 600,
    E_LAYER_POPUP = 999, // popups
@@ -87,9 +87,27 @@ typedef struct E_Comp_Canvas
    Evas_Object *resize_object; //object to monitor for comp canvas resizes
    Evas_Object *fps_bg;
    Evas_Object *fps_fg;
+   Evas_Object *fps_gr;
    Evas_Object *gadget_site; //desktop gadget site
 } E_Comp_Canvas;
 
+#define E_COMP_FRAME_EVENT_COUNT 4096
+
+#define E_COMP_FRAME_EVENT_RENDER_BEGIN   1
+#define E_COMP_FRAME_EVENT_RENDER_END     2
+#define E_COMP_FRAME_EVENT_RENDER2_BEGIN  3
+#define E_COMP_FRAME_EVENT_RENDER2_END    4
+#define E_COMP_FRAME_EVENT_CLIENT_DAMAGE  5
+#define E_COMP_FRAME_EVENT_HANDLE_DAMAGE  6
+#define E_COMP_FRAME_EVENT_IDLE_EXIT      7
+#define E_COMP_FRAME_EVENT_IDLE_ENTER     8
+
+typedef struct E_Comp_Frame_Event
+{
+   int info[4];
+   double t;
+} E_Comp_Event;
+
 struct _E_Comp
 {
    E_Object e_obj_inherit;
@@ -145,7 +163,8 @@ struct _E_Comp
    Ecore_Timer    *nocomp_delay_timer; //delay before activating nocomp in x11
    Ecore_Timer    *nocomp_override_timer; //delay before overriding nocomp in x11
    int             animating; //number of animating comp objects
-   double          frametimes[122]; //used for calculating fps
+   E_Comp_Event    frame_events[E_COMP_FRAME_EVENT_COUNT]; // history of frame events for debug
+   int             frame_event_now;
    int             frameskip;
 
    int             nocomp_override; //number of times nocomp override has been requested
@@ -171,6 +190,8 @@ struct _E_Comp
 
    Eina_Bool       rendering E_BITFIELD; // we've received a pre-render callback but no post-render yet.
    Eina_Bool       updating E_BITFIELD; // running client updates
+
+   Eina_Bool       frozen E_BITFIELD; // frozen - when power save freeze mode is on
 };
 
 
@@ -222,6 +243,8 @@ E_API Eina_Bool e_comp_grab_input(Eina_Bool mouse, Eina_Bool kbd);
 E_API void e_comp_ungrab_input(Eina_Bool mouse, Eina_Bool kbd);
 E_API void e_comp_gl_set(Eina_Bool set);
 E_API Eina_Bool e_comp_gl_get(void);
+E_API void e_comp_frame_event_add(int info[4], double t);
+E_API void e_comp_client_frame_add(Evas_Object *obj);
 
 E_API void e_comp_button_bindings_grab_all(void);
 E_API void e_comp_button_bindings_ungrab_all(void);
