@@ -39,23 +39,24 @@ _e_comp_canvas_cb_first_frame(void *data EINA_UNUSED, Evas *e, void *event_info
    evas_event_callback_del_full(e, EVAS_CALLBACK_RENDER_POST, _e_comp_canvas_cb_first_frame, NULL);
 }
 
+static Ecore_Job *_e_comp_update_job = NULL;
+
 static void
-_e_comp_canvas_render_post(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *event_info EINA_UNUSED)
+_e_comp_canvas_fps_update_job(void *data EINA_UNUSED)
 {
    E_Comp_Config *conf = e_comp_config_get();
+   _e_comp_update_job = NULL;
+   if (conf->fps_show) e_comp_fps_update();
+}
+
+static Ecore_Job *_e_comp_post_render_job = NULL;
+
+static void
+_e_comp_canvas_render_post_job(void *data EINA_UNUSED)
+{
    E_Client *ec;
-   //Evas_Event_Render_Post *ev = event_info;
-   //Eina_List *l;
-   //Eina_Rectangle *r;
-
-   //if (ev)
-     //{
-        //EINA_LIST_FOREACH(ev->updated_area, l, r)
-          //INF("POST RENDER: %d,%d %dx%d", r->x, r->y, r->w, r->h);
-     //}
-
-   e_comp->rendering = EINA_FALSE;
 
+   _e_comp_post_render_job = NULL;
    EINA_LIST_FREE(e_comp->post_updates, ec)
      {
         //INF("POST %p", ec);
@@ -66,6 +67,69 @@ _e_comp_canvas_render_post(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *ev
         UNREFD(ec, 111);
         e_object_unref(E_OBJECT(ec));
      }
+}
+
+static void
+_e_comp_canvas_render_track_pre(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   E_Comp_Config *conf = e_comp_config_get();
+   if (conf->fps_show)
+     {
+        int info[4] = { E_COMP_FRAME_EVENT_RENDER_BEGIN, 0, 0, 0 };
+        e_comp_frame_event_add(info, ecore_time_get());
+     }
+}
+
+static void
+_e_comp_canvas_render_track_post(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   E_Comp_Config *conf = e_comp_config_get();
+   if (conf->fps_show)
+     {
+        int info[4] = { E_COMP_FRAME_EVENT_RENDER_END, 0, 0, 0 };
+        e_comp_frame_event_add(info, ecore_time_get());
+     }
+}
+
+static void
+_e_comp_canvas_render_track_flush_pre(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   E_Comp_Config *conf = e_comp_config_get();
+   if (conf->fps_show)
+     {
+        int info[4] = { E_COMP_FRAME_EVENT_RENDER2_BEGIN, 0, 0, 0 };
+        e_comp_frame_event_add(info, ecore_time_get());
+     }
+}
+
+static void
+_e_comp_canvas_render_track_flush_post(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   E_Comp_Config *conf = e_comp_config_get();
+   if (conf->fps_show)
+     {
+        int info[4] = { E_COMP_FRAME_EVENT_RENDER2_END, 0, 0, 0 };
+        e_comp_frame_event_add(info, ecore_time_get());
+     }
+}
+
+static void
+_e_comp_canvas_render_post(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   E_Comp_Config *conf = e_comp_config_get();
+
+   // NOTE: don't modify objects here as we may lose updates - defer...
+   if (conf->fps_show)
+     {
+        if (_e_comp_update_job) ecore_job_del(_e_comp_update_job);
+        ecore_job_add(_e_comp_canvas_fps_update_job, NULL);
+     }
+
+   e_comp->rendering = EINA_FALSE;
+
+   if (_e_comp_post_render_job) ecore_job_del(_e_comp_post_render_job);
+   _e_comp_post_render_job = ecore_job_add(_e_comp_canvas_render_post_job, NULL);
+
    if (conf->grab && e_comp->grabbed)
      {
         if (e_comp->grab_cb) e_comp->grab_cb();
@@ -264,8 +328,50 @@ _e_comp_cb_mouse_move(void *d EINA_UNUSED, int t EINA_UNUSED, Ecore_Event_Mouse_
 ////////////////////////////////////
 
 static Eina_Bool
-_e_comp_cb_zone_change()
+_e_comp_cb_zone_change(void *d EINA_UNUSED, int type, void *event)
 {
+   if (type == E_EVENT_ZONE_ADD)
+     {
+        E_Event_Zone_Add *ev = event;
+        E_Client *ec;
+
+        E_CLIENT_FOREACH(ec)
+          {
+             if (ec->restore_zone_id)
+               {
+                  E_Zone *restore_zone =
+                    e_zone_for_id_get(ec->restore_zone_id);
+                  if ((restore_zone) && (ev->zone) &&
+                      (restore_zone == ev->zone))
+                    {
+                       int dx, dy;
+                       E_Desk *desk;
+
+                       dx = dy = -1;
+                       if (ec->desk)
+                         {
+                            dx = ec->desk->x;
+                            dy = ec->desk->y;
+                         }
+                       e_client_zone_set(ec, restore_zone);
+                       if ((dx >= 0) && (dy >= 0))
+                         {
+                            desk = e_desk_at_xy_get(restore_zone, dx, dy);
+                            if (desk)
+                              {
+                                 e_client_desk_set(ec, desk);
+                                 if (desk == e_desk_current_get(restore_zone))
+                                   {
+                                      if ((ec->visible) && (!ec->hidden) && (!ec->iconic))
+                                        evas_object_show(ec->frame);
+                                   }
+                              }
+                         }
+                       e_client_res_change_geometry_restore(ec);
+                    }
+               }
+          }
+     }
    e_comp_canvas_update();
    return ECORE_CALLBACK_PASS_ON;
 }
@@ -275,7 +381,6 @@ _e_comp_cb_zone_change()
 static Eina_Bool
 _e_comp_cb_screensaver_active_delay(void *data EINA_UNUSED)
 {
-   ecore_animator_frametime_set(10.0);
    if (!e_comp->nocomp)
      ecore_evas_manual_render_set(e_comp->ee, EINA_TRUE);
    timer_post_screensaver_on = NULL;
@@ -362,6 +467,12 @@ _e_comp_canvas_prerender(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *even
    Eina_List *l;
    E_Comp_Config *conf = e_comp_config_get();
 
+   if (conf->fps_show)
+     {
+        int info[4] = { E_COMP_FRAME_EVENT_RENDER_BEGIN, 0, 0, 0 };
+        e_comp_frame_event_add(info, ecore_time_get());
+     }
+
    e_comp->rendering = EINA_TRUE;
 
    if (conf->grab && (!e_comp->grabbed))
@@ -374,6 +485,30 @@ _e_comp_canvas_prerender(void *data EINA_UNUSED, Evas *e EINA_UNUSED, void *even
      cb();
 }
 
+static Eina_Bool
+_e_comp_canvas_cb_idle_enterer(void *data EINA_UNUSED)
+{
+   E_Comp_Config *conf = e_comp_config_get();
+   if (conf->fps_show)
+     {
+        int info[4] = { E_COMP_FRAME_EVENT_IDLE_ENTER, 0, 0, 0 };
+        e_comp_frame_event_add(info, ecore_time_get());
+     }
+   return EINA_TRUE;
+}
+
+static Eina_Bool
+_e_comp_canvas_cb_idle_exiter(void *data EINA_UNUSED)
+{
+   E_Comp_Config *conf = e_comp_config_get();
+   if (conf->fps_show)
+   {
+      int info[4] = { E_COMP_FRAME_EVENT_IDLE_EXIT, 0, 0, 0 };
+      e_comp_frame_event_add(info, ecore_time_get());
+   }
+   return EINA_TRUE;
+}
+
 E_API Eina_Bool
 e_comp_canvas_init(int w, int h)
 {
@@ -397,6 +532,17 @@ e_comp_canvas_init(int w, int h)
 
    e_comp->evas = ecore_evas_get(e_comp->ee);
 
+   static int inited = 0;
+   if (!inited)
+     {
+        inited = 1;
+        ecore_idle_enterer_before_add(_e_comp_canvas_cb_idle_enterer, NULL);
+        ecore_idle_exiter_add(_e_comp_canvas_cb_idle_exiter, NULL);
+     }
+   evas_event_callback_add(e_comp->evas, EVAS_CALLBACK_RENDER_PRE, _e_comp_canvas_render_track_pre, NULL);
+   evas_event_callback_add(e_comp->evas, EVAS_CALLBACK_RENDER_POST, _e_comp_canvas_render_track_post, NULL);
+   evas_event_callback_add(e_comp->evas, EVAS_CALLBACK_RENDER_FLUSH_PRE, _e_comp_canvas_render_track_flush_pre, NULL);
+   evas_event_callback_add(e_comp->evas, EVAS_CALLBACK_RENDER_FLUSH_POST, _e_comp_canvas_render_track_flush_post, NULL);
    if (e_first_frame)
      evas_event_callback_add(e_comp->evas, EVAS_CALLBACK_RENDER_POST, _e_comp_canvas_cb_first_frame, NULL);
    o = evas_object_rectangle_add(e_comp->evas);
@@ -465,6 +611,7 @@ e_comp_canvas_clear(void)
 
    E_FREE_FUNC(e_comp->canvas->fps_fg, evas_object_del);
    E_FREE_FUNC(e_comp->canvas->fps_bg, evas_object_del);
+   E_FREE_FUNC(e_comp->canvas->fps_gr, evas_object_del);
    E_FREE_FUNC(e_comp->autoclose.rect, evas_object_del);
    E_FREE_FUNC(e_comp->shape_job, ecore_job_del);
    E_FREE_FUNC(e_comp->pointer, e_object_del);
@@ -687,12 +834,37 @@ e_comp_canvas_update(void)
                     {
                        if (ec->zone == zone)
                          {
+                            const char *tmp;
+
+                            if (ec->restore_zone_id)
+                              tmp = eina_stringshare_add(ec->restore_zone_id);
+                            else
+                              tmp = NULL;
                             if (spare_zone)
-                              e_client_zone_set(ec, spare_zone);
+                              {
+                                 int dx, dy;
+                                 E_Desk *desk;
+
+                                 dx = dy = -1;
+                                 if (ec->desk)
+                                   {
+                                      dx = ec->desk->x;
+                                      dy = ec->desk->y;
+                                   }
+                                 e_client_res_change_geometry_save(ec);
+                                 e_client_zone_set(ec, spare_zone);
+                                 if ((dx >= 0) && (dy >= 0))
+                                   {
+                                      desk = e_desk_at_xy_get(spare_zone, dx, dy);
+                                      if (desk) e_client_desk_set(ec, desk);
+                                   }
+                              }
                             else
                               printf("EEEK! should not be here - but no\n"
                                      "spare zones exist to move this\n"
                                      "window to!!! help!\n");
+                            eina_stringshare_replace(&(ec->restore_zone_id), tmp);
+                            if (tmp) eina_stringshare_del(tmp);
                          }
                     }
                   e_object_del(E_OBJECT(zone));
