@@ -433,17 +433,41 @@ _mode_screen_find(Ecore_X_Window root, E_Randr2_Screen *s, Ecore_X_Randr_Output
    return mode;
 }
 
+static E_Dialog *screen_error_dialog = NULL;
+
+static void
+_screen_error_remove_cb(void *data EINA_UNUSED, E_Dialog *dia EINA_UNUSED)
+{
+   screen_error_dialog = NULL;
+   e_object_del(E_OBJECT(dia));
+}
+
 static Eina_Bool
 _cb_no_outputs_timer(void *data EINA_UNUSED)
 {
-   e_util_dialog_show(_("Screen setup Error"),
-                      _("You seem to have no screens configured to<br>"
-                        "be on given the outputs you have. This should<br>"
-                        "be fixed by going to:<br>"
-                        "<br>"
-                        "<b>Settings -> Screen -> Screen Setup</><br>"
-                        "<br>"
-                        "And configure at least one screen to be on."));
+   if (!screen_error_dialog)
+     {
+        screen_error_dialog = e_dialog_new(NULL, "E", "_error_dialog");
+
+        e_dialog_title_set(screen_error_dialog, _("Screen setup Error"));
+        e_dialog_text_set(screen_error_dialog, _("You seem to have no screens configured to<br>"
+                                 "be on given the outputs you have. This should<br>"
+                                 "be fixed by going to:<br>"
+                                 "<br>"
+                                 "<b>Settings -> Screen -> Screen Setup</><br>"
+                                 "<br>"
+                                 "And configure at least one screen to be on."));
+       e_dialog_icon_set(screen_error_dialog, "dialog-error", 64);
+       e_dialog_button_add(screen_error_dialog, _("OK"), NULL, _screen_error_remove_cb, NULL);
+       e_dialog_button_focus_num(screen_error_dialog, 0);
+       elm_win_center(screen_error_dialog->win, 1, 1);
+       e_dialog_show(screen_error_dialog);
+     }
+   else
+     {
+       elm_win_raise(screen_error_dialog->win);
+     }
+
    return EINA_FALSE;
 }
 
@@ -708,6 +732,7 @@ e_comp_x_randr_create(void)
 
    printf("RRR: ................. info get!\n");
    // do this to force xrandr to update its content
+   ecore_x_randr_screen_refresh(root);
    ecore_x_randr_config_timestamp_get(root);
    ecore_x_randr_screen_size_range_get(root, NULL, NULL, NULL, NULL);
    ecore_x_sync();
@@ -724,6 +749,7 @@ e_comp_x_randr_create(void)
         E_Randr2_Screen *s = calloc(1, sizeof(E_Randr2_Screen));
         if (!s) continue;
 
+        printf("RRR: NEW SCREEN ...\n");
         s->info.name = _output_name_get(root, outputs[i]);
         printf("RRR: .... out %s\n", s->info.name);
         if (!s->info.name)
@@ -790,6 +816,7 @@ e_comp_x_randr_create(void)
                             m->refresh =
                             (double)minfo->dotClock /
                             (double)(minfo->hTotal * minfo->vTotal);
+                            m->flags = minfo->modeFlags;
                             if (j == (modes_pref - 1))
                               m->preferred = EINA_TRUE;
                             s->info.modes = eina_list_append(s->info.modes, m);
