@@ -79,6 +79,7 @@ _e_deskmirror_visible_get(E_Smart_Data *sd, Mirror *m)
          */
         if (m->sd->handlers && m->ec->iconic)
           visible = evas_object_visible_get(m->ec->frame);
+        if (m->ec->iconic) visible = EINA_FALSE;
         if (visible)
           {
              visible = (sd->desk == m->ec->desk) || (m->ec->sticky && (!m->ec->hidden));
@@ -282,6 +283,7 @@ _mirror_client_theme_setup(Mirror_Border *mb, Evas_Object *o)
 {
    char buf[4096];
 
+   if ((!mb) || (!mb->m) || (!mb->m->ec)) return;
    if (e_comp_object_frame_exists(mb->m->ec->frame))
      snprintf(buf, sizeof(buf), "e/deskmirror/frame/%s", mb->m->ec->border.name);
    else
@@ -989,11 +991,15 @@ e_deskmirror_util_wins_print(Evas_Object *obj)
 E_API void
 e_deskmirror_update_force(Evas_Object *obj)
 {
+   Mirror *m;
+
    API_ENTRY(obj);
 
    e_layout_freeze(sd->layout);
    sd->force = EINA_TRUE;
    _e_deskmirror_smart_hide(obj);
+   EINA_INLIST_FOREACH(sd->mirrors, m)
+     _e_deskmirror_mirror_reconfigure(m);
    _e_deskmirror_smart_reconfigure(sd);
    _e_deskmirror_smart_show(obj);
    sd->force = EINA_FALSE;
