@@ -315,12 +315,17 @@ e_hints_client_list_set(void)
      {
         E_Client *ec;
         const Eina_List *ll;
+        const char *name;
 
         clients = calloc(e_clients_count(), sizeof(Ecore_X_Window));
         EINA_LIST_FOREACH(e_comp->clients, ll, ec)
           {
              if (e_pixmap_type_get(ec->pixmap) != E_PIXMAP_TYPE_X) continue;
-             clients[i++] = e_client_util_win_get(ec);
+             name = evas_object_name_get(ec->frame);
+             if (!((name) && (!strcmp(name, "layer_obj"))))
+               {
+                  clients[i++] = e_client_util_win_get(ec);
+               }
           }
      }
    ecore_x_netwm_client_list_set(e_comp->root, clients, i);
@@ -335,12 +340,13 @@ e_hints_client_stacking_set(void)
 {
 #ifdef HAVE_WAYLAND_ONLY
 #else
-   unsigned int c, i = 0, non_x = 0;
+   unsigned int c, i = 0, non_x = 0, count = 0;
    Ecore_X_Window *clients = NULL;
+   const char *name;
 
 //#define CLIENT_STACK_DEBUG
    /* Get client count */
-   c = e_clients_count();
+   c = 20 + (e_clients_count() * 2);
    if (c)
      {
         E_Client *ec;
@@ -355,36 +361,28 @@ e_hints_client_stacking_set(void)
                   non_x++;
                   continue;
                }
-             clients[i++] = e_client_util_win_get(ec);
+             name = evas_object_name_get(ec->frame);
+             if (!((name) && (!strcmp(name, "layer_obj"))))
+               {
+                  clients[i++] = e_client_util_win_get(ec);
+                  count++;
+               }
 #ifdef CLIENT_STACK_DEBUG
              ll = eina_list_append(ll, ec);
 #endif
              if (i > c)
                {
-                  CRI("Window list size greater than window count.");
+                  ERR("Window list size greater than window count.");
                   break;
                }
           }
-
-        if (i < c - non_x)
-          {
-#ifdef CLIENT_STACK_DEBUG
-             Eina_List *lll = eina_list_clone(e_comp->clients);
-
-             EINA_LIST_FREE(ll, ec)
-               lll = eina_list_remove(lll, ec);
-             EINA_LIST_FREE(lll, ec)
-               WRN("Missing %p: %snew client", ec, ec->new_client ? "" : "not ");
-#endif
-             CRI("Window list size less than window count.");
-          }
      }
    /* XXX: it should be "more correct" to be setting the stacking atom as "windows per root"
     * since any apps using it are probably not going to want windows from other screens
     * to be returned in the list
     */
    if (i <= c)
-     ecore_x_netwm_client_list_stacking_set(e_comp->root, clients, c);
+     ecore_x_netwm_client_list_stacking_set(e_comp->root, clients, count);
    free(clients);
 #endif
 }
@@ -1014,7 +1012,7 @@ e_hints_window_state_get(E_Client *ec)
 #ifdef HAVE_WAYLAND_ONLY
    (void)ec;
 #else
-   unsigned int i, num;
+   unsigned int i, num, was_here = 0;
    Ecore_X_Window_State *state;
 
    if (!e_client_has_xwindow(ec)) return;
@@ -1029,6 +1027,17 @@ e_hints_window_state_get(E_Client *ec)
    ec->netwm.state.fullscreen = 0;
    ec->netwm.state.stacking = 0;
 
+   if (ecore_x_window_prop_card32_get(e_client_util_win_get(ec),
+                                      E_ATOM_E_WAS_HERE,
+                                      &was_here, 1) != 1)
+     {
+        unsigned int val = 1;
+
+        was_here = 0;
+        ecore_x_window_prop_card32_set(e_client_util_win_get(ec),
+                                       E_ATOM_E_WAS_HERE,
+                                       &val, 1);
+     }
    ecore_x_netwm_window_state_get(e_client_util_win_get(ec), &state, &num);
    if (state)
      {
@@ -1069,7 +1078,8 @@ e_hints_window_state_get(E_Client *ec)
                   break;
 
                 case ECORE_X_WINDOW_STATE_HIDDEN:
-                  ec->netwm.state.hidden = 1;
+                  if (!was_here)
+                    ec->netwm.state.hidden = 1;
                   break;
 
                 case ECORE_X_WINDOW_STATE_FULLSCREEN:
@@ -1249,11 +1259,18 @@ e_hints_window_visible_set(E_Client *ec)
    (void)ec;
 #else
    if (!e_client_has_xwindow(ec)) return;
-   if (ec->icccm.state != ECORE_X_WINDOW_STATE_HINT_WITHDRAWN)
+   if ((ec->icccm.state != ECORE_X_WINDOW_STATE_HINT_WITHDRAWN) &&
+       (!ec->frozen))
      {
         ec->icccm.state = ECORE_X_WINDOW_STATE_HINT_NORMAL;
         ecore_x_icccm_state_set(e_client_util_win_get(ec), ECORE_X_WINDOW_STATE_HINT_NORMAL);
      }
+   else if (ec->frozen)
+     {
+        if (ec->icccm.state != ECORE_X_WINDOW_STATE_HINT_WITHDRAWN)
+          ec->icccm.state = ECORE_X_WINDOW_STATE_HINT_NORMAL;
+        ecore_x_icccm_state_set(e_client_util_win_get(ec), ECORE_X_WINDOW_STATE_HINT_ICONIC);
+     }
    if (ec->netwm.state.hidden)
      {
         ec->netwm.update.state = 1;
@@ -1276,6 +1293,12 @@ e_hints_window_iconic_set(E_Client *ec)
         ec->icccm.state = ECORE_X_WINDOW_STATE_HINT_ICONIC;
         ecore_x_icccm_state_set(e_client_util_win_get(ec), ECORE_X_WINDOW_STATE_HINT_ICONIC);
      }
+   else if (ec->frozen)
+     {
+        if (ec->icccm.state != ECORE_X_WINDOW_STATE_HINT_WITHDRAWN)
+          ec->icccm.state = ECORE_X_WINDOW_STATE_HINT_ICONIC;
+        ecore_x_icccm_state_set(e_client_util_win_get(ec), ECORE_X_WINDOW_STATE_HINT_ICONIC);
+     }
    if (!ec->netwm.state.hidden)
      {
         ec->netwm.update.state = 1;
