@@ -93,18 +93,6 @@ _e_int_menus_wallpaper_cb()
    e_configure_registry_call("appearance/wallpaper", NULL, NULL);
 }
 
-static void
-_e_int_menus_bryce_cb()
-{
-   e_bryce_edit(NULL);
-}
-
-static void
-_e_int_menus_desktop_gadget_add_cb()
-{
-   e_gadget_site_desktop_edit(e_comp->canvas->gadget_site);
-}
-
 #ifdef ISCOMFITOR
 static void
 _TEST_ADD(void *data, E_Dialog *dia EINA_UNUSED)
@@ -366,16 +354,6 @@ e_int_menus_desktops_new(void)
    e_util_menu_item_theme_icon_set(mi, "preferences-desktop-wallpaper");
    e_menu_item_callback_set(mi, _e_int_menus_wallpaper_cb, NULL);
 
-   mi = e_menu_item_new(m);
-   e_menu_item_label_set(mi, _("Add Gadget Bar (EXPERIMENTAL)"));
-   e_util_menu_item_theme_icon_set(mi, "list-add");
-   e_menu_item_callback_set(mi, _e_int_menus_bryce_cb, NULL);
-
-   mi = e_menu_item_new(m);
-   e_menu_item_label_set(mi, _("Add Gadgets To Desktop"));
-   e_util_menu_item_theme_icon_set(mi, "list-add");
-   e_menu_item_callback_set(mi, _e_int_menus_desktop_gadget_add_cb, NULL);
-
    mi = e_menu_item_new(m);
    e_menu_item_separator_set(mi, 1);
 
@@ -1964,7 +1942,7 @@ _e_int_menus_shelves_add_cb(void *data EINA_UNUSED, E_Menu *m EINA_UNUSED, E_Men
    E_Zone *zone;
 
    zone = e_zone_current_get();
-   e_shelf_new_dialog(zone);
+   e_shelf_new_dialog(NULL, zone);
 }
 
 static void
