@@ -113,6 +113,7 @@ E_API Eina_Bool stopping = EINA_FALSE;
 E_API Eina_Bool restart = EINA_FALSE;
 E_API Eina_Bool e_nopause = EINA_FALSE;
 E_API Eina_Bool after_restart = EINA_FALSE;
+E_API Eina_Bool e_main_loop_running = EINA_FALSE;
 EINTERN const char *e_first_frame = NULL;
 EINTERN double e_first_frame_start_time = -1;
 
@@ -207,6 +208,7 @@ _precache_thread(void *data, Eina_Thread thr EINA_UNUSED)
    unsigned int reads = 0;
 
 
+   printf("PRECACHE: BEGIN\n");
    EINA_LIST_FREE(precache_files, path)
      {
         double tt = ecore_time_get();
@@ -343,6 +345,7 @@ main(int argc, char **argv)
    e_util_env_set("E_RESTART_OK", NULL);
    e_util_env_set("PANTS", "ON");
    e_util_env_set("DESKTOP", "Enlightenment");
+   e_util_env_set("XDG_CURRENT_DESKTOP", "Enlightenment");
    if (getenv("E_ALERT_FONT_DIR"))
      e_util_env_set("E_ALERT_FONT_DIR", NULL);
    if (getenv("E_ALERT_SYSTEM_BIN"))
@@ -599,6 +602,21 @@ main(int argc, char **argv)
         e_util_env_set("QT_QPA_PLATFORMTHEME", "gtk2");
         e_util_env_set("QT_STYLE_OVERRIDE", "gtk2");
      }
+   // make fonts NOT BLURRY. after 35 (v 38, v40 of interpreter) fonts become
+   // horizontally blurry - they seemingly want the interpreter for hinting to
+   // sub-pixel position on sub-pixel boundaries. this ends up with blurry
+   // horizontal positioning/hinting that is on a sub-pixel. yes - this
+   // requires logging out and logging in to get e to not set this env var.
+   // for now that's good enough. the aim is to get everyone to render the
+   // same way and this does it. efl, gtk, qt, chromium, firtefox, ...
+   if (e_config->scale.set_xapp_dpi)
+     {
+        s = getenv("FREETYPE_PROPERTIES");
+        if (!s)
+          {
+             e_util_env_set("FREETYPE_PROPERTIES", "truetype:interpreter-version=35");
+          }
+     }
 
    TS("E_Env Init");
    if (!e_env_init())
@@ -666,6 +684,7 @@ main(int argc, char **argv)
           }
         eina_hash_foreach(files, _precache_file, &precache_files);
         eina_hash_free(files);
+        printf("PRECACHE: SPAWN\n");
         if (!eina_thread_create(&thr, EINA_THREAD_BACKGROUND, -1,
                                 _precache_thread, precache_files))
           {
@@ -810,18 +829,15 @@ main(int argc, char **argv)
    TS("E_Scale Init Done");
    _e_main_shutdown_push(e_scale_shutdown);
 
-   if (e_config->show_splash)
+   TS("E_Splash Init");
+   if (!e_init_init())
      {
-        TS("E_Splash Init");
-        if (!e_init_init())
-          {
-             e_error_message_show(_("Enlightenment cannot set up its init screen.\n"));
-             _e_main_shutdown(101);
-          }
-        TS("E_Splash Init Done");
-        _e_main_shutdown_push(e_init_shutdown);
+        e_error_message_show(_("Enlightenment cannot set up its init screen.\n"));
+        _e_main_shutdown(101);
      }
-   if (!((!e_config->show_splash) || (after_restart)))
+   TS("E_Splash Init Done");
+   _e_main_shutdown_push(e_init_shutdown);
+   if (!after_restart)
      {
         TS("E_Splash Show");
         e_init_show();
@@ -843,6 +859,11 @@ main(int argc, char **argv)
    TS("E_Acpi Init Done");
    _e_main_shutdown_push(e_acpi_shutdown);
 
+   TS("E_Auth Init");
+   e_auth_init();
+   TS("E_Auth Init Done");
+   _e_main_shutdown_push(e_auth_shutdown);
+
    TS("E_Backlight Init");
    if (!e_backlight_init())
      {
@@ -945,11 +966,6 @@ main(int argc, char **argv)
    TS("E_Remember Init Done");
    _e_main_shutdown_push(e_remember_shutdown);
 
-   TS("E_Gadget Init");
-   e_gadget_init();
-   TS("E_Gadget Init Done");
-   _e_main_shutdown_push((void*)e_gadget_shutdown);
-
    TS("E_Gadcon Init");
    if (!e_gadcon_init())
      {
@@ -1042,6 +1058,15 @@ main(int argc, char **argv)
    TS("E_Comp_Canvas Keys Grab");
    e_comp_canvas_keys_grab();
    TS("E_Comp_Canvas Keys Grab Done");
+#ifdef HAVE_ELPUT
+   if (e_config->gesture_open_input_devices)
+     {
+        TS("E_Gesture Init");
+        e_gesture_init();
+        TS("E_Gesture Init Done");
+        _e_main_shutdown_push(e_gesture_shutdown);
+     }
+#endif
 
    TS("Run Startup Apps");
    if (!nostartup)
@@ -1065,14 +1090,6 @@ main(int argc, char **argv)
    _e_main_modules_load(safe_mode);
    TS("Load Modules Done");
 
-   if (E_EFL_VERSION_MINIMUM(1, 17, 99))
-     {
-        TS("Bryce Init");
-        e_bryce_init();
-        TS("Bryce Init Done");
-        _e_main_shutdown_push((void*)e_bryce_shutdown);
-     }
-
    TS("E_Shelf Init");
    if (!e_shelf_init())
      {
@@ -1084,15 +1101,14 @@ main(int argc, char **argv)
 
    _idle_after = ecore_idle_enterer_add(_e_main_cb_idle_after, NULL);
 
-   if (e_config->show_splash)
-     e_init_status_set(_("Welcome to Enlightenment"));
+   e_init_status_set(_("Welcome to Enlightenment"));
 
    starting = EINA_FALSE;
    inloop = EINA_TRUE;
 
    e_util_env_set("E_RESTART", "1");
 
-   if (e_config->show_splash && (!after_restart))
+   if (!after_restart)
      ecore_timer_add(2.0, _e_main_cb_startup_fake_end, NULL);
 
    if (after_restart)
@@ -1100,11 +1116,17 @@ main(int argc, char **argv)
         E_LIST_FOREACH(e_comp->zones, e_comp_canvas_zone_restarted);
      }
 
+//   e_watchdog_begin();
    TS("MAIN LOOP AT LAST");
    if (!setjmp(x_fatal_buff))
-     ecore_main_loop_begin();
+     {
+        e_main_loop_running = EINA_TRUE;
+        ecore_main_loop_begin();
+     }
    else
      CRI("FATAL: X Died. Connection gone. Abbreviated Shutdown\n");
+//   e_watchdog_end();
+   e_main_loop_running = EINA_FALSE;
 
    inloop = EINA_FALSE;
    stopping = EINA_TRUE;
@@ -1570,7 +1592,6 @@ _e_main_screens_init(void)
         _e_main_shutdown(101);
      }
    TS("E_Dnd Init Done");
-   _e_main_shutdown_push(e_dnd_shutdown);
 #endif
 
    return 1;
@@ -1582,6 +1603,7 @@ _e_main_screens_shutdown(void)
    e_win_shutdown();
    e_menu_shutdown();
    e_shelf_shutdown();
+   e_dnd_shutdown();
    e_comp_shutdown();
    e_client_volume_shutdown();
    e_screensaver_shutdown();
@@ -1634,16 +1656,41 @@ _e_main_desk_restore(void)
         unsigned int desks[2];
         Ecore_X_Atom a;
 
+        // hide all windows first as desk show will restore
+        E_CLIENT_FOREACH(ec)
+          {
+             ec->hidden = 1;
+             evas_object_show(ec->frame);
+             ec->changes.visible = 0;
+             evas_object_hide(ec->frame);
+             e_client_comp_hidden_set(ec, EINA_TRUE);
+          }
         EINA_LIST_FOREACH(e_comp->zones, l, zone)
           {
+             desk = NULL;
+
              snprintf(buf, sizeof(buf), "E_DESK_%i", zone->num);
              a = ecore_x_atom_get(buf);
              ret = ecore_x_window_prop_card32_get(e_comp->root, a, desks, 2);
              if (ret == 2)
                {
                   desk = e_desk_at_xy_get(zone, desks[0], desks[1]);
-                  if (!desk) continue;
-                  e_desk_show(desk);
+                  if (desk) e_desk_show(desk);
+               }
+             if (!desk) desk = e_desk_current_get(zone);
+             if (desk)
+               {
+                  // ensure windows for this desk are shown
+                  E_CLIENT_REVERSE_FOREACH(ec)
+                    {
+                       if (ec->iconic) continue;
+                       if ((ec->desk == desk) || (ec->sticky))
+                         {
+                            ec->hidden = 0;
+                            e_client_comp_hidden_set(ec, ec->hidden || ec->shaded);
+                            evas_object_show(ec->frame);
+                         }
+                    }
                }
           }
         E_CLIENT_REVERSE_FOREACH(ec)
