@@ -53,6 +53,7 @@ struct _E_Pixmap
    void *data;
    Eina_Rectangle opaque;
    Eina_List *free_buffers;
+   Eina_List *busy_list;
 #endif
 
    Eina_Bool usable E_BITFIELD;
@@ -62,8 +63,6 @@ struct _E_Pixmap
 
 #ifdef HAVE_WAYLAND
 
-double wayland_time_base;
-
 static void
 _e_pixmap_cb_deferred_buffer_destroy(struct wl_listener *listener, void *data EINA_UNUSED)
 {
@@ -138,6 +137,8 @@ static void
 _e_pixmap_wl_resource_release(E_Comp_Wl_Buffer *buffer)
 {
    buffer->busy--;
+   if (buffer->dmabuf_buffer)
+     linux_dmabuf_buffer_unref(buffer->dmabuf_buffer);
    if (buffer->busy) return;
 
    if (buffer->pool)
@@ -173,6 +174,11 @@ _e_pixmap_wayland_buffer_release(E_Pixmap *cp, E_Comp_Wl_Buffer *buffer)
         return;
      }
 
+   if (buffer->busy == 1)
+     {
+        // we are at busy 1 abut about to go to 0 below, so remove now
+        cp->busy_list = eina_list_remove(cp->busy_list, buffer);
+     }
    _e_pixmap_wl_resource_release(buffer);
 }
 
@@ -195,6 +201,18 @@ _e_pixmap_wayland_image_clear(E_Pixmap *cp)
 {
    EINA_SAFETY_ON_NULL_RETURN(cp);
 
+   if (cp->busy_list)
+     {
+        E_Comp_Wl_Buffer *buffer;
+
+        EINA_LIST_FREE(cp->busy_list, buffer)
+          {
+             buffer->busy--;
+             if (buffer->dmabuf_buffer)
+               linux_dmabuf_buffer_unref(buffer->dmabuf_buffer);
+          }
+     }
+
    if (!cp->held_buffer) return;
    if (!cp->held_buffer->pool) return;
 
@@ -393,7 +411,6 @@ e_pixmap_new(E_Pixmap_Type type, ...)
         else
           {
              pixmaps[type] = eina_hash_int64_new((Eina_Free_Cb)_e_pixmap_free);
-             wayland_time_base = ecore_loop_time_get();
           }
         cp = _e_pixmap_new(type);
         cp->win = id;
@@ -470,7 +487,8 @@ e_pixmap_usable_set(E_Pixmap *cp, Eina_Bool set)
 E_API Eina_Bool
 e_pixmap_usable_get(const E_Pixmap *cp)
 {
-   EINA_SAFETY_ON_NULL_RETURN_VAL(cp, EINA_FALSE);
+   if (!cp) return EINA_FALSE;
+//   EINA_SAFETY_ON_NULL_RETURN_VAL(cp, EINA_FALSE);
    return cp->usable;
 }
 
@@ -573,12 +591,13 @@ e_pixmap_refresh(E_Pixmap *cp)
              format = wl_shm_buffer_get_format(buffer->shm_buffer);
            else if (buffer->dmabuf_buffer)
              format = buffer->dmabuf_buffer->attributes.format;
-           else
+           else if (e_comp_wl->wl.glapi)
              {
                 e_comp_wl->wl.glapi->evasglQueryWaylandBuffer
                   (e_comp_wl->wl.gl, buffer->resource, EGL_TEXTURE_FORMAT,
                       &format);
              }
+           else return EINA_FALSE;
 
            switch (format)
              {
@@ -705,7 +724,17 @@ e_pixmap_resource_set(E_Pixmap *cp, void *resource)
    if (cp->buffer == resource) return;
 
    if (cp->buffer)
-     _e_pixmap_wl_resource_release(cp->buffer);
+     {
+        if (resource)
+          {
+        if (cp->buffer->busy == 1)
+          {
+             // about to go to busy 0, so remove from list
+             cp->busy_list = eina_list_remove(cp->busy_list, cp->buffer);
+          }
+        _e_pixmap_wl_resource_release(cp->buffer);
+          }
+     }
 
    if (cp->buffer_destroy_listener.notify)
      {
@@ -718,7 +747,14 @@ e_pixmap_resource_set(E_Pixmap *cp, void *resource)
    wl_signal_add(&cp->buffer->destroy_signal,
                  &cp->buffer_destroy_listener);
 
+   if (cp->buffer->dmabuf_buffer)
+     linux_dmabuf_buffer_ref(cp->buffer->dmabuf_buffer);
    cp->buffer->busy++;
+   if (cp->buffer->busy > 0)
+     {
+        cp->busy_list = eina_list_remove(cp->busy_list, cp->buffer);
+        cp->busy_list = eina_list_append(cp->busy_list, cp->buffer);
+     }
 #else
    (void)resource;
 #endif
@@ -753,23 +789,32 @@ e_pixmap_is_pixels(E_Pixmap *cp)
 static void
 _e_pixmap_scanout_handler(void *data, Evas_Native_Surface_Status status)
 {
-   E_Comp_Wl_Buffer *buffer;
+   E_Comp_Wl_Buffer *buffer = data;
 
    printf("EWL: %s, Status: %d\n", __FUNCTION__, status);
 
-   buffer = data;
    switch (status)
      {
       case EVAS_NATIVE_SURFACE_STATUS_SCANOUT_ON:
+        if (buffer->dmabuf_buffer)
+          linux_dmabuf_buffer_ref(buffer->dmabuf_buffer);
         buffer->busy++;
+        // XXX: fixme - handle cp->busy_list
         break;
       case EVAS_NATIVE_SURFACE_STATUS_SCANOUT_OFF:
+        // busy--
+        // XXX: fixme - handle cp->busy_list
         _e_pixmap_wl_resource_release(buffer);
         break;
       case EVAS_NATIVE_SURFACE_STATUS_PLANE_ASSIGN:
+        if (buffer->dmabuf_buffer)
+          linux_dmabuf_buffer_ref(buffer->dmabuf_buffer);
+        // XXX: fixme - handle cp->busy_list
         buffer->busy++;
         break;
       case EVAS_NATIVE_SURFACE_STATUS_PLANE_RELEASE:
+        // busy--
+        // XXX: fixme - handle cp->busy_list
         _e_pixmap_wl_resource_release(buffer);
         break;
      }
@@ -889,6 +934,7 @@ e_pixmap_image_clear(E_Pixmap *cp, Eina_Bool cache)
              E_Comp_Wl_Client_Data *cd;
              struct wl_resource *cb;
              Eina_List *free_list;
+             double t;
 
              if ((!cp->client) || (!cp->client->comp_data)) return;
 
@@ -899,9 +945,10 @@ e_pixmap_image_clear(E_Pixmap *cp, Eina_Bool cache)
               */
              free_list = cd->frames;
              cd->frames = NULL;
+             t = ecore_loop_time_get();
              EINA_LIST_FREE(free_list, cb)
                {
-                  double t = ecore_loop_time_get();
+//                  printf("E: frame request: %p [%ix%i] %1.5f @ %1.5f\n", cp, cp->w, cp->h, t, ecore_time_get());
                   wl_callback_send_done(cb, t * 1000);
                   wl_resource_destroy(cb);
                }
@@ -965,6 +1012,8 @@ e_pixmap_image_refresh(E_Pixmap *cp)
            cp->held_buffer_destroy_listener.notify =
              _e_pixmap_cb_held_buffer_destroy;
 
+           if (cp->held_buffer_destroy_listener.link.next)
+             wl_list_remove(&cp->held_buffer_destroy_listener.link);
            wl_signal_add(&cp->held_buffer->destroy_signal,
                          &cp->held_buffer_destroy_listener);
            return EINA_TRUE;
@@ -1200,19 +1249,91 @@ e_pixmap_dmabuf_test(struct linux_dmabuf_buffer *dmabuf)
    return EINA_TRUE;
 }
 
-E_API Eina_Bool
-e_pixmap_dmabuf_formats_query(int **formats EINA_UNUSED, int *num_formats)
-{
-   *num_formats = 0;
+/*
+if (weston_check_egl_extension(extensions,
+                               "EGL_EXT_image_dma_buf_import_modifiers")) {
+   gr->query_dmabuf_formats =
+   (void *) eglGetProcAddress("eglQueryDmaBufFormatsEXT");
+   gr->query_dmabuf_modifiers =
+   (void *) eglGetProcAddress("eglQueryDmaBufModifiersEXT");
+   assert(gr->query_dmabuf_formats);
+   assert(gr->query_dmabuf_modifiers);
+   gr->has_dmabuf_import_modifiers = true;
+}
+*/
 
+extern Eina_Bool e_comp_wl_query_dmabuf_formats(int max_formats, int *formats, int *num_formats);
+extern Eina_Bool e_comp_wl_query_dmabuf_modifiers(int format, int max_modifiers, uint64_t *modifiers, Eina_Bool *external_only, int *num_modifiers);
+
+E_API Eina_Bool
+e_pixmap_dmabuf_formats_query(int **formats, int *num_formats)
+{
+   static const int fallback_formats[] =
+     {
+        DRM_FORMAT_ARGB8888,
+        DRM_FORMAT_XRGB8888,
+// support later ...
+//      DRM_FORMAT_YUYV,
+//      DRM_FORMAT_NV12,
+//      DRM_FORMAT_YUV420,
+//      DRM_FORMAT_YUV444,
+//      DRM_FORMAT_XYUV8888,
+     };
+   Eina_Bool fallback = EINA_FALSE;
+   int num = 0;
+
+   if (!e_comp_wl_query_dmabuf_formats(0, NULL, &num))
+     {
+        num = 2;
+        fallback = EINA_TRUE;
+     }
+   *formats = calloc(num, sizeof(int));
+   if (!(*formats))
+     {
+        *num_formats = 0;
+        return EINA_FALSE;
+     }
+   if (fallback)
+     {
+        memcpy(*formats, fallback_formats, num * sizeof(int));
+        *num_formats = num;
+        return EINA_TRUE;
+     }
+   if (!e_comp_wl_query_dmabuf_formats(num, *formats, &num))
+     {
+        *num_formats = 0;
+        free(*formats);
+        return EINA_FALSE;
+     }
+   *num_formats = num;
    return EINA_TRUE;
 }
 
 E_API Eina_Bool
-e_pixmap_dmabuf_modifiers_query(int format EINA_UNUSED, uint64_t **modifiers EINA_UNUSED, int *num_modifiers)
+e_pixmap_dmabuf_modifiers_query(int format, uint64_t **modifiers, int *num_modifiers)
 {
-   *num_modifiers = 0;
+   int num;
 
+   if (!e_comp_wl_query_dmabuf_modifiers(format, 0, NULL, NULL, &num) ||
+       (num == 0))
+     {
+        *num_modifiers = 0;
+        return EINA_FALSE;
+     }
+   *modifiers = calloc(num, sizeof(uint64_t));
+   if (!(*modifiers))
+     {
+        *num_modifiers = 0;
+        return EINA_FALSE;
+     }
+   if (!e_comp_wl_query_dmabuf_modifiers
+       (format, num, *modifiers, NULL, &num))
+     {
+        *num_modifiers = 0;
+        free(*modifiers);
+        return EINA_FALSE;
+     }
+   *num_modifiers = num;
    return EINA_TRUE;
 }
 
