@@ -217,29 +217,44 @@ _e_pointer_cb_mouse_wheel(void *data EINA_UNUSED, int type EINA_UNUSED, void *ev
    return ECORE_CALLBACK_PASS_ON;
 }
 
+static void
+_hot_update(E_Pointer *ptr)
+{
+   int x = 0, y = 0, ox = 0, oy = 0;
+
+   if (ptr->e_cursor && (e_comp->comp_type != E_PIXMAP_TYPE_WL))
+     evas_object_geometry_get(ptr->buffer_o_ptr, &ox, &oy, NULL, NULL);
+   else
+     evas_object_geometry_get(ptr->o_ptr, &ox, &oy, NULL, NULL);
+   evas_object_geometry_get(ptr->o_hot, &x, &y, NULL, NULL);
+   _e_pointer_hot_update(ptr, x - ox, y - oy);
+}
+
 static void
 _e_pointer_cb_hot_move(void *data, Evas *evas EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event EINA_UNUSED)
 {
    E_Pointer *ptr = data;
-   int x = 0, y = 0;
 
    if (!ptr->e_cursor) return;
    if (!evas_object_visible_get(ptr->o_ptr)) return;
-   edje_object_part_geometry_get(ptr->o_ptr, "e.swallow.hotspot",
-                                 &x, &y, NULL, NULL);
-   _e_pointer_hot_update(ptr, x, y);
+   _hot_update(ptr);
 }
 
 static void
 _e_pointer_cb_hot_show(void *data, Evas *evas EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event EINA_UNUSED)
 {
    E_Pointer *ptr = data;
-   int x = 0, y = 0;
 
    if (!ptr->e_cursor) return;
-   edje_object_part_geometry_get(ptr->o_ptr, "e.swallow.hotspot",
-                                 &x, &y, NULL, NULL);
-   _e_pointer_hot_update(ptr, x, y);
+   _hot_update(ptr);
+}
+
+static void
+_del_cb(void *data, const Efl_Event *ev EINA_UNUSED) {
+  E_Pointer *ptr = data;
+  INF("Cursor-Object (%p) of %p was removed!", ptr->o_ptr, data);
+  ptr->o_ptr = NULL;
+  return;
 }
 
 static void
@@ -248,12 +263,17 @@ _e_pointer_pointer_canvas_init(E_Pointer *ptr, Evas *e, Evas_Object **o_ptr, Eva
    /* create pointer object */
    *o_ptr = edje_object_add(e);
 
+   efl_event_callback_add(*o_ptr, EFL_EVENT_DEL, _del_cb, ptr);
+
    /* create hotspot object */
    *o_hot = evas_object_rectangle_add(e);
    evas_object_color_set(*o_hot, 0, 0, 0, 0);
 
+
    evas_object_event_callback_add(*o_hot, EVAS_CALLBACK_MOVE,
                                   _e_pointer_cb_hot_move, ptr);
+   evas_object_event_callback_add(*o_ptr, EVAS_CALLBACK_MOVE,
+                                  _e_pointer_cb_hot_move, ptr);
    evas_object_event_callback_add(*o_hot, EVAS_CALLBACK_SHOW,
                                   _e_pointer_cb_hot_show, ptr);
 
@@ -459,7 +479,6 @@ _e_pointer_type_set(E_Pointer *ptr, const char *type)
    if (ptr->e_cursor)
      {
         char cursor[1024];
-        int x = 0, y = 0;
 
         if ((!ptr->buffer_evas) && ptr->win) _e_pointer_canvas_add(ptr);
         _e_pointer_theme_buf(ptr, cursor);
@@ -477,9 +496,7 @@ _e_pointer_type_set(E_Pointer *ptr, const char *type)
         _e_pointer_x11_setup(ptr, cursor);
         if (!cursor[0]) return;
 
-        edje_object_part_geometry_get(ptr->o_ptr, "e.swallow.hotspot",
-                                      &x, &y, NULL, NULL);
-        _e_pointer_hot_update(ptr, x, y);
+        _hot_update(ptr);
 
         if (ptr->canvas)
           e_pointer_object_set(ptr, NULL, 0, 0);
@@ -616,6 +633,9 @@ e_pointer_hide(E_Pointer *ptr)
 E_API void
 e_pointer_show(E_Pointer *ptr)
 {
+   EINA_SAFETY_ON_NULL_RETURN(ptr);
+   EINA_SAFETY_ON_NULL_RETURN(ptr->o_ptr);
+
    if ((!ptr->buffer_evas) && ptr->win) _e_pointer_canvas_add(ptr);
    if (ptr->canvas)
      evas_object_show(ptr->o_ptr);
@@ -627,6 +647,7 @@ e_pointer_type_push(E_Pointer *ptr, void *obj, const char *type)
    E_Pointer_Stack *stack;
 
    EINA_SAFETY_ON_NULL_RETURN(ptr);
+   EINA_SAFETY_ON_NULL_RETURN(obj);
 
    _e_pointer_type_set(ptr, type);
 
@@ -646,7 +667,7 @@ e_pointer_type_pop(E_Pointer *ptr, void *obj, const char *type)
 
    EINA_LIST_FOREACH_SAFE(ptr->stack, l, ll, stack)
      {
-        if ((stack->obj == obj) &&
+        if (((stack->obj == obj) || (!obj)) &&
             ((!type) || (!e_util_strcmp(stack->type, type))))
           {
              _e_pointer_stack_free(stack);
@@ -655,18 +676,28 @@ e_pointer_type_pop(E_Pointer *ptr, void *obj, const char *type)
           }
      }
 
-   if (!ptr->stack)
+   if ((!obj) && (!type))
      {
-        e_pointer_hide(ptr);
-        eina_stringshare_replace(&ptr->type, NULL);
-        return;
+        if (!ptr->stack)
+          {
+             e_pointer_type_push(ptr, ptr, "default");
+          }
+     }
+   else
+     {
+        if (!ptr->stack)
+          {
+             e_pointer_hide(ptr);
+             eina_stringshare_replace(&ptr->type, NULL);
+             return;
+          }
      }
 
    if (!(stack = eina_list_data_get(ptr->stack))) return;
 
    _e_pointer_type_set(ptr, stack->type);
 
-   eina_stringshare_replace(&ptr->type, stack->type);
+   /* eina_stringshare_replace(&ptr->type, stack->type); */
 }
 
 E_API void
@@ -767,10 +798,15 @@ e_pointer_idler_before(void)
 
    if (!e_config->show_cursor) return;
 
+   if (ecore_evas_manual_render_get(e_comp->ee)) return;
    EINA_LIST_FOREACH(_ptrs, l, ptr)
      {
         if ((!ptr->e_cursor) || (!ptr->buffer_evas)) continue;
 
+        if (!ptr->o_ptr) {
+          EINA_SAFETY_ERROR("ptr->o_ptr is NULL should not be NULL!");
+        }
+
         if (ptr->hot.update)
           _e_pointer_type_set(ptr, ptr->type);
 
@@ -782,9 +818,15 @@ e_pointer_idler_before(void)
                {
 #ifndef HAVE_WAYLAND_ONLY
                   Ecore_X_Cursor cur;
+                  int hotx = ptr->hot.x;
+                  int hoty = ptr->hot.y;
 
-                  cur = ecore_x_cursor_new(ptr->win, ptr->pixels, ptr->w,
-                                           ptr->h, ptr->hot.x, ptr->hot.y);
+                  if      (hotx < 0)       hotx = 0;
+                  else if (hotx >= ptr->w) hotx = ptr->w - 1;
+                  if      (hoty < 0)       hoty = 0;
+                  else if (hoty >= ptr->h) hoty = ptr->h - 1;
+                  cur = ecore_x_cursor_new(ptr->win, ptr->pixels,
+                                           ptr->w, ptr->h, hotx, hoty);
                   ecore_x_window_cursor_set(ptr->win, cur);
                   ecore_x_cursor_free(cur);
 #endif
@@ -803,6 +845,9 @@ e_pointer_object_set(E_Pointer *ptr, Evas_Object *obj, int x, int y)
    E_Client *ec;
    int px, py;
 
+   EINA_SAFETY_ON_NULL_RETURN(ptr);
+   EINA_SAFETY_ON_NULL_RETURN(ptr->o_ptr);
+
    ecore_evas_cursor_get(ptr->ee, &o, NULL, &px, &py);
    if (o)
      {
@@ -830,7 +875,8 @@ e_pointer_object_set(E_Pointer *ptr, Evas_Object *obj, int x, int y)
    else if ((o != ptr->o_ptr) || (x != px) || (y != py))
      {
         ecore_evas_cursor_unset(ptr->ee);
-        ecore_evas_object_cursor_set(ptr->ee, ptr->o_ptr, E_LAYER_MAX - 1, ptr->hot.x, ptr->hot.y);
+        ecore_evas_object_cursor_set(ptr->ee, ptr->o_ptr, E_LAYER_MAX - 1,
+                                     ptr->hot.x, ptr->hot.y);
         evas_object_show(ptr->o_ptr);
      }
    ptr->client.ec = ec;
@@ -848,6 +894,9 @@ e_pointer_window_add(E_Pointer *ptr, Ecore_Window win)
 E_API void
 e_pointer_grab_set(E_Pointer *ptr, Eina_Bool grab)
 {
+   EINA_SAFETY_ON_NULL_RETURN(ptr);
+   EINA_SAFETY_ON_NULL_RETURN(ptr->o_ptr);
+
    if (grab) ptr->grabcount++;
    else
      {
@@ -864,6 +913,21 @@ e_pointer_grab_set(E_Pointer *ptr, Eina_Bool grab)
      }
 }
 
+E_API void
+e_pointer_reset(E_Pointer *ptr)
+{
+   EINA_SAFETY_ON_NULL_RETURN(ptr);
+
+   /* free stack of pointer types */
+   E_FREE_LIST(ptr->stack, _e_pointer_stack_free);
+
+   eina_stringshare_del(ptr->type);
+   eina_stringshare_del(ptr->deferred_type);
+
+   /* reset pointer to default */
+   e_pointer_type_push(ptr, ptr, "default");
+}
+
 EINTERN void
 e_pointers_freeze_set(Eina_Bool set)
 {
