@@ -1,6 +1,6 @@
 #include "e.h"
 
-#define E_RANDR_CONFIG_VERSION 2
+#define E_RANDR_CONFIG_VERSION 3
 
 /////////////////////////////////////////////////////////////////////////
 static Eina_Bool               _screen_closed(E_Randr2_Screen *s);
@@ -34,6 +34,8 @@ static Eina_List     *_ev_handlers = NULL;
 static Ecore_Timer   *_screen_delay_timer = NULL;
 static Eina_Bool      event_screen = EINA_FALSE;
 static Eina_Bool      event_ignore = EINA_FALSE;
+static Eina_Bool      initted = EINA_FALSE;
+static Eina_Bool      blocked = EINA_FALSE;
 
 /////////////////////////////////////////////////////////////////////////
 E_API E_Config_Randr2 *e_randr2_cfg = NULL;
@@ -54,7 +56,9 @@ e_randr2_init(void)
    int count;
 
    if (!E_EVENT_RANDR_CHANGE) E_EVENT_RANDR_CHANGE = ecore_event_type_new();
+   if (blocked) return EINA_FALSE;
    if ((!e_comp->screen) || (!e_comp->screen->available) || (!e_comp->screen->available())) return EINA_FALSE;
+   initted = EINA_TRUE;
    // create data descriptors for config storage
    _e_randr2_cfg_screen_edd =
      E_CONFIG_DD_NEW("E_Config_Randr2_Screen", E_Config_Randr2_Screen);
@@ -72,6 +76,8 @@ e_randr2_init(void)
    E_CONFIG_VAL(D, T, priority, INT);
    E_CONFIG_VAL(D, T, rel_mode, UCHAR);
    E_CONFIG_VAL(D, T, enabled, UCHAR);
+   E_CONFIG_VAL(D, T, ignore_disconnect, UCHAR);
+   E_CONFIG_VAL(D, T, custom_label_screen, STR);
    E_CONFIG_VAL(D, T, profile, STR);
    E_CONFIG_VAL(D, T, scale_multiplier, DOUBLE);
 
@@ -82,6 +88,7 @@ e_randr2_init(void)
 #define D _e_randr2_cfg_edd
    E_CONFIG_VAL(D, T, version, INT);
    E_CONFIG_LIST(D, T, screens, _e_randr2_cfg_screen_edd);
+   E_CONFIG_VAL(D, T, hotplug_response, DOUBLE);
    E_CONFIG_VAL(D, T, restore, UCHAR);
    E_CONFIG_VAL(D, T, ignore_hotplug_events, UCHAR);
    E_CONFIG_VAL(D, T, ignore_acpi_events, UCHAR);
@@ -121,6 +128,8 @@ e_randr2_init(void)
 EINTERN int
 e_randr2_shutdown(void)
 {
+   if (!initted) return 0;
+   initted = EINA_FALSE;
    _animated_apply_abort();
    // nuke any screen config delay handler
    if (_screen_delay_timer) ecore_timer_del(_screen_delay_timer);
@@ -141,6 +150,13 @@ e_randr2_shutdown(void)
    return 1;
 }
 
+E_API void
+e_randr2_stop(void)
+{
+   blocked = EINA_TRUE;
+   if (initted) e_randr2_shutdown();
+}
+
 E_API Eina_Bool
 e_randr2_config_save(void)
 {
@@ -154,12 +170,46 @@ e_randr2_config_apply(void)
    _animated_apply();
 }
 
+static E_Randr2_Screen *
+_randr_screen_find(E_Randr2 *r, E_Randr2_Screen *src)
+{
+   Eina_List *l;
+   E_Randr2_Screen *s;
+
+   EINA_LIST_FOREACH(r->screens, l, s)
+     {
+        if ((s->info.name) && (src->info.name) &&
+            (!strcmp(s->info.name, src->info.name)))
+          return s;
+     }
+   return NULL;
+}
+
+static void
+_randr_screen_props_copyover(E_Randr2 *rold, E_Randr2 *rnew)
+{
+   Eina_List *l;
+   E_Randr2_Screen *s, *s2;
+
+   // copy s->config.ignore_disconnect over from tr to e_randr2
+   EINA_LIST_FOREACH(rnew->screens, l, s)
+     {
+        s2 = _randr_screen_find(rold, s);
+        if (s2) s->config.ignore_disconnect = s2->config.ignore_disconnect;
+     }
+}
+
 E_API void
 e_randr2_screeninfo_update(void)
 {
+   E_Randr2 *tr = e_randr2;
    // re-fetch/update current screen info
-   _info_free(e_randr2);
    e_randr2 = e_comp->screen->create();
+   if ((tr) && (e_randr2))
+     {
+        _randr_screen_props_copyover(tr, e_randr2);
+        _info_free(tr);
+     }
    _screen_config_maxsize();
 }
 
@@ -265,10 +315,16 @@ _animated_apply(void)
 static void
 _do_apply(void)
 {
+   E_Randr2 *tr = e_randr2;
    // take current screen config and apply it to the driver
    printf("RRR: re-get info before applying..\n");
-   _info_free(e_randr2);
+   tr = e_randr2;
    e_randr2 = e_comp->screen->create();
+   if ((tr) && (e_randr2))
+     {
+        _randr_screen_props_copyover(tr, e_randr2);
+        _info_free(tr);
+     }
    _screen_config_maxsize();
    printf("RRR: apply config...\n");
    _config_apply(e_randr2, e_randr2_cfg);
@@ -308,6 +364,9 @@ _config_upgrade(E_Config_Randr2 *cfg)
 {
    if (cfg->version < 2)
      cfg->default_policy = E_RANDR2_POLICY_EXTEND;
+   if (cfg->version < 3)
+     cfg->hotplug_response = 1.0;
+   cfg->version = E_RANDR_CONFIG_VERSION;
 }
 
 static E_Config_Randr2 *
@@ -321,6 +380,10 @@ _config_load(void)
      {
         if (cfg->version < E_RANDR_CONFIG_VERSION)
           _config_upgrade(cfg);
+        if (cfg->hotplug_response < 0.2)
+          cfg->hotplug_response = 0.2;
+        else if (cfg->hotplug_response > 9.0)
+          cfg->hotplug_response = 9.0;
         printf("RRR: loaded existing config\n");
         return cfg;
      }
@@ -333,6 +396,7 @@ _config_load(void)
    cfg->ignore_hotplug_events = 0;
    cfg->ignore_acpi_events = 0;
    cfg->default_policy = E_RANDR2_POLICY_EXTEND;
+   cfg->hotplug_response = 1.0;
    printf("RRR: fresh config\n");
    return cfg;
 }
@@ -348,6 +412,7 @@ _config_free(E_Config_Randr2 *cfg)
      {
         eina_stringshare_del(cs->id);
         eina_stringshare_del(cs->rel_to);
+        eina_stringshare_del(cs->custom_label_screen);
         eina_stringshare_del(cs->profile);
         free(cs);
      }
@@ -381,8 +446,13 @@ _config_update(E_Randr2 *r, E_Config_Randr2 *cfg, Eina_Bool update_only)
    printf("--------------------------------------------------\n");
    EINA_LIST_FOREACH(r->screens, l, s)
      {
-        printf("RRR: out id=%s:  connected=%i\n", s->id, s->info.connected);
-        if ((!s->id) || (!s->info.connected) || (_screen_closed(s))) continue;
+        printf("RRR: out id=%s:  connected=%i enabled=%i configured=%i\n",
+               s->id, s->info.connected,
+               s->config.enabled, s->config.configured);
+        if (!s->id) continue;
+        if (!s->info.connected) continue;
+        if (_screen_closed(s)) continue;
+
         cs = e_randr2_config_screen_find(s, cfg);
         if (cs && update_only) continue;
         if (!cs)
@@ -469,6 +539,7 @@ _config_update(E_Randr2 *r, E_Config_Randr2 *cfg, Eina_Bool update_only)
                }
              printf("RRR: store scale mul %1.5f\n", cs->scale_multiplier);
              s->config.scale_multiplier = cs->scale_multiplier;
+             s->config.ignore_disconnect = cs->ignore_disconnect;
              ret = EINA_TRUE;
           }
      }
@@ -498,6 +569,8 @@ _config_really_apply(E_Randr2_Screen *s, E_Config_Randr2_Screen *cs)
         if (cs->profile) s->config.profile = strdup(cs->profile);
         else s->config.profile = NULL;
         s->config.scale_multiplier = cs->scale_multiplier;
+        s->config.ignore_disconnect = cs->ignore_disconnect;
+        printf("RRR: really apply '%s' ignore discon %i\n", s->info.name, s->config.ignore_disconnect);
      }
    else
      {
@@ -519,6 +592,7 @@ _config_really_apply(E_Randr2_Screen *s, E_Config_Randr2_Screen *cs)
         free(s->config.profile);
         s->config.profile = NULL;
         s->config.scale_multiplier = 0.0;
+        s->config.ignore_disconnect = EINA_FALSE;
      }
 }
 
@@ -546,7 +620,10 @@ _config_apply(E_Randr2 *r, E_Config_Randr2 *cfg)
         else
           {
              printf("RRR: ... disabled\n");
-             _config_really_apply(s, NULL);
+             if (!s->config.ignore_disconnect)
+               _config_really_apply(s, NULL);
+             else
+               printf("RRR: ... ignore disconnected\n");
           }
         s->config.configured = EINA_TRUE;
      }
@@ -643,13 +720,25 @@ _screens_differ(E_Randr2 *r1, E_Randr2 *r2)
    printf("RRR: --------\n");
    EINA_LIST_FOREACH(r2->screens, l, s)
      {
+        if (s->id)
+          printf("RRR: look at r2 screen ID %s\n", s->id);
+        else
+          printf("RRR: look at r2 screen ID NIL\n");
         if (!s->id) continue;
         EINA_LIST_FOREACH(r1->screens, ll, ss)
           {
+             if (ss->id)
+               printf("RRR: look at r1 screen ID %s\n", ss->id);
+             else
+               printf("RRR: look at r1 screen ID NIL\n");
              if ((ss->id) && (!strcmp(s->id, ss->id))) break;
              ss = NULL;
           }
-        if (!ss) changed = EINA_TRUE;
+        if (!ss)
+          {
+             printf("RRR: do change because cannot find screen matching ID %s\n", s->id);
+             changed = EINA_TRUE;
+          }
         else if ((s->config.geom.x != ss->config.geom.x) ||
                  (s->config.geom.y != ss->config.geom.y) ||
                  (s->config.geom.w != ss->config.geom.w) ||
@@ -658,7 +747,19 @@ _screens_differ(E_Randr2 *r1, E_Randr2 *r2)
                  (s->config.mode.h != ss->config.mode.h) ||
                  (s->config.enabled != ss->config.enabled) ||
                  (s->config.rotation != ss->config.rotation))
-          changed = EINA_TRUE;
+          {
+             printf("RRR: do change because geom/mode/rotation don't match\n");
+             printf("RRR: geom=%i,%i-%ix%i mode=%ix%i enable=%i rot=%i  !=  geom=%i,%i-%ix%i mode=%ix%i enable=%i rot=%i\n",
+                    s->config.geom.x, s->config.geom.y,
+                    s->config.geom.w, s->config.geom.h,
+                    s->config.mode.w, s->config.mode.h,
+                    s->config.enabled, s->config.rotation,
+                    ss->config.geom.x, ss->config.geom.y,
+                    ss->config.geom.w, ss->config.geom.h,
+                    ss->config.mode.w, ss->config.mode.h,
+                    ss->config.enabled, ss->config.rotation);
+             changed = EINA_TRUE;
+          }
         else
           {
              if (r1_screen_num != r2_screen_num)
@@ -734,6 +835,7 @@ _cb_screen_change_delay(void *data EINA_UNUSED)
         rtemp = e_comp->screen->create();
         if (rtemp)
           {
+             if (e_randr2) _randr_screen_props_copyover(e_randr2, rtemp);
              if (_screens_differ(e_randr2, rtemp)) change = EINA_TRUE;
              if (e_randr2_cfg->default_policy != E_RANDR2_POLICY_NONE)
                {
@@ -752,7 +854,7 @@ _cb_screen_change_delay(void *data EINA_UNUSED)
                     {
                        printf("RRR: is lid, lid++\n");
                        lid_screens++;
-                       if (s->info.lid_closed)
+                       if ((s->info.lid_closed) && (!s->config.ignore_disconnect))
                          {
                             printf("RRR: is lid, is closed, closed++\n");
                             close_lid_screens++;
@@ -1349,7 +1451,13 @@ e_randr2_screen_refresh_queue(Eina_Bool lid_event)
    if (_screen_delay_timer)
      ecore_timer_loop_reset(_screen_delay_timer);
    else
-     _screen_delay_timer = ecore_timer_loop_add(1.0, _cb_screen_change_delay, NULL);
+     {
+        double t = 1.0;
+
+        if ((e_randr2_cfg) && (e_randr2_cfg->hotplug_response > 0.2))
+          t = e_randr2_cfg->hotplug_response;
+        _screen_delay_timer = ecore_timer_loop_add(t, _cb_screen_change_delay, NULL);
+     }
    event_screen |= !!lid_event;
 }
 
