@@ -9,6 +9,11 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
+#ifdef HAVE_PRCTL
+# include <sys/prctl.h>
+#elif defined(HAVE_PROCCTL)
+# include <sys/procctl.h>
+#endif
 
 /* the ptrace interface used here is really linux specific -
  * FreeBSD, NetBSD and Mac OS X use slightly different ptrace API that should
@@ -24,7 +29,6 @@
 # endif
 #endif
 
-#include <limits.h>
 #include <fcntl.h>
 #ifdef HAVE_ALLOCA_H
 # include <alloca.h>
@@ -32,36 +36,51 @@
 #include <signal.h>
 #include <errno.h>
 
+# if !defined (__FreeBSD__) && !defined (__OpenBSD__)
+#  ifdef HAVE_MALLOC_H
+#   include <malloc.h>
+#  endif
+# endif
+
 #include <Eina.h>
 
-# ifdef E_API
-#  undef E_API
-# endif
-# ifdef WIN32
-#  ifdef BUILDING_DLL
-#   define E_API __declspec(dllexport)
-#  else
-#   define E_API __declspec(dllimport)
-#  endif
+#define myasprintf(__b, __fmt, args...) do { \
+   char __bb[sizeof(__fmt) + 1]; \
+   int __cnt = snprintf(__bb, sizeof(__bb), __fmt, ##args); \
+   if (__cnt >= 0) { \
+      *(__b) = alloca(__cnt + 1); \
+      snprintf(*(__b), __cnt + 1, __fmt, ##args); \
+   } \
+} while (0)
+
+#ifdef E_API
+# undef E_API
+#endif
+#ifdef WIN32
+# ifdef BUILDING_DLL
+#  define E_API __declspec(dllexport)
 # else
-#  ifdef __GNUC__
-#   if __GNUC__ >= 4
+#  define E_API __declspec(dllimport)
+# endif
+#else
+# ifdef __GNUC__
+#  if __GNUC__ >= 4
 /* BROKEN in gcc 4 on amd64 */
-#    if 0
-#     pragma GCC visibility push(hidden)
-#    endif
-#    define E_API __attribute__ ((visibility("default")))
-#   else
-#    define E_API
+#   if 0
+#    pragma GCC visibility push(hidden)
 #   endif
+#   define E_API __attribute__ ((visibility("default")))
 #  else
 #   define E_API
 #  endif
+# else
+#  define E_API
 # endif
+#endif
 
 static Eina_Bool stop_ptrace = EINA_FALSE;
 
-static void env_set(const char *var, const char *val);
+static void  env_set(const char *var, const char *val);
 E_API int    prefix_determine(char *argv0);
 
 static void
@@ -109,82 +128,10 @@ prefix_determine(char *argv0)
    return 1;
 }
 
-static int
-find_valgrind(char *path, size_t path_len)
-{
-   const char *env = getenv("PATH");
-
-   while (env)
-     {
-        const char *p = strchr(env, ':');
-        ssize_t p_len;
-
-        if (p) p_len = p - env;
-        else p_len = strlen(env);
-        if (p_len <= 0) goto next;
-        else if (p_len + sizeof("/valgrind") >= path_len)
-          goto next;
-        memcpy(path, env, p_len);
-        memcpy(path + p_len, "/valgrind", sizeof("/valgrind"));
-        if (access(path, X_OK | R_OK) == 0) return 1;
-next:
-        if (p) env = p + 1;
-        else break;
-     }
-   path[0] = '\0';
-   return 0;
-}
-
-/* maximum number of arguments added above */
-#define VALGRIND_MAX_ARGS 11
-/* bitmask with all supported bits set */
-#define VALGRIND_MODE_ALL 15
-
-static int
-valgrind_append(char **dst, int valgrind_gdbserver, int valgrind_mode, int valgrind_tool, char *valgrind_path, const char *valgrind_log)
-{
-   int i = 0;
-
-   if (valgrind_tool)
-     {
-        dst[i++] = valgrind_path;
-        switch (valgrind_tool)
-          {
-           case 1: dst[i++] = "--tool=massif"; break;
-
-           case 2: dst[i++] = "--tool=callgrind"; break;
-          }
-        return i;
-     }
-   if (valgrind_gdbserver) dst[i++] = "--db-attach=yes";
-   if (!valgrind_mode) return 0;
-   dst[i++] = valgrind_path;
-   dst[i++] = "--num-callers=40";
-   dst[i++] = "--track-origins=yes";
-   dst[i++] = "--malloc-fill=13"; /* invalid pointer, make it crash */
-   if (valgrind_log)
-     {
-        static char logparam[PATH_MAX + sizeof("--log-file=")];
-
-        snprintf(logparam, sizeof(logparam), "--log-file=%s", valgrind_log);
-        dst[i++] = logparam;
-     }
-   if (valgrind_mode & 2) dst[i++] = "--trace-children=yes";
-   if (valgrind_mode & 4)
-     {
-        dst[i++] = "--leak-check=full";
-        dst[i++] = "--leak-resolution=high";
-        dst[i++] = "--track-fds=yes";
-     }
-   if (valgrind_mode & 8) dst[i++] = "--show-reachable=yes";
-   return i;
-}
-
 static void
 copy_args(char **dst, char **src, size_t count)
 {
-   for (; count > 0; count--, dst++, src++)
-     *dst = *src;
+   for (; count > 0; count--, dst++, src++) *dst = *src;
 }
 
 static void
@@ -269,7 +216,7 @@ _sigusr1(int x EINA_UNUSED, siginfo_t *info EINA_UNUSED, void *data EINA_UNUSED)
 {
    struct sigaction action;
 
-   /* release ptrace */
+   // release ptrace
    stop_ptrace = EINA_TRUE;
 
    action.sa_sigaction = _sigusr1;
@@ -307,24 +254,9 @@ _sighup(int x EINA_UNUSED, siginfo_t *info EINA_UNUSED, void *data EINA_UNUSED)
 static void
 _print_usage(const char *hstr)
 {
-   printf("Options:\n"
-          "\t-valgrind[=MODE]\n"
-          "\t\tRun enlightenment from inside valgrind, mode is OR of:\n"
-          "\t\t   1 = plain valgrind to catch crashes (default)\n"
-          "\t\t   2 = trace children (thumbnailer, efm slaves, ...)\n"
-          "\t\t   4 = check leak\n"
-          "\t\t   8 = show reachable after processes finish.\n"
-          "\t\t all = all of above\n"
-          "\t-massif\n"
-          "\t\tRun enlightenment from inside massif valgrind tool.\n"
-          "\t-callgrind\n"
-          "\t\tRun enlightenment from inside callgrind valgrind tool.\n"
-          "\t-valgrind-log-file=<FILENAME>\n"
-          "\t\tSave valgrind log to file, see valgrind's --log-file for details.\n"
-          "\n"
-          "Please run:\n"
+   printf("Please run:\n"
           "\tenlightenment %s\n"
-          "for more options.\n",
+          "\tfor more options.\n",
           hstr);
    exit(0);
 }
@@ -332,10 +264,10 @@ _print_usage(const char *hstr)
 static Eina_Bool
 _sig_continue(siginfo_t sig)
 {
-   return (sig.si_signo != SIGSEGV &&
-           sig.si_signo != SIGFPE &&
-//         sig.si_signo != SIGBUS &&
-           sig.si_signo != SIGABRT);
+   return ((sig.si_signo != SIGSEGV) &&
+           (sig.si_signo != SIGFPE) &&
+//         (sig.si_signo != SIGBUS) &&
+           (sig.si_signo != SIGABRT));
 }
 
 static void
@@ -345,8 +277,7 @@ _sig_remember(siginfo_t sig, Eina_Bool *susr1, Eina_Bool *sill)
      {
         if (*sill) *susr1 = EINA_TRUE;
      }
-   else
-     *sill = (sig.si_signo == SIGILL);
+   else *sill = (sig.si_signo == SIGILL);
 }
 
 static int
@@ -355,14 +286,12 @@ _e_ptrace_attach(int child, int *status, Eina_Bool really_know)
    int result = 0;
 
 #ifdef HAVE_SYS_PTRACE_H
-   if (really_know)
-     return waitpid(child, status, 0);
+   if (really_know) return waitpid(child, status, 0);
 
    ptrace(PT_ATTACH, child, NULL, 0);
    result = waitpid(child, status, 0);
 
-   if (!stop_ptrace && WIFSTOPPED(*status))
-     ptrace(PT_CONTINUE, child, NULL, 0);
+   if (!stop_ptrace && WIFSTOPPED(*status)) ptrace(PT_CONTINUE, child, NULL, 0);
 #else
    (void)child;
    (void)really_know;
@@ -376,8 +305,7 @@ static void
 _e_ptrace_detach(int child, int back, Eina_Bool really_know)
 {
 #ifdef HAVE_SYS_PTRACE_H
-   if (!really_know)
-     ptrace(PT_DETACH, child, NULL, back);
+   if (!really_know) ptrace(PT_DETACH, child, NULL, back);
 #else
    (void)child;
    (void)back;
@@ -389,8 +317,7 @@ static void
 _e_ptrace_traceme(Eina_Bool really_know)
 {
 #ifdef HAVE_SYS_PTRACE_H
-   if (!really_know)
-     ptrace(PT_TRACE_ME, 0, NULL, 0);
+   if (!really_know) ptrace(PT_TRACE_ME, 0, NULL, 0);
 #else
    (void)really_know;
 #endif
@@ -401,8 +328,7 @@ _e_ptrace_getsiginfo(int child, siginfo_t *sig, Eina_Bool really_know)
 {
    memset(sig, 0, sizeof(siginfo_t));
 #ifdef HAVE_SYS_PTRACE_H
-   if (!really_know)
-     return ptrace(PT_GETSIGINFO, child, NULL, sig);
+   if (!really_know) return ptrace(PT_GETSIGINFO, child, NULL, sig);
 #else
    (void)child;
    (void)sig;
@@ -415,8 +341,7 @@ static void
 _e_ptrace_continue(int child, int back, Eina_Bool really_know)
 {
 #ifdef HAVE_SYS_PTRACE_H
-   if (!really_know)
-     ptrace(PT_CONTINUE, child, NULL, back);
+   if (!really_know) ptrace(PT_CONTINUE, child, NULL, back);
 #else
    (void)child;
    (void)back;
@@ -424,29 +349,74 @@ _e_ptrace_continue(int child, int back, Eina_Bool really_know)
 #endif
 }
 
-static int
-_e_start_child(char **args, Eina_Bool really_know)
+static void
+_e_start_stdout_err_redir(const char *home)
 {
+   int logfd;
+   char *logf = NULL, *logf_old = NULL;
+
+   // rename old olg file
+   myasprintf(&logf, "%s/.e-log.log", home);
+   myasprintf(&logf_old, "%s/.e-log.log.old", home);
+   rename(logf, logf_old);
+   // open new log file and move stdout/err to it
+   logfd = open(logf, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+   printf("Enlightenment: See logs in: %s\n", logf);
+   if (logfd < 0) return;
+   dup2(logfd, 1); // stdout to file
+   dup2(logfd, 2); // stderr to file
+}
+
+static int
+_e_start_child(const char *home, char **args, Eina_Bool really_know)
+{
+   // have e process die with parent enlightenment_start
+#ifdef HAVE_PRCTL
+   prctl(PR_SET_PDEATHSIG, SIGTERM);
+#elif defined(HAVE_PROCCTL)
+   int sig = SIGTERM;
+   procctl(P_PID, 0, PROC_PDEATHSIG_CTL, &sig);
+#endif
+   _e_start_stdout_err_redir(home);
    _e_ptrace_traceme(really_know);
    execv(args[0], args);
-   /* We failed, 0 mean normal exit from E with no restart or crash so let's exit */
+   // We failed, 0 means normal exit from E with no restart or crash so
+   // let's exit
    return 0;
 }
 
 static Eina_Bool
-_e_ptrace_kernel_check()
+_e_ptrace_kernel_check(const char *home)
 {
 #ifdef __linux__
-   /* Check if patch to prevent ptrace to another process is present in the kernel. */
+   // Check if patch to prevent ptrace to another process is present
+   // in the kernel
    Eina_Bool ret = EINA_FALSE;
    int fd = open("/proc/sys/kernel/yama/ptrace_scope", O_RDONLY);
    if (fd != -1)
-    {
-       char c;
-       ret = (read(fd, &c, sizeof (c)) == sizeof (c) && c != '0');
-       close(fd);
-    }
-    return ret;
+     {
+        char c;
+        ret = read(fd, &c, (sizeof(c)) == sizeof(c)) && (c != '0');
+        close(fd);
+     }
+   if (ret)
+     {
+        char *buf = NULL;
+        FILE *f;
+
+        myasprintf(&buf, "%s/.e-crashdump.txt", home);
+        f = fopen(buf, "a");
+        if (f)
+          {
+             fprintf(f,
+                     "ERROR: /proc/sys/kernel/yama/ptrace_scope is 1 disallowing remote\n"
+                     "attachment to a process. This means a gdb backtrace cannot be logged.\n"
+                     "To fix this, as root please do:\n"
+                     "  echo 0 > /proc/sys/kernel/yama/ptrace_scope\n");
+             fclose(f);
+          }
+     }
+   return ret;
 #else
    return EINA_FALSE;
 #endif
@@ -456,27 +426,22 @@ static int
 _e_call_gdb(int child, const char *home, char **backtrace_str)
 {
    int r = 0;
-   char buf[4096];
+   char *buf = NULL;
    /* call e_sys gdb */
-   snprintf(buf, sizeof(buf),
-            "gdb "
-            "--pid=%i "
-            "-batch "
-            "-ex 'set logging file %s/.e-crashdump.txt' "
-            "-ex 'set logging on' "
-            "-ex 'thread apply all backtrace full' "
-            "-ex detach > /dev/null 2>&1 < /dev/zero",
-            child,
-            home);
+   myasprintf(&buf,
+              "gdb "
+              "--pid=%i "
+              "-batch "
+              "-ex 'set logging file %s/.e-crashdump.txt' "
+              "-ex 'set logging on' "
+              "-ex 'thread apply all backtrace full' "
+              "-ex detach > /dev/null 2>&1 < /dev/zero",
+              child,
+              home);
    r = system(buf);
 
-   fprintf(stderr, "called gdb with '%s' = %i\n",
-           buf, WEXITSTATUS(r));
-
-   snprintf(buf, 4096,
-            "%s/.e-crashdump.txt",
-            home);
-
+   fprintf(stderr, "called gdb with '%s' = %i\n", buf, WEXITSTATUS(r));
+   myasprintf(&buf, "%s/.e-crashdump.txt", home);
    *backtrace_str = strdup(buf);
    return WEXITSTATUS(r);
 }
@@ -485,17 +450,17 @@ static int
 _e_call_alert(int child, siginfo_t sig, int exit_gdb, const char *backtrace_str,
               Eina_Bool susr1)
 {
-   char buf[4096];
+   char *buf = NULL;
 
-   snprintf(buf, sizeof(buf),
-            backtrace_str ?
-            "%s/enlightenment/utils/enlightenment_alert %i %i %i '%s'" :
-            "%s/enlightenment/utils/enlightenment_alert %i %i %i",
-            eina_prefix_lib_get(pfx),
-            (sig.si_signo == SIGSEGV && susr1) ? SIGILL : sig.si_signo,
-            child,
-            exit_gdb,
-            backtrace_str);
+   myasprintf(&buf,
+              backtrace_str ?
+              "%s/enlightenment/utils/enlightenment_alert %i %i %i '%s'" :
+              "%s/enlightenment/utils/enlightenment_alert %i %i %i",
+              eina_prefix_lib_get(pfx),
+              (sig.si_signo == SIGSEGV && susr1) ? SIGILL : sig.si_signo,
+              child,
+              exit_gdb,
+              backtrace_str);
    return system(buf);
 }
 
@@ -549,42 +514,14 @@ done:
    return ret;
 }
 
-/*
-static void
-rmrf(const char *path)
-{
-   Eina_Iterator *iter = eina_file_direct_ls(path);
-
-   if (iter)
-     {
-        Eina_File_Direct_Info *info;
-
-        EINA_ITERATOR_FOREACH(iter, info)
-          {
-             if (info->type == EINA_FILE_DIR) rmrf(info->path);
-             else eina_file_unlink(info->path);
-          }
-        eina_iterator_free(iter);
-     }
-   eina_file_unlink(path);
-}
-*/
-
 int
 main(int argc, char **argv)
 {
-   int i, valgrind_mode = 0;
-   int valgrind_tool = 0;
-   int valgrind_gdbserver = 0;
-   char buf[8192], buf2[4096], buf3[4096], **args, *home;
-   char valgrind_path[PATH_MAX] = "";
-   const char *valgrind_log = NULL;
+   int i, ret = -1;
+   char *buf = NULL, *buf2 = NULL, *buf3 = NULL, **args, *home;
    const char *bindir;
    Eina_Bool really_know = EINA_FALSE;
    struct sigaction action;
-//   struct stat st;
-//   const char *s;
-   int ret = -1;
    pid_t child = -1;
    Eina_Bool restart = EINA_TRUE;
 
@@ -608,26 +545,6 @@ main(int argc, char **argv)
    sigemptyset(&action.sa_mask);
    sigaction(SIGHUP, &action, NULL);
 
-/* leave XDG_RUNTIME_DIR alone - if distro/os doesn't use it - too bad
-   s = getenv("XDG_RUNTIME_DIR");
-   if ((!s) || (stat(s, &st) != 0) || (!S_ISDIR(st.st_mode)))
-     {
-        const char *dir;
-
-        snprintf(buf, sizeof(buf), "/tmp/xdg-XXXXXX");
-        dir = mkdtemp(buf);
-        if (!dir) dir = "/tmp";
-        else
-          {
-             FILE *f;
-
-             snprintf(buf2, sizeof(buf2), "%s/.e-deleteme", dir);
-             f = fopen(buf2, "w");
-             if (f) fclose(f);
-          }
-        env_set("XDG_RUNTIME_DIR", dir);
-     }
- */
    eina_init();
 
    /* reexcute myself with dbus-launch if dbus-launch is not running yet */
@@ -656,43 +573,11 @@ main(int argc, char **argv)
           {
              _print_usage(argv[i]);
           }
-        if (!strcmp(argv[i], "-valgrind-gdb"))
-          valgrind_gdbserver = 1;
-        else if (!strcmp(argv[i], "-massif"))
-          valgrind_tool = 1;
-        else if (!strcmp(argv[i], "-callgrind"))
-          valgrind_tool = 2;
         else if (!strcmp(argv[i], "-display"))
           {
              i++;
              env_set("DISPLAY", argv[i]);
           }
-        else if (!strncmp(argv[i], "-valgrind", sizeof("-valgrind") - 1))
-          {
-             const char *val = argv[i] + sizeof("-valgrind") - 1;
-             switch (*val)
-               {
-                case '\0':
-                  valgrind_mode = 1;
-                  break;
-                case '=':
-                  val++;
-                  if (!strcmp(val, "all")) valgrind_mode = VALGRIND_MODE_ALL;
-                  else valgrind_mode = atoi(val);
-                  break;
-                case '-':
-                  val++;
-                  if (!strncmp(val, "log-file=", sizeof("log-file=") - 1))
-                    {
-                       valgrind_log = val + sizeof("log-file=") - 1;
-                       if (*valgrind_log == '\0') valgrind_log = NULL;
-                    }
-                  break;
-                default:
-                  printf("Unknown valgrind option: %s\n", argv[i]);
-                  break;
-               }
-          }
         else if (!strcmp(argv[i], "-i-really-know-what-i-am-doing-and-accept"
                                   "-full-responsibility-for-it"))
           {
@@ -706,63 +591,40 @@ main(int argc, char **argv)
         if (really_know) _env_path_append("PATH", bindir);
         else _env_path_prepend("PATH", bindir);
      }
-   snprintf(buf2, sizeof(buf2),
-            "E_ALERT_FONT_DIR=%s/data/fonts", eina_prefix_data_get(pfx));
+   myasprintf(&buf2,
+              "E_ALERT_FONT_DIR=%s/data/fonts",
+              eina_prefix_data_get(pfx));
    putenv(buf2);
-   snprintf(buf3, sizeof(buf3),
-            "E_ALERT_SYSTEM_BIN=%s/enlightenment/utils/enlightenment_system", eina_prefix_lib_get(pfx));
+   myasprintf(&buf3,
+              "E_ALERT_SYSTEM_BIN=%s/enlightenment/utils/enlightenment_system",
+              eina_prefix_lib_get(pfx));
    putenv(buf3);
 
-   if ((valgrind_mode || valgrind_tool) &&
-       !find_valgrind(valgrind_path, sizeof(valgrind_path)))
-     {
-        printf("E - valgrind required but no binary found! Ignoring request.\n");
-        valgrind_mode = 0;
-     }
-
-   printf("E - PID=%i, valgrind=%d", getpid(), valgrind_mode);
-   if (valgrind_mode)
-     {
-        printf(" valgrind-command='%s'", valgrind_path);
-        if (valgrind_log)
-          printf(" valgrind-log-file='%s'", valgrind_log);
-     }
-   putchar('\n');
-
    home = getenv("HOME");
    if (home)
      {
         const char *tmps = getenv("XDG_DATA_HOME");
 
-        if (tmps)
-          snprintf(buf, sizeof(buf), "%s/Applications/.bin", tmps);
-        else
-          snprintf(buf, sizeof(buf), "%s/Applications/.bin", home);
-
-        if (really_know)
-          _env_path_append("PATH", buf);
-        else
-          _env_path_prepend("PATH", buf);
+        if (tmps)        myasprintf(&buf, "%s/Applications/.bin", tmps);
+        else             myasprintf(&buf, "%s/Applications/.bin", home);
+        if (really_know) _env_path_append("PATH", buf);
+        else             _env_path_prepend("PATH", buf);
      }
 
    /* run e directly now */
-   snprintf(buf, sizeof(buf), "%s/enlightenment", eina_prefix_bin_get(pfx));
+   myasprintf(&buf, "%s/enlightenment", eina_prefix_bin_get(pfx));
 
-   args = alloca((argc + 2 + VALGRIND_MAX_ARGS) * sizeof(char *));
-   i = valgrind_append(args, valgrind_gdbserver, valgrind_mode, valgrind_tool,
-                       valgrind_path, valgrind_log);
-   args[i++] = buf;
-   copy_args(args + i, argv + 1, argc - 1);
-   args[i + argc - 1] = NULL;
-
-   if (valgrind_tool || valgrind_mode)
-     really_know = EINA_TRUE;
+   args = alloca((argc + 1) * sizeof(char *));
+   printf("Enlightenment: Command: %s\n", buf);
+   args[0] = buf;
+   copy_args(&args[1], argv + 1, argc - 1);
+   args[argc] = NULL;
 
    /* Now looping until */
    while (restart)
      {
         pid_t result;
-        int status;
+        int status, back;
         Eina_Bool done = EINA_FALSE;
         Eina_Bool remember_sigill = EINA_FALSE;
         Eina_Bool remember_sigusr1 = EINA_FALSE;
@@ -778,7 +640,7 @@ main(int argc, char **argv)
           }
         else if (child == 0)
           { // we are in the child fork - so exec
-             ret = _e_start_child(args, really_know);
+             ret = _e_start_child(home, args, really_know);
              break;
           }
 
@@ -789,15 +651,15 @@ main(int argc, char **argv)
 
         /* now loop until done */
 not_done:
+#ifdef HAVE_MALLOC_TRIM
+        malloc_trim(0);
+#endif
         result = waitpid(child, &status, WNOHANG);
         /* Wait for E */
         if (!result) result = waitpid(-1, &status, 0);
 
         if (result == child)
           {
-             printf("WIFSTOPPED=%i WIFEXITED=%i WEXITSTATUS=%i stop_ptrace=%i\n",
-                    WIFSTOPPED(status), WIFEXITED(status),
-                    WEXITSTATUS(status), stop_ptrace);
              if (WIFSTOPPED(status) && (!stop_ptrace))
                {
                   char *backtrace_str = NULL;
@@ -806,20 +668,21 @@ not_done:
                   int r = _e_ptrace_getsiginfo(child, &sig,
                                                really_know);
 
-                  int back = (r == 0 && sig.si_signo != SIGTRAP)
-                              ? sig.si_signo : 0;
+                  back = ((r == 0) && (sig.si_signo != SIGTRAP))
+                          ? sig.si_signo : 0;
 
                   _sig_remember(sig, &remember_sigusr1, &remember_sigill);
 
-                  if (r != 0 || _sig_continue(sig))
+                  if ((r != 0) || (_sig_continue(sig)))
                     {
                        _e_ptrace_continue(child, back, really_know);
                        goto not_done;
                     }
                   _e_ptrace_detach(child, back, really_know);
+                  usleep(200000);
 
                   /* And call gdb if available */
-                  if (home && !_e_ptrace_kernel_check())
+                  if (home && !_e_ptrace_kernel_check(home))
                     r = _e_call_gdb(child, home, &backtrace_str);
                   else
                     r = 0;
@@ -837,13 +700,20 @@ not_done:
                   free(backtrace_str);
 
                   if (getenv("DISPLAY")) kill(child, SIGKILL);
-                  if (WEXITSTATUS(r) == 1) restart = EINA_FALSE;
+                  if (WEXITSTATUS(r) == 101) restart = EINA_FALSE;
+                  done = EINA_TRUE;
                }
              else if (!WIFEXITED(status) || (stop_ptrace))
                {
                   restart = EINA_TRUE;
                   done = EINA_TRUE;
                }
+             else if (WEXITSTATUS(status) == 121)
+               {
+                  putenv("E_RESTART_OK=1");
+                  restart = EINA_TRUE;
+                  done = EINA_TRUE;
+               }
              else if (WEXITSTATUS(status) == 111)
                {
                   putenv("E_RESTART_OK=1");
@@ -852,7 +722,7 @@ not_done:
                }
              else if (WEXITSTATUS(status) == 101)
                {
-                  printf("Explici error exit from enlightenment\n");
+                  printf("Explicit error exit from enlightenment\n");
                   restart = EINA_FALSE;
                   done = EINA_TRUE;
                }
@@ -884,15 +754,6 @@ not_done:
           }
         if (!done) goto not_done;
      }
-   // clean up xdg runtime_dir if we created it
-/* leave XDG_RUNTIME_DIR alone - if distro/os doesn't use it - too bad
-   s = getenv("XDG_RUNTIME_DIR");
-   if ((s) && (stat(s, &st) == 0) && (S_ISDIR(st.st_mode)))
-     {
-        snprintf(buf, sizeof(buf), "%s/.e-deleteme", s);
-        if (stat(buf, &st) == 0) rmrf(s);
-     }
- */
    return ret;
 }
 
