@@ -62,7 +62,7 @@ e_util_glob_match(const char *str, const char *pattern)
      }
    if (str == pattern) return 1;
    if (!strcmp(pattern, "*")) return 1;
-   if (!fnmatch(pattern, str, 0)) return 1;
+   if (eina_fnmatch(pattern, str, 0)) return 1;
    return 0;
 }
 
@@ -87,7 +87,7 @@ e_util_glob_case_match(const char *str, const char *pattern)
    for (tp = tglob, p = pattern; *p != 0; p++, tp++)
      *tp = tolower(*p);
    *tp = 0;
-   if (!fnmatch(tglob, tstr, 0)) return 1;
+   if (eina_fnmatch(tglob, tstr, 0)) return 1;
    return 0;
 }
 
@@ -584,41 +584,54 @@ e_util_file_time_get(time_t ftime)
    ltime = time(NULL);
    diff = ltime - ftime;
    buf[0] = 0;
-   if (ftime > ltime)
-     snprintf(buf, sizeof(buf), _("In the future"));
+   if (e_config->explicit_date_time)
+     {
+        struct tm tm;
+
+        if (localtime_r(&ftime, &tm))
+          {
+             s = eina_strftime("%F %T", &tm);
+             if (s) return s;
+          }
+     }
    else
      {
-        if (diff <= 60)
-          snprintf(buf, sizeof(buf), _("In the last minute"));
-        else if (diff >= 31526000)
+        if (ftime > ltime)
+          snprintf(buf, sizeof(buf), _("In the future"));
+        else
           {
-             test = diff / 31526000;
-             snprintf(buf, sizeof(buf), P_("Last year", "%li Years ago", test), test);
-          }
-        else if (diff >= 2592000)
-          {
-             test = diff / 2592000;
-             snprintf(buf, sizeof(buf), P_("Last month", "%li Months ago", test), test);
-          }
-        else if (diff >= 604800)
-          {
-             test = diff / 604800;
-             snprintf(buf, sizeof(buf), P_("Last week", "%li Weeks ago", test), test);
-          }
-        else if (diff >= 86400)
-          {
-             test = diff / 86400;
-             snprintf(buf, sizeof(buf), P_("Yesterday", "%li Days ago", test), test);
-          }
-        else if (diff >= 3600)
-          {
-             test = diff / 3600;
-             snprintf(buf, sizeof(buf), P_("An hour ago", "%li Hours ago", test), test);
-          }
-        else if (diff > 60)
-          {
-             test = diff / 60;
-             snprintf(buf, sizeof(buf), P_("A minute ago", "%li Minutes ago", test), test);
+             if (diff <= 60)
+               snprintf(buf, sizeof(buf), _("In the last minute"));
+             else if (diff >= 31526000)
+               {
+                  test = diff / 31526000;
+                  snprintf(buf, sizeof(buf), P_("Last year", "%li Years ago", test), test);
+               }
+             else if (diff >= 2592000)
+               {
+                  test = diff / 2592000;
+                  snprintf(buf, sizeof(buf), P_("Last month", "%li Months ago", test), test);
+               }
+             else if (diff >= 604800)
+               {
+                  test = diff / 604800;
+                  snprintf(buf, sizeof(buf), P_("Last week", "%li Weeks ago", test), test);
+               }
+             else if (diff >= 86400)
+               {
+                  test = diff / 86400;
+                  snprintf(buf, sizeof(buf), P_("Yesterday", "%li Days ago", test), test);
+               }
+             else if (diff >= 3600)
+               {
+                  test = diff / 3600;
+                  snprintf(buf, sizeof(buf), P_("An hour ago", "%li Hours ago", test), test);
+               }
+             else if (diff > 60)
+               {
+                  test = diff / 60;
+                  snprintf(buf, sizeof(buf), P_("A minute ago", "%li Minutes ago", test), test);
+               }
           }
      }
 
@@ -714,31 +727,56 @@ e_util_dir_check(const char *dir)
 static Eina_Array *_delay_del_array = NULL;
 static Ecore_Idle_Enterer *_delay_del_idler = NULL;
 
+static Eina_Bool
+_e_util_del_array_contains(E_Object *obj)
+{
+   unsigned long c, i;
+
+   if (!_delay_del_array) return EINA_FALSE;
+
+   c = eina_array_count_get(_delay_del_array);
+   for (i = 0; i < c; i++)
+     {
+        if (eina_array_data_get(_delay_del_array, i) == obj) return EINA_TRUE;
+     }
+   return EINA_FALSE;
+}
+
 static Eina_Bool
 _e_util_cb_delayed_del(void *data EINA_UNUSED)
 {
+   _delay_del_idler = NULL;
+
    while (_delay_del_array)
      {
         Eina_Array *arr = _delay_del_array;
-        Eina_Iterator *itr = eina_array_iterator_new(arr);
-        void *ptr;
+        unsigned long c, i;
 
         _delay_del_array = NULL;
-        while (eina_iterator_next(itr, &ptr))
+        c = eina_array_count_get(arr);
+        for (i = 0; i < c; i++)
           {
-             if (ptr) e_object_del(E_OBJECT(ptr));
+             E_Object *obj = eina_array_data_get(arr, i);
+             if (obj) e_object_del(obj);
           }
         eina_array_free(arr);
      }
-   _delay_del_idler = NULL;
    return ECORE_CALLBACK_CANCEL;
 }
 
 static void
-_e_util_cb_delayed_cancel(void *data, void *obj EINA_UNUSED)
+_e_util_cb_delayed_cancel(void *data EINA_UNUSED, void *obj)
 {
-   unsigned long c = (unsigned long)data;
-   if (_delay_del_array) eina_array_data_set(_delay_del_array, c, NULL);
+   unsigned long c, i;
+
+   if (!_delay_del_array) return;
+
+   c = eina_array_count_get(_delay_del_array);
+   for (i = 0; i < c; i++)
+     {
+        if (eina_array_data_get(_delay_del_array, i) == obj)
+          eina_array_data_set(_delay_del_array, i, NULL);
+     }
 }
 
 E_API void
@@ -748,19 +786,19 @@ e_util_defer_object_del(E_Object *obj)
      e_object_del(obj);
    else
      {
-        if (!_delay_del_array)
+        if (!_e_util_del_array_contains(obj))
           {
-             _delay_del_array = eina_array_new(8);
-             if (!_delay_del_idler)
-               _delay_del_idler = ecore_idle_enterer_before_add
-                 (_e_util_cb_delayed_del, NULL);
-          }
-        if (_delay_del_array)
-          {
-             if (eina_array_push(_delay_del_array, obj))
+             if (!_delay_del_array)
                {
-                  unsigned long c = eina_array_count_get(_delay_del_array);
-                  e_object_delfn_add(obj, _e_util_cb_delayed_cancel, (void *)c);
+                  _delay_del_array = eina_array_new(8);
+                  if (!_delay_del_idler)
+                    _delay_del_idler = ecore_idle_enterer_before_add
+                      (_e_util_cb_delayed_del, NULL);
+               }
+             if (_delay_del_array)
+               {
+                  if (eina_array_push(_delay_del_array, obj))
+                    e_object_delfn_add(obj, _e_util_cb_delayed_cancel, NULL);
                }
           }
      }
@@ -1544,3 +1582,12 @@ e_util_key_geometry_action_get(const char *key, int *x, int *y, int dx, int dy)
 
    return E_UTIL_ACTION_DO;
 }
+
+E_API const char *
+e_username_get(void)
+{
+   struct passwd *pw = getpwuid(getuid());
+
+   if (pw) return pw->pw_name;
+   return "";
+}
