@@ -21,8 +21,9 @@ _light_set(Light *lig, int val)
    lig->val = nval;
 #ifdef HAVE_EEZE
    char buf[PATH_MAX];
+   int fd;
    snprintf(buf, sizeof(buf), "%s/brightness", lig->dev);
-   int fd = open(buf, O_WRONLY);
+   fd = open(buf, O_WRONLY);
    if (fd >= 0)
      {
         char buf2[32];
@@ -31,9 +32,31 @@ _light_set(Light *lig, int val)
           ERR("Write failed of [%s] to [%s]\n", buf2, buf);
         close(fd);
      }
+   snprintf(buf, sizeof(buf), "%s/bl_power", lig->dev);
+   fd = open(buf, O_WRONLY);
+   if (fd >= 0)
+     {
+        int ret;
+
+        if (val == 0) ret = write(fd, "4", 1);
+        else ret = write(fd, "0", 1);
+        if (ret < 0) ERR("Write failed to [%s]\n", buf);
+        close(fd);
+     }
 #elif defined(__FreeBSD_kernel__)
+#if __FreeBSD__ > 12
+   int fd = open("/dev/backlight/backlight0", O_RDWR);
+   if (fd >= 0)
+     {
+        struct backlight_props props;
+        props.brightness = lig->val;
+        ioctl(fd, BACKLIGHTUPDATESTATUS, &props);
+        close(fd);
+     }
+#else
    sysctlbyname(lig->dev, NULL, NULL, &(lig->val), sizeof(lig->val));
 #endif
+#endif
 }
 
 static void
@@ -41,16 +64,41 @@ _light_get(Light *lig)
 {
 #ifdef HAVE_EEZE
    const char *s;
-   s = eeze_udev_syspath_get_sysattr(lig->dev, "brightness");
+   s = eeze_udev_syspath_get_sysattr(lig->dev, "bl_power");
+   if (s)
+     {
+        if (atoi(s) != 0)
+          {
+             lig->val = 0;
+             eina_stringshare_del(s);
+             return;
+          }
+        eina_stringshare_del(s);
+     }
+   s = eeze_udev_syspath_get_sysattr(lig->dev, "actual_brightness");
+   if (!s) s = eeze_udev_syspath_get_sysattr(lig->dev, "brightness");
    if (s)
      {
         lig->val = atoi(s);
         eina_stringshare_del(s);
      }
 #elif defined(__FreeBSD_kernel__)
+#if __FreeBSD__ > 12
+   struct backlight_props props;
+   int fd = open("/dev/backlight/backlight0", O_RDWR);
+   if (fd >= 0)
+     {
+        if (ioctl(fd, BACKLIGHTGETSTATUS, &props) != -1)
+          {
+             lig->val = props.brightness;
+          }
+        close(fd);
+     }
+#else
    size_t plen = sizeof(lig->val);
    sysctlbyname(lig->dev, &(lig->val), &plen, NULL, 0);
 #endif
+#endif
 }
 
 static void
@@ -105,7 +153,7 @@ _cb_worker_message(void *data EINA_UNUSED, Ecore_Thread *th EINA_UNUSED, void *m
 
    if (lig->max > 0)
      {
-        val = ((1000 * lig->val) + 500) / lig->max;
+        val = (1000 * lig->val) / lig->max;
         if (val < 0) val = 0;
         else if (val > 1000) val = 1000;
      }
@@ -154,9 +202,22 @@ _light_add(const char *dev)
      }
    if (lig->max <= 0) lig->max = 255;
    lig->prefer = eeze_udev_syspath_check_sysattr(lig->dev, "type", "firmware");
-#elif defined(__FreeBSD_kernel__)
+#elif defined (__FreeBSD_kernel__)
+#if __FreeBSD__ > 12
+   struct backlight_props props;
+   int fd = open("/dev/backlight/backlight0", O_RDWR);
+   if (fd >= 0)
+     {
+        if (ioctl(fd, BACKLIGHTGETSTATUS, &props) != -1)
+          {
+             lig->val = props.brightness;
+          }
+        close(fd);
+     }
+# else
    size_t plen = sizeof(lig->val);
    sysctlbyname(lig->dev, &(lig->val), &plen, NULL, 0);
+#endif
    lig->max = 100;
 #endif
    _devices = eina_list_append(_devices, lig);
@@ -214,8 +275,12 @@ _light_refresh_devices()
      }
 #elif defined(__FreeBSD_kernel__)
    // XXX; shouldn't we scan for devices?
+#if __FreeBSD__ > 12
+   _light_add("backlight0");
+#else
    _light_add("hw.acpi.video.lcd0.brightness");
 #endif
+#endif
 }
 
 static Light *
@@ -304,7 +369,7 @@ _cb_bklight_set(void *data EINA_UNUSED, const char *params)
    if (!params) return;
    if (sscanf(params, "%1023s %i", dev, &val) != 2) return;
    eina_lock_take(&_devices_lock);
-   fprintf(stderr, "BL: set [%s] -> %i\n", dev, val);
+//   fprintf(stderr, "BL: set [%s] -> %i\n", dev, val);
    lig = _light_find(dev);
    if (!lig) goto done;
    lig->val_set = val;
