@@ -24,19 +24,42 @@ static const E_Gadcon_Client_Class _gadcon_class =
    E_GADCON_CLIENT_STYLE_PLAIN
 };
 
+#define MAX_UNITS 8
+
 /* actual module specifics */
 typedef struct _Instance Instance;
 
+typedef struct __Popup_Widgets
+{
+   Evas_Object *pb_usage;
+   Evas_Object *state;
+   Evas_Object *remaining;
+   Evas_Object *health;
+   Evas_Object *technology;
+} _Popup_Widgets;
+
+typedef struct __Popup_Data
+{
+   Instance        *inst;
+   unsigned int     n_units;
+   _Popup_Widgets   widgets[MAX_UNITS];
+} _Popup_Data;
+
 struct _Instance
 {
    E_Gadcon_Client *gcc;
    Evas_Object     *o_battery;
+
+   E_Gadcon_Popup  *popup;
+   Ecore_Timer     *popup_timer;
+   _Popup_Data     *popup_data;
+
    Evas_Object     *popup_battery;
    E_Gadcon_Popup  *warning;
    unsigned int     notification_id;
 };
 
-static void      _battery_update(int full, int time_left, int time_full, Eina_Bool have_battery, Eina_Bool have_power);
+static void      _battery_update(int full, int time_left, int time_full, Eina_Bool have_battery, Eina_Bool have_power, Eina_Bool charging);
 static Eina_Bool _battery_cb_exe_data(void *data, int type, void *event);
 static Eina_Bool _battery_cb_exe_del(void *data, int type, void *event);
 static void      _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
@@ -50,6 +73,9 @@ static void      _battery_cb_warning_popup_hide(void *data, Evas *e, Evas_Object
 static void      _battery_warning_popup_destroy(Instance *inst);
 static void      _battery_warning_popup(Instance *inst, int time, double percent);
 
+static void      _battery_popup_usage_destroy(Instance *inst);
+static void      _battery_popup_usage_new(Instance *inst);
+
 static Eina_Bool _powersave_cb_config_update(void *data, int type, void *event);
 
 static E_Config_DD *conf_edd = NULL;
@@ -108,6 +134,7 @@ _gc_shutdown(E_Gadcon_Client *gcc)
         e_object_del(E_OBJECT(inst->warning));
         inst->popup_battery = NULL;
      }
+   _battery_popup_usage_destroy(inst);
    E_FREE(inst);
 }
 
@@ -159,6 +186,276 @@ _gc_id_new(const E_Gadcon_Client_Class *client_class)
    return buf;
 }
 
+static void
+_battery_popup_usage_destroy(Instance *inst)
+{
+   if (inst->popup_timer) ecore_timer_del(inst->popup_timer);
+   E_FREE_FUNC(inst->popup, e_object_del);
+   if (inst->popup_data) E_FREE(inst->popup_data);
+   inst->popup = NULL; inst->popup_timer = NULL;
+   inst->popup_data = NULL;
+}
+
+static void
+_battery_popup_usage_destroy_cb(void *obj)
+{
+   _battery_popup_usage_destroy(e_object_data_get(obj));
+}
+
+static Eina_Bool
+_battery_popup_usage_content_update_cb(void *data)
+{
+   Instance *inst;
+   _Popup_Data *pd;
+   Eina_List *l;
+   Battery *bat;
+   int hrs, mins, t = 0;
+   unsigned int i = 0;
+   char buf[256];
+
+   pd = data;
+   inst = pd->inst;
+
+   if (!battery_config->have_battery)
+     {
+        _battery_popup_usage_destroy(inst);
+        return ECORE_CALLBACK_CANCEL;
+     }
+
+   if (!inst->popup) return ECORE_CALLBACK_CANCEL;
+
+   EINA_LIST_FOREACH(device_batteries, l, bat)
+     {
+        _Popup_Widgets *w = &pd->widgets[i++];
+
+        elm_progressbar_value_set(w->pb_usage, (double) bat->percent / 100.0);
+
+        t = bat->time_left;
+        if ((battery_config->have_power) && (battery_config->full < 100))
+          {
+             elm_object_text_set(w->state, _("Charging"));
+          }
+        else if ((!battery_config->have_power) && (battery_config->full < 100))
+          {
+             elm_object_text_set(w->state, _("Discharging"));
+          }
+        else
+          {
+             t = 0;
+             elm_object_text_set(w->state, _("Charged"));
+          }
+
+        hrs = (t / 3600);
+        if (hrs < 0) hrs = 0;
+        mins = (t / 60) - (hrs * 60);
+        if (mins < 0) mins = 0;
+        snprintf(buf, sizeof(buf), "%i:%02i", hrs, mins);
+        elm_object_text_set(w->remaining, buf);
+
+        if ((bat->last_full_charge > 0) && (bat->design_charge > 0))
+          snprintf(buf, sizeof(buf), "%1.1f%%",
+                   100.0 * ((double)bat->last_full_charge / bat->design_charge));
+        else
+          snprintf(buf, sizeof(buf), "???%%");
+        elm_object_text_set(w->health, buf);
+
+        if (bat->technology)
+          elm_object_text_set(w->technology, bat->technology);
+        else
+          elm_object_text_set(w->technology, _("Unknown"));
+        if (i == (pd->n_units - 1)) break;
+     }
+   return ECORE_CALLBACK_RENEW;
+}
+
+static char *
+_text_get(void *data, Evas_Object *obj EINA_UNUSED, const char *part EINA_UNUSED)
+{
+   char *buf;
+   if (strcmp(part, "elm.text")) return NULL;
+   buf = data;
+   return strdup(buf);
+}
+
+static Evas_Object *
+_icon_get(void *data EINA_UNUSED, Evas_Object *obj, const char *part)
+{
+   Evas_Object *ic;
+   if (strcmp(part, "elm.swallow.icon")) return NULL;
+
+   ic = elm_icon_add(obj);
+   elm_icon_standard_set(ic, "battery");
+   evas_object_size_hint_min_set(ic, ELM_SCALE_SIZE(20), ELM_SCALE_SIZE(20));
+   evas_object_show(ic);
+   return ic;
+}
+
+static Evas_Object *
+_content_get(void *data, Evas_Object *obj, const char *part)
+{
+   Evas_Object *tb, *o;
+   _Popup_Widgets *w;
+   if (strcmp(part, "elm.swallow.content")) return NULL;
+
+   w = data;
+
+   tb = o = elm_table_add(obj);
+   elm_table_padding_set(o, ELM_SCALE_SIZE(4), ELM_SCALE_SIZE(4));
+   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
+   evas_object_size_hint_weight_set(o, 1.0, 0);
+
+
+   o = evas_object_rectangle_add(evas_object_evas_get(obj));
+   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
+   evas_object_size_hint_weight_set(o, 1.0, 0);
+   evas_object_size_hint_min_set(o, ELM_SCALE_SIZE(220), ELM_SCALE_SIZE(20));
+   elm_table_pack(tb, o, 0, 0, 8, 1);
+
+   w->pb_usage = o = elm_progressbar_add(obj);
+   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
+   evas_object_size_hint_weight_set(o, 1.0, 0);
+   elm_progressbar_span_size_set(o, 1.0);
+   elm_table_pack(tb, o, 0, 0, 8, 1);
+   evas_object_show(o);
+
+
+   o = elm_icon_add(obj);
+   elm_icon_standard_set(o, "power-plug");
+   evas_object_size_hint_min_set(o, ELM_SCALE_SIZE(20), ELM_SCALE_SIZE(20));
+   elm_table_pack(tb, o, 0, 1, 1, 1);
+   evas_object_show(o);
+
+   w->state = o = elm_label_add(obj);
+   evas_object_size_hint_align_set(o, 0.0, 0.5);
+   evas_object_size_hint_weight_set(o, 1.0, 0);
+   elm_table_pack(tb, o, 1, 1, 1, 1);
+   evas_object_show(o);
+
+
+   o = elm_icon_add(obj);
+   elm_icon_standard_set(o, "clock");
+   evas_object_size_hint_min_set(o, ELM_SCALE_SIZE(20), ELM_SCALE_SIZE(20));
+   elm_table_pack(tb, o, 2, 1, 1, 1);
+   evas_object_show(o);
+
+   w->remaining = o = elm_label_add(obj);
+   evas_object_size_hint_align_set(o, 0.0, 0.5);
+   evas_object_size_hint_weight_set(o, 1.0, 0);
+   elm_table_pack(tb, o, 3, 1, 1, 1);
+   evas_object_show(o);
+
+
+   o = elm_icon_add(obj);
+   elm_icon_standard_set(o, "health");
+   evas_object_size_hint_min_set(o, ELM_SCALE_SIZE(20), ELM_SCALE_SIZE(20));
+   elm_table_pack(tb, o, 4, 1, 1, 1);
+   evas_object_show(o);
+
+   w->health = o = elm_label_add(obj);
+   evas_object_size_hint_align_set(o, 0.0, 0.5);
+   evas_object_size_hint_weight_set(o, 1.0, 0);
+   elm_table_pack(tb, o, 5, 1, 1, 1);
+   evas_object_show(o);
+
+
+   o = elm_icon_add(obj);
+   elm_icon_standard_set(o, "tech");
+   evas_object_size_hint_min_set(o, ELM_SCALE_SIZE(20), ELM_SCALE_SIZE(20));
+   elm_table_pack(tb, o, 6, 1, 1, 1);
+   evas_object_show(o);
+
+   w->technology = o = elm_label_add(obj);
+   evas_object_size_hint_align_set(o, 0.0, 0.5);
+   evas_object_size_hint_weight_set(o, 1.0, 0);
+   elm_table_pack(tb, o, 7, 1, 1, 1);
+   evas_object_show(o);
+
+   return tb;
+}
+
+static void
+_battery_popup_usage_new(Instance *inst)
+{
+   Evas_Object *base, *tb, *rec, *glist;
+   _Popup_Data *pd;
+   static char buf[512];
+   Eina_List *l;
+   Battery *bat;
+   int n, i = 0;
+   Elm_Genlist_Item_Class *itc, *itc2;
+
+   n = eina_list_count(device_batteries);
+   if (!n) return;
+
+   base = e_comp->elm;
+   inst->popup = e_gadcon_popup_new(inst->gcc, 0);
+
+   pd = E_NEW(_Popup_Data, 1);
+   pd->inst = inst;
+   pd->n_units = n;
+   inst->popup_data = pd;
+
+   tb = elm_table_add(base);
+   E_FILL(tb); E_EXPAND(tb);
+   evas_object_show(tb);
+
+   rec = evas_object_rectangle_add(evas_object_evas_get(base));
+   evas_object_size_hint_min_set(rec, ELM_SCALE_SIZE(360), ELM_SCALE_SIZE(160));
+   evas_object_size_hint_max_set(rec, ELM_SCALE_SIZE(560), ELM_SCALE_SIZE(400));
+   elm_table_pack(tb, rec, 0, 0, 1, 1);
+
+   glist = elm_genlist_add(base);
+   E_FILL(glist);
+   E_EXPAND(glist);
+   elm_genlist_select_mode_set(glist, ELM_OBJECT_SELECT_MODE_NONE);
+   elm_table_pack(tb, glist, 0, 0, 1, 1);
+   evas_object_show(glist);
+
+   itc = elm_genlist_item_class_new();
+   itc->item_style = "full";
+   itc->func.text_get = NULL;
+   itc->func.content_get = _content_get;
+   itc->func.filter_get = NULL;
+   itc->func.state_get = NULL;
+   itc->func.del = NULL;
+
+   itc2 = elm_genlist_item_class_new();
+   itc2->item_style = "group_index";
+   itc2->func.text_get = _text_get;
+   itc2->func.content_get = _icon_get;
+   itc2->func.filter_get = NULL;
+   itc2->func.state_get = NULL;
+   itc2->func.del = NULL;
+
+   EINA_LIST_FOREACH(device_batteries, l, bat)
+     {
+        _Popup_Widgets *w = &pd->widgets[i++];
+        if ((bat->vendor) && (bat->model))
+          snprintf(buf, sizeof(buf), _("Battery: %s (%s)"), bat->vendor, bat->model);
+        else if (bat->vendor)
+          snprintf(buf, sizeof(buf), _("Battery: %s"), bat->vendor);
+        else if (bat->model)
+          snprintf(buf, sizeof(buf), _("Battery: %s"), bat->model);
+        else
+          snprintf(buf, sizeof(buf), _("Battery"));
+        elm_genlist_item_append(glist, itc2, buf, NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
+        elm_genlist_item_append(glist, itc, w, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
+     }
+
+   if (pd->n_units)
+     inst->popup_timer = ecore_timer_add(2.0, _battery_popup_usage_content_update_cb, pd);
+
+   elm_genlist_item_class_free(itc);
+   elm_genlist_item_class_free(itc2);
+
+   e_gadcon_popup_content_set(inst->popup, tb);
+   e_gadcon_popup_show(inst->popup);
+   e_object_data_set(E_OBJECT(inst->popup),inst);
+   E_OBJECT_DEL_SET(inst->popup, _battery_popup_usage_destroy_cb);
+
+   _battery_popup_usage_content_update_cb(pd);
+}
+
 static void
 _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 {
@@ -198,7 +495,14 @@ _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
                                  EVAS_BUTTON_NONE, ev->timestamp, NULL);
      }
    if (ev->button == 1)
-     _battery_cb_warning_popup_hide(data, e, obj, event_info);
+     {
+        _battery_cb_warning_popup_hide(data, e, obj, event_info);
+
+        if (!inst->popup)
+          _battery_popup_usage_new(inst);
+        else
+          _battery_popup_usage_destroy(inst);
+     }
 }
 
 static void
@@ -227,6 +531,7 @@ _battery_face_time_set(Evas_Object *battery, int t)
 
    hrs = (t / 3600);
    mins = ((t) / 60 - (hrs * 60));
+   if (hrs < 0) hrs = 0;
    if (mins < 0) mins = 0;
    snprintf(buf, sizeof(buf), "%i:%02i", hrs, mins);
    edje_object_part_text_set(battery, "e.text.time", buf);
@@ -299,7 +604,7 @@ _battery_device_update(void)
 
    EINA_LIST_FOREACH(device_batteries, l, bat)
      {
-        if ((!bat->got_prop) || (!bat->technology))
+        if ((!bat->got_prop)/* || (!bat->technology)*/)
           continue;
         have_battery = 1;
         batnum++;
@@ -336,7 +641,7 @@ _battery_device_update(void)
    if (time_left < 1) time_left = -1;
    if (time_full < 1) time_full = -1;
 
-   _battery_update(full, time_left, time_full, have_battery, have_power);
+   _battery_update(full, time_left, time_full, have_battery, have_power, charging);
 }
 
 void
@@ -345,7 +650,7 @@ _battery_config_updated(void)
    Eina_List *l;
    Instance *inst;
    char buf[4096];
-   int ok = 0;
+   int ok = 1;
 
    if (!battery_config) return;
 
@@ -365,11 +670,14 @@ _battery_config_updated(void)
        (battery_config->force_mode == SUBSYSTEM))
      {
 #ifdef HAVE_EEZE
-        ok = _battery_udev_start();
-#elif defined __OpenBSD__ || defined __DragonFly__ || defined __FreeBSD__ || defined __NetBSD__
-        ok = _battery_sysctl_start();
+        if (!eina_list_count(device_batteries))
+          ok = _battery_udev_start();
+#elif defined(__OpenBSD__) || defined(__DragonFly__) || defined(__FreeBSD__)
+        if (!eina_list_count(device_batteries))
+          ok = _battery_sysctl_start();
 #else
-        ok = _battery_upower_start();
+        if (!eina_list_count(device_batteries))
+          ok = _battery_upower_start();
 #endif
      }
    if (ok) return;
@@ -523,7 +831,7 @@ _powersave_cb_config_update(void *data EINA_UNUSED, int type EINA_UNUSED, void *
 }
 
 static void
-_battery_update(int full, int time_left, int time_full, Eina_Bool have_battery, Eina_Bool have_power)
+_battery_update(int full, int time_left, int time_full, Eina_Bool have_battery, Eina_Bool have_power, Eina_Bool charging)
 {
    Eina_List *l;
    Instance *inst;
@@ -533,18 +841,21 @@ _battery_update(int full, int time_left, int time_full, Eina_Bool have_battery,
      {
         if (have_power != battery_config->have_power)
           {
+             if (have_power)
+               edje_object_signal_emit(inst->o_battery, "e,state,ac,on", "e");
+             else
+               edje_object_signal_emit(inst->o_battery, "e,state,ac,off", "e");
              if (have_power && (full < 100))
-               edje_object_signal_emit(inst->o_battery,
-                                       "e,state,charging",
-                                       "e");
+               {
+                  edje_object_signal_emit(inst->o_battery, "e,state,charging", "e");
+                  if (inst->popup_battery)
+                    edje_object_signal_emit(inst->popup_battery, "e,state,charging", "e");
+               }
              else
                {
-                  edje_object_signal_emit(inst->o_battery,
-                                          "e,state,discharging",
-                                          "e");
+                  edje_object_signal_emit(inst->o_battery, "e,state,discharging", "e");
                   if (inst->popup_battery)
-                    edje_object_signal_emit(inst->popup_battery,
-                                            "e,state,discharging", "e");
+                    edje_object_signal_emit(inst->popup_battery, "e,state,discharging", "e");
                }
           }
         if (have_battery)
@@ -563,9 +874,12 @@ _battery_update(int full, int time_left, int time_full, Eina_Bool have_battery,
         else
           {
              _battery_face_level_set(inst->o_battery, 0.0);
-             edje_object_part_text_set(inst->o_battery,
-                                       "e.text.reading",
-                                       _("N/A"));
+             edje_object_part_text_set(inst->o_battery, "e.text.reading", _("N/A"));
+             if (inst->popup_battery)
+               {
+                  _battery_face_level_set(inst->popup_battery, 0.0);
+                  edje_object_part_text_set(inst->popup_battery, "e.text.reading", _("N/A"));
+               }
           }
 
         if ((time_full < 0) && (time_left != battery_config->time_left))
@@ -612,7 +926,7 @@ _battery_update(int full, int time_left, int time_full, Eina_Bool have_battery,
                {
                   printf("t-debounce = %3.3f\n", (t - debounce_time));
                   debounce_time = t;
-                  if ((t - init_time) > 5.0)
+                  if (((t - init_time) > 5.0) && (full < 15))
                     _battery_warning_popup(inst, time_left, (double)full / 100.0);
                }
           }
@@ -647,6 +961,7 @@ _battery_update(int full, int time_left, int time_full, Eina_Bool have_battery,
    battery_config->time_left = time_left;
    battery_config->have_battery = have_battery;
    battery_config->have_power = have_power;
+   battery_config->charging = charging;
 }
 
 static Eina_Bool
@@ -690,11 +1005,12 @@ _battery_cb_exe_data(void *data EINA_UNUSED, int type EINA_UNUSED, void *event)
                   int time_full = 0;
                   int have_battery = 0;
                   int have_power = 0;
+                  int charging = 0;
 
                   if (sscanf(ev->lines[i].line, "%i %i %i %i %i", &full, &time_left, &time_full,
                              &have_battery, &have_power) == 5)
                     _battery_update(full, time_left, time_full,
-                                    have_battery, have_power);
+                                    have_battery, have_power, charging);
                   else
                     e_powersave_mode_set(E_POWERSAVE_MODE_LOW);
                }
@@ -723,8 +1039,6 @@ E_API E_Module_Api e_modapi =
 E_API void *
 e_modapi_init(E_Module *m)
 {
-   char buf[4096];
-
    conf_edd = E_CONFIG_DD_NEW("Battery_Config", Config);
 #undef T
 #undef D
@@ -736,7 +1050,7 @@ e_modapi_init(E_Module *m)
    E_CONFIG_VAL(D, T, alert_timeout, INT);
    E_CONFIG_VAL(D, T, suspend_below, INT);
    E_CONFIG_VAL(D, T, force_mode, INT);
-#if defined HAVE_EEZE || defined __OpenBSD__ || defined __NetBSD__
+#if defined HAVE_EEZE || defined(__OpenBSD__)
    E_CONFIG_VAL(D, T, fuzzy, INT);
 #endif
    E_CONFIG_VAL(D, T, desktop_notifications, INT);
@@ -751,7 +1065,7 @@ e_modapi_init(E_Module *m)
         battery_config->alert_timeout = 0;
         battery_config->suspend_below = 0;
         battery_config->force_mode = 0;
-#if defined HAVE_EEZE || defined __OpenBSD__ || defined __NetBSD__
+#if defined HAVE_EEZE || defined(__OpenBSD__)
         battery_config->fuzzy = 0;
 #endif
         battery_config->desktop_notifications = 0;
@@ -782,11 +1096,10 @@ e_modapi_init(E_Module *m)
 
    e_gadcon_provider_register(&_gadcon_class);
 
-   snprintf(buf, sizeof(buf), "%s/e-module-battery.edj", e_module_dir_get(m));
    e_configure_registry_category_add("advanced", 80, _("Advanced"), NULL,
                                      "preferences-advanced");
    e_configure_registry_item_add("advanced/battery", 100, _("Battery Meter"),
-                                 NULL, buf, e_int_config_battery_module);
+                                 NULL, "battery", e_int_config_battery_module);
 
    return m;
 }
@@ -824,7 +1137,7 @@ e_modapi_shutdown(E_Module *m EINA_UNUSED)
 
 #ifdef HAVE_EEZE
    _battery_udev_stop();
-#elif defined (__OpenBSD__) || defined (__DragonFly__) || defined (__FreeBSD__) || defined (__NetBSD__)
+#elif defined(__OpenBSD__) || defined(__DragonFly__) || defined(__FreeBSD__)
    _battery_sysctl_stop();
 #else
    _battery_upower_stop();
