@@ -14,15 +14,17 @@ typedef struct _Obj Obj;
 struct _Obj {
    //// internal object data
    Eldbus_Object *obj;
-   Eldbus_Proxy *proxy;
-   Eldbus_Proxy *prop_proxy;
-   Eldbus_Signal_Handler *prop_sig;
+   Eldbus_Proxy *proxy, *proxy_bat;
+   Eldbus_Proxy *prop_proxy, *prop_proxy_bat;
+   Eldbus_Signal_Handler *prop_sig, *prop_sig_bat;
    unsigned int ref;
    Eina_Bool in_table : 1;
    Eina_Bool add_called : 1;
    Eina_Bool ping_ok : 1;
    Eina_Bool ping_busy : 1;
+   Eina_Bool ping_block : 1;
    Ecore_Timer *ping_timer;
+   Ecore_Timer *power_retry_timer;
    //// public data to read
    const char *path;
    Obj_Type type;
@@ -43,6 +45,7 @@ struct _Obj {
    unsigned short appearance;
    unsigned short txpower;
    short rssi;
+   signed char bat_percent;
    Eina_Bool paired : 1;
    Eina_Bool connected : 1;
    Eina_Bool trusted : 1;
