@@ -643,8 +643,6 @@ _gc_init(E_Gadcon *gc, const char *name, const char *id, const char *style)
      edje_object_signal_emit(o, "e,state,seconds,off", "e");
 
    edje_object_part_text_set(o, "e.text.today", todaystr);
-   edje_object_message_signal_process(o);
-   evas_object_show(o);
 
    gcc = e_gadcon_client_new(gc, name, id, style, o);
    gcc->data = inst;
@@ -656,6 +654,8 @@ _gc_init(E_Gadcon *gc, const char *name, const char *id, const char *style)
                                   EVAS_CALLBACK_MOUSE_DOWN,
                                   _clock_cb_mouse_down,
                                   inst);
+   edje_object_message_signal_process(o);
+   evas_object_show(o);
 
    clock_instances = eina_list_append(clock_instances, inst);
 
@@ -845,6 +845,15 @@ _clock_time_update(void *d EINA_UNUSED, int type EINA_UNUSED, void *event EINA_U
 static Eina_Bool
 _clock_screensaver_on()
 {
+   Eina_List *l;
+   Instance *inst;
+
+   EINA_LIST_FOREACH(clock_instances, l, inst)
+     {
+        edje_object_signal_emit(inst->o_clock, "e,state,freeze", "e");
+        if (inst->o_popclock)
+          edje_object_signal_emit(inst->o_popclock, "e,state,freeze", "e");
+     }
    E_FREE_FUNC(update_today, ecore_timer_del);
    return ECORE_CALLBACK_RENEW;
 }
@@ -852,6 +861,15 @@ _clock_screensaver_on()
 static Eina_Bool
 _clock_screensaver_off()
 {
+   Eina_List *l;
+   Instance *inst;
+
+   EINA_LIST_FOREACH(clock_instances, l, inst)
+     {
+        edje_object_signal_emit(inst->o_clock, "e,state,thaw", "e");
+        if (inst->o_popclock)
+          edje_object_signal_emit(inst->o_popclock, "e,state,thaw", "e");
+     }
    if (clock_instances) _update_today_timer(NULL);
    return ECORE_CALLBACK_RENEW;
 }
