@@ -1390,8 +1390,8 @@ _conf_new(void)
      {
         _conf = E_NEW(Module_Config, 1);
         /* setup defaults */
-        _conf->cmd_terminal = eina_stringshare_add("/usr/bin/xterm -hold -e");
-        _conf->cmd_sudo = eina_stringshare_add("/usr/bin/gksudo --preserve-env");
+        _conf->cmd_terminal = eina_stringshare_add("terminology -H -e");
+        _conf->cmd_sudo = eina_stringshare_add("sudo");
      }
 
     _conf->version = MOD_CONFIG_FILE_VERSION;
