@@ -1,10 +1,13 @@
 #include "private.h"
 
+// XXX: 
+
 #define MUSIC_CONTROL_DOMAIN "module.music_control"
 
 static E_Module *music_control_mod = NULL;
 static Eina_Bool was_playing_before_lock = EINA_FALSE;
 static const char _e_music_control_Name[] = N_("Music controller");
+static Ecore_Event_Handler *music_control_powersave_event_handler = NULL;
 static Eina_Bool have_player = EINA_FALSE;
 
 const Player music_player_players[] =
@@ -22,11 +25,16 @@ const Player music_player_players[] =
    {"Rage",                 "org.mpris.MediaPlayer2.rage",             "rage"},
    {"Rhythmbox",            "org.gnome.Rhythmbox3",                    "rhythmbox"},
    {"Spotify",              "org.mpris.MediaPlayer2.spotify",          "spotify"},
+   {"Spotifyd",             "org.mpris.MediaPlayer2.spotifyd",         "spotifyd"},
+   {"Squeezebox",           "org.mpris.MediaPlayer2.squeezebox",       "slimpris2"},
    {"Tomahawk",             "org.mpris.MediaPlayer2.tomahawk",         "tomahawk"},
    {"VLC",                  "org.mpris.MediaPlayer2.vlc",              "vlc"},
    {"XMMS2",                "org.mpris.MediaPlayer2.xmms2",            "xmms2"},
    {"gmusicbrowser",        "org.mpris.MediaPlayer2.gmusicbrowser",    "gmusicbrowser"},
-#define PLAYER_COUNT 16
+   {"Chromium",             "org.mpris.MediaPlayer2.chromium.*",       "chromium"},
+   {"Firefox",              "org.mpris.MediaPlayer2.firefox.*",        "firefox"},
+   {"Automatic",            "org.mpris.MediaPlayer2.*",                ""},
+#define PLAYER_COUNT 21
    {NULL, NULL, NULL}
 };
 
@@ -60,19 +68,28 @@ _music_control(E_Object *obj EINA_UNUSED, const char *params)
 {
    E_Music_Control_Module_Context *ctxt = music_control_mod->data;
    EINA_SAFETY_ON_NULL_RETURN(music_control_mod->data);
+   EINA_SAFETY_ON_NULL_RETURN(params);
 
-   if (!strcmp(params, "play"))
+   if (!strcmp(params, "play")) //legacy compatibility - this should probebly be called playpause
      media_player2_player_play_pause_call(ctxt->mpris2_player);
    else if (!strcmp(params, "next"))
      media_player2_player_next_call(ctxt->mpris2_player);
    else if (!strcmp(params, "previous"))
      media_player2_player_previous_call(ctxt->mpris2_player);
+   else if (!strcmp(params, "pause"))
+     media_player2_player_pause_call(ctxt->mpris2_player);
+   else if (!strcmp(params, "play_music")) //the name play was already taken by play pause, see above
+     media_player2_player_play_call(ctxt->mpris2_player);
 }
 
 #define ACTION_NEXT "next_music"
 #define ACTION_NEXT_NAME "Next Music"
 #define ACTION_PLAY_PAUSE "playpause_music"
 #define ACTION_PLAY_PAUSE_NAME "Play/Pause Music"
+#define ACTION_PAUSE "pause_music"
+#define ACTION_PAUSE_NAME "Pause Music"
+#define ACTION_PLAY "play_music"
+#define ACTION_PLAY_NAME "Play Music"
 #define ACTION_PREVIOUS "previous_music"
 #define ACTION_PREVIOUS_NAME "Previous Music"
 
@@ -90,6 +107,14 @@ _actions_register(E_Music_Control_Module_Context *ctxt)
    action->func.go = _music_control;
    e_action_predef_name_set(_e_music_control_Name, ACTION_PLAY_PAUSE_NAME,
                             ACTION_PLAY_PAUSE, "play", NULL, 0);
+   action = e_action_add(ACTION_PAUSE);
+   action->func.go = _music_control;
+   e_action_predef_name_set(_e_music_control_Name, ACTION_PAUSE_NAME,
+                            ACTION_PAUSE, "pause", NULL, 0);
+   action = e_action_add(ACTION_PLAY);
+   action->func.go = _music_control;
+   e_action_predef_name_set(_e_music_control_Name, ACTION_PLAY_NAME,
+                            ACTION_PLAY, "play_music", NULL, 0);
    action = e_action_add(ACTION_PREVIOUS);
    action->func.go = _music_control;
    e_action_predef_name_set(_e_music_control_Name, ACTION_PREVIOUS_NAME,
@@ -212,52 +237,101 @@ parse_metadata(E_Music_Control_Module_Context *ctxt, Eina_Value *array)
    // DBG("Metadata: %s", eina_value_to_string(array));
    for (i = 0; i < eina_value_array_count(array); i++)
      {
-        const char *key, *str_val;
+        const char *key = NULL, *str_val;
+        long long llval;
         char *str_markup;
         Eina_Value st, subst;
         Efreet_Uri *uri;
 
-        eina_value_array_value_get(array, i, &st);
-        eina_value_struct_get(&st, "arg0", &key);
-        if (!strcmp(key, "xesam:title"))
+        eina_value_setup(&st, EINA_VALUE_TYPE_UINT64);
+        eina_value_setup(&subst, EINA_VALUE_TYPE_UINT64);
+        if (eina_value_array_value_get(array, i, &st) &&
+            eina_value_struct_get(&st, "arg0", &key))
           {
-             eina_value_struct_value_get(&st, "arg1", &subst);
-             eina_value_struct_get(&subst, "arg0", &str_val);
-             str_markup = evas_textblock_text_utf8_to_markup(NULL, str_val);
-             ctxt->meta_title = eina_stringshare_add(str_markup);
-             free(str_markup);
-             eina_value_flush(&subst);
-          }
-        else if (!strcmp(key, "xesam:album"))
-          {
-             eina_value_struct_value_get(&st, "arg1", &subst);
-             eina_value_struct_get(&subst, "arg0", &str_val);
-             str_markup = evas_textblock_text_utf8_to_markup(NULL, str_val);
-             ctxt->meta_album = eina_stringshare_add(str_markup);
-             free(str_markup);
-             eina_value_flush(&subst);
-          }
-        else if (!strcmp(key, "xesam:artist"))
-          {
-             Eina_Value arr;
-             eina_value_struct_value_get(&st, "arg1", &subst);
-             eina_value_struct_value_get(&subst, "arg0", &arr);
-             eina_value_array_get(&arr, 0, &str_val);
-             str_markup = evas_textblock_text_utf8_to_markup(NULL, str_val);
-             ctxt->meta_artist = eina_stringshare_add(str_markup);
-             free(str_markup);
-             eina_value_flush(&arr);
-             eina_value_flush(&subst);
-          }
-        else if (!strcmp(key, "mpris:artUrl"))
-          {
-             eina_value_struct_value_get(&st, "arg1", &subst);
-             eina_value_struct_get(&subst, "arg0", &str_val);
-             uri = efreet_uri_decode(str_val);
-             if (uri && !strncmp(uri->protocol, "file", 4))
-               ctxt->meta_cover = eina_stringshare_add(uri->path);
-             E_FREE_FUNC(uri, efreet_uri_free);
-             eina_value_flush(&subst);
+             if (!strcmp(key, "xesam:title"))
+               {
+                  if (eina_value_struct_value_get(&st, "arg1", &subst) &&
+                      eina_value_struct_get(&subst, "arg0", &str_val))
+                    {
+                       if ((str_markup = evas_textblock_text_utf8_to_markup(NULL, str_val)))
+                         {
+                            ctxt->meta_title = eina_stringshare_add(str_markup);
+                            free(str_markup);
+                         }
+                    }
+                  eina_value_flush(&subst);
+               }
+             else if (!strcmp(key, "xesam:album"))
+               {
+                  if (eina_value_struct_value_get(&st, "arg1", &subst) &&
+                      eina_value_struct_get(&subst, "arg0", &str_val))
+                    {
+                       if ((str_markup = evas_textblock_text_utf8_to_markup(NULL, str_val)))
+                         {
+                            ctxt->meta_album = eina_stringshare_add(str_markup);
+                            free(str_markup);
+                         }
+                    }
+                  eina_value_flush(&subst);
+               }
+             else if (!strcmp(key, "xesam:artist"))
+               {
+                  Eina_Value arr;
+
+                  eina_value_setup(&arr, EINA_VALUE_TYPE_UINT64);
+                  if (eina_value_struct_value_get(&st, "arg1", &subst) &&
+                      eina_value_struct_value_get(&subst, "arg0", &arr) &&
+                      eina_value_array_get(&arr, 0, &str_val))
+                    {
+                       if ((str_markup = evas_textblock_text_utf8_to_markup(NULL, str_val)))
+                         {
+                            ctxt->meta_artist = eina_stringshare_add(str_markup);
+                            free(str_markup);
+                         }
+                    }
+                  eina_value_flush(&arr);
+                  eina_value_flush(&subst);
+               }
+             else if (!strcmp(key, "mpris:artUrl"))
+               {
+                  if (eina_value_struct_value_get(&st, "arg1", &subst) &&
+                      eina_value_struct_get(&subst, "arg0", &str_val))
+                    {
+                       uri = efreet_uri_decode(str_val);
+                       if (uri && !strncmp(uri->protocol, "file", 4))
+                         ctxt->meta_cover = eina_stringshare_add(uri->path);
+                       else
+                         {
+                            if (!strncmp(str_val, "https://open.spotify.com/image/", 31))
+                              {
+                                 // fix spotify brokenness:
+                                 //   https://open.spotify.com/image/ab67616d00001e02216a271725a34eba54dd0b55
+                                 // becomes
+                                 //   https://i.scdn.co/image/ab67616d00001e02216a271725a34eba54dd0b55
+                                 char buf[1024];
+                                 snprintf(buf, sizeof(buf), "https://i.scdn.co/image/%s", str_val + 31);
+                                 ctxt->meta_cover = eina_stringshare_add(buf);
+                              }
+                            else
+                              ctxt->meta_cover = eina_stringshare_add(str_val);
+                         }
+                       E_FREE_FUNC(uri, efreet_uri_free);
+                    }
+                  eina_value_flush(&subst);
+               }
+             else if (!strcmp(key, "mpris:length"))
+               {
+                  llval = -1;
+                  if (eina_value_struct_value_get(&st, "arg1", &subst) &&
+                      eina_value_struct_get(&subst, "arg0", &llval))
+                    {
+                       if (llval < 0) llval = 0;
+                       ctxt->meta_length = (double)llval / 1000000.0;
+                    }
+                  eina_value_flush(&subst);
+               }
+             // FIXME: to handle in future:
+             // xesam:url - s
           }
         eina_value_flush(&st);
      }
@@ -273,8 +347,7 @@ cb_playback_status_get(void *data, Eldbus_Pending *p EINA_UNUSED,
 
    if (error_info)
      {
-        fprintf(stderr, "MUSIC-CONTROL: %s %s",
-                error_info->error, error_info->message);
+        printf("MUSIC-CONTROL: %s %s", error_info->error, error_info->message);
         return;
      }
 
@@ -310,11 +383,129 @@ prop_changed(void *data, Eldbus_Proxy *proxy EINA_UNUSED, void *event_info)
         else ctxt->playing = EINA_FALSE;
         music_control_state_update_all(ctxt);
      }
+   else if (!strcmp(event->name, "LoopStatus"))
+     {
+        const Eina_Value *value = event->value;
+        const char *status;
+
+        eina_value_get(value, &status);
+        if (!strcmp(status, "None")) ctxt->loop = EINA_FALSE;
+        else if (!strcmp(status, "Track")) ctxt->loop = EINA_TRUE;
+        if (!strcmp(status, "Playlist")) ctxt->loop = EINA_TRUE;
+        music_control_state_update_all(ctxt);
+     }
    else if (!strcmp(event->name, "Metadata"))
      {
         parse_metadata(ctxt, (Eina_Value*)event->value);
         music_control_metadata_update_all(ctxt);
      }
+   else if (!strcmp(event->name, "Position"))
+     {
+        const Eina_Value *value = event->value;
+        long long llval = 0;
+
+        eina_value_get(value, &llval);
+        if (llval < 0) llval = 0;
+        ctxt->position = (double)llval / 1000000.0;
+        music_control_pos_update(ctxt);
+     }
+}
+
+static void _bus_list(E_Music_Control_Module_Context *ctxt);
+
+static Eina_Bool
+_poll_dbus_cb(void *data)
+{
+   E_Music_Control_Module_Context *ctxt = data;
+   ctxt->poll_timer = NULL;
+   _bus_list(ctxt);
+   return EINA_FALSE;
+}
+
+static void
+_timer_redo(E_Music_Control_Module_Context *ctxt)
+{
+   E_Powersave_Mode pm;
+   double t = 5.0;
+
+   if (ctxt->poll_timer) ecore_timer_del(ctxt->poll_timer);
+   ctxt->poll_timer = NULL;
+   pm = e_powersave_mode_get();
+   if (e_powersave_mode_screen_get() > pm) pm = e_powersave_mode_screen_get();
+   if      (pm >= E_POWERSAVE_MODE_FREEZE)  t = 3600.0;
+   else if (pm >= E_POWERSAVE_MODE_EXTREME) t =  600.0;
+   else if (pm >= E_POWERSAVE_MODE_HIGH)    t =   10.0;
+   else                                     t =    5.0;
+   ctxt->poll_timer = ecore_timer_add(t, _poll_dbus_cb, ctxt);
+}
+
+static void
+_bus_list_cb(void *data, const Eldbus_Message *msg, Eldbus_Pending *pending EINA_UNUSED)
+{
+   E_Music_Control_Module_Context *ctxt = data;
+   Eldbus_Message_Iter *array = NULL;
+   const char *bus;
+   char *bus_name = NULL;
+   int i, chosen = -1, auto_chosen = -1;
+
+   ctxt->bus_list_pend = NULL;
+   if (!eldbus_message_arguments_get(msg, "as", &array)) return;
+   while (eldbus_message_iter_get_and_next(array, 's', &bus))
+     {
+        if ((!bus) || (bus[0] == ':')) continue;
+        if ((ctxt->config) &&
+            (ctxt->config->player_selected >= 0) &&
+            (ctxt->config->player_selected < PLAYER_COUNT) &&
+            (e_util_glob_match(bus, music_player_players[ctxt->config->player_selected].dbus_name)))
+          {
+             if (bus_name) free(bus_name);
+             bus_name = strdup(bus);
+             chosen = ctxt->config->player_selected;
+          }
+        else
+          {
+             for (i = 0; music_player_players[i].name; i++)
+               {
+                  if (e_util_glob_match(bus, music_player_players[i].dbus_name))
+                    {
+                       if (bus_name) free(bus_name);
+                       bus_name = strdup(bus);
+                       auto_chosen = i;
+                       break;
+                    }
+               }
+          }
+     }
+   if (chosen < 0) chosen = auto_chosen;
+   if (chosen < 0)
+     {
+        eina_stringshare_del(ctxt->dbus_name);
+        ctxt->dbus_name = NULL;
+        _timer_redo(ctxt);
+        return;
+     }
+   if ((!ctxt->dbus_name) ||
+       (!bus_name) ||
+       (!!strcmp(ctxt->dbus_name, bus_name)))
+     {
+        eina_stringshare_del(ctxt->dbus_name);
+        ctxt->dbus_name = NULL;
+        if (bus_name)
+          {
+             music_control_dbus_init(ctxt, bus_name);
+             free(bus_name);
+             if (ctxt->poll_timer) ecore_timer_del(ctxt->poll_timer);
+             ctxt->poll_timer = NULL;
+          }
+        else _timer_redo(ctxt);
+     }
+}
+
+static void
+_bus_list(E_Music_Control_Module_Context *ctxt)
+{
+   if (ctxt->bus_list_pend) eldbus_pending_cancel(ctxt->bus_list_pend);
+   ctxt->bus_list_pend = eldbus_names_list(ctxt->conn, _bus_list_cb, ctxt);
 }
 
 static void
@@ -334,6 +525,8 @@ cb_name_owner_changed(void *data,
           (ctxt->mpris2_player, cb_metadata_get, ctxt);
         have_player = EINA_TRUE;
      }
+   else
+     _bus_list(ctxt);
 }
 
 void
@@ -374,6 +567,15 @@ music_control_launch(E_Music_Control_Instance *inst)
      }
 }
 
+static Eina_Bool
+_powersave_change_cb(void *data, int type EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   E_Music_Control_Module_Context *ctxt = data;
+
+   _timer_redo(ctxt);
+   return ECORE_CALLBACK_PASS_ON;
+}
+
 Eina_Bool
 music_control_dbus_init(E_Music_Control_Module_Context *ctxt, const char *bus)
 {
@@ -381,10 +583,9 @@ music_control_dbus_init(E_Music_Control_Module_Context *ctxt, const char *bus)
      ctxt->conn = eldbus_connection_get(ELDBUS_CONNECTION_TYPE_SESSION);
    EINA_SAFETY_ON_NULL_RETURN_VAL(ctxt->conn, EINA_FALSE);
 
-   if (ctxt->mpris2_player)
-     mpris_media_player2_proxy_unref(ctxt->mpris2_player);
-   if (ctxt->mrpis2)
-     media_player2_player_proxy_unref(ctxt->mrpis2);
+   if (ctxt->mpris2_player) mpris_media_player2_proxy_unref(ctxt->mpris2_player);
+   if (ctxt->mrpis2) media_player2_player_proxy_unref(ctxt->mrpis2);
+
    ctxt->mrpis2 = mpris_media_player2_proxy_get(ctxt->conn, bus, NULL);
    ctxt->mpris2_player = media_player2_player_proxy_get(ctxt->conn, bus, NULL);
    eldbus_proxy_event_callback_add(ctxt->mpris2_player,
@@ -426,17 +627,11 @@ e_modapi_init(E_Module *m)
         ctxt->config->player_selected = -1;
      }
 
-   if (ctxt->config->player_selected < 0)
-     {
-     }
-   else
-     {
-        if (ctxt->config->player_selected >= PLAYER_COUNT)
-          ctxt->config->player_selected = PLAYER_COUNT - 1;
-        if (!music_control_dbus_init
-            (ctxt, music_player_players[ctxt->config->player_selected].dbus_name))
-          goto error_dbus_bus_get;
-     }
+   if (ctxt->config->player_selected >= PLAYER_COUNT)
+     ctxt->config->player_selected = PLAYER_COUNT - 1;
+   if (!ctxt->conn)
+     ctxt->conn = eldbus_connection_get(ELDBUS_CONNECTION_TYPE_SESSION);
+   _bus_list(ctxt);
    music_control_mod = m;
 
    e_gadcon_provider_register(&_gc_class);
@@ -444,11 +639,10 @@ e_modapi_init(E_Module *m)
    if (ctxt->config->pause_on_desklock)
      desklock_handler = ecore_event_handler_add(E_EVENT_DESKLOCK,
                                                 _desklock_cb, ctxt);
+   music_control_powersave_event_handler =
+     ecore_event_handler_add(E_EVENT_POWERSAVE_UPDATE,
+                             _powersave_change_cb, ctxt);
    return ctxt;
-
-error_dbus_bus_get:
-   free(ctxt);
-   return NULL;
 }
 
 E_API int
@@ -458,16 +652,28 @@ e_modapi_shutdown(E_Module *m EINA_UNUSED)
    EINA_SAFETY_ON_NULL_RETURN_VAL(music_control_mod, 0);
    ctxt = music_control_mod->data;
 
+   ecore_event_handler_del(music_control_powersave_event_handler);
+   music_control_powersave_event_handler = NULL;
+
    E_FREE_FUNC(ctxt->meta_title, eina_stringshare_del);
    E_FREE_FUNC(ctxt->meta_album, eina_stringshare_del);
    E_FREE_FUNC(ctxt->meta_artist, eina_stringshare_del);
    E_FREE_FUNC(ctxt->meta_cover, eina_stringshare_del);
+   E_FREE_FUNC(ctxt->meta_cover_prev, eina_stringshare_del);
+
+   if (ctxt->poll_timer) ecore_timer_del(ctxt->poll_timer);
+   ctxt->poll_timer = NULL;
 
    free(ctxt->config);
    E_CONFIG_DD_FREE(ctxt->conf_edd);
 
    E_FREE_FUNC(desklock_handler, ecore_event_handler_del);
 
+   if (ctxt->bus_list_pend)
+     {
+        eldbus_pending_cancel(ctxt->bus_list_pend);
+        ctxt->bus_list_pend = NULL;
+     }
    eldbus_name_owner_changed_callback_del
      (ctxt->conn, ctxt->dbus_name, cb_name_owner_changed, ctxt);
    eina_stringshare_del(ctxt->dbus_name);
@@ -478,7 +684,7 @@ e_modapi_shutdown(E_Module *m EINA_UNUSED)
    e_gadcon_provider_unregister(&_gc_class);
 
    if (eina_list_count(ctxt->instances))
-     fprintf(stderr, "MUSIC-CONTROL: Live instances.");
+     printf("MUSIC-CONTROL: Live instances.");
 
    free(ctxt);
    music_control_mod = NULL;
