@@ -110,11 +110,9 @@ notification_popup_notify(E_Notification_Notify *n,
      {
       case E_NOTIFICATION_NOTIFY_URGENCY_LOW:
         if (!notification_cfg->show_low) return;
-        if (e_config->mode.presentation) return;
         break;
       case E_NOTIFICATION_NOTIFY_URGENCY_NORMAL:
         if (!notification_cfg->show_normal) return;
-        if (e_config->mode.presentation) return;
         break;
       case E_NOTIFICATION_NOTIFY_URGENCY_CRITICAL:
         if (!notification_cfg->show_critical) return;
@@ -344,6 +342,7 @@ _notification_popup_new(E_Notification_Notify *n, unsigned id)
              if (zone == e_comp_object_util_zone_get(popup->win)) continue;
              o = e_comp_object_util_mirror_add(popup->theme);
              o = e_comp_object_util_add(o, E_COMP_OBJECT_TYPE_POPUP);
+             edje_object_signal_emit(o, "e,state,shadow,off", "e");
              evas_object_name_set(o, "notification_mirror");
              evas_object_data_set(o, "zone", zone);
              evas_object_geometry_get(popup->win, NULL, NULL, &w, &h);
