@@ -47,10 +47,14 @@ struct _Tasks_Item
    E_Client    *client; // The client this item points to
    Evas_Object *o_item; // The edje theme object
    Evas_Object *o_icon; // The icon
+   Evas_Object *o_preview; // The preview
+   Ecore_Timer *timer; // The preview timer
    Eina_Bool skip_taskbar E_BITFIELD;
    Eina_Bool focused E_BITFIELD;
    Eina_Bool urgent E_BITFIELD;
    Eina_Bool iconified E_BITFIELD;
+   Eina_Bool iconifying E_BITFIELD;
+   Eina_Bool delete_me E_BITFIELD;
 };
 
 static Tasks       *_tasks_new(Evas *e, E_Zone *zone, const char *id);
@@ -61,6 +65,7 @@ static void         _tasks_refill_border(E_Client *ec);
 
 static Tasks_Item  *_tasks_item_find(Tasks *tasks, E_Client *ec);
 static Tasks_Item  *_tasks_item_new(Tasks *tasks, E_Client *ec);
+static Tasks_Item  *_tasks_all_item_find(E_Client *ec);
 
 static int          _tasks_item_check_add(Tasks *tasks, E_Client *ec);
 static void         _tasks_item_add(Tasks *tasks, E_Client *ec);
@@ -69,6 +74,8 @@ static void         _tasks_item_refill(Tasks_Item *item);
 static void         _tasks_item_fill(Tasks_Item *item);
 static void         _tasks_item_free(Tasks_Item *item);
 static void         _tasks_item_signal_emit(Tasks_Item *item, char *sig, char *src);
+static void         _tasks_item_preview_add(Tasks_Item *item);
+static void         _tasks_item_preview_del(Tasks_Item *item);
 
 static Config_Item *_tasks_config_item_get(const char *id);
 
@@ -76,6 +83,9 @@ static void         _tasks_cb_menu_configure(void *data, E_Menu *m, E_Menu_Item
 static void         _tasks_cb_item_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void         _tasks_cb_item_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void         _tasks_cb_item_mouse_wheel(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info);
+static void         _tasks_cb_item_mouse_in(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info);
+static void         _tasks_cb_item_mouse_out(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info);
+static void         _tasks_cb_item_del(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info);
 
 static Eina_Bool    _tasks_cb_event_client_add(void *data, int type, void *event);
 static Eina_Bool    _tasks_cb_event_client_remove(void *data, int type, void *event);
@@ -87,6 +97,7 @@ static Eina_Bool    _tasks_cb_event_client_desk_set(void *data, int type, E_Even
 static Eina_Bool    _tasks_cb_window_focus_in(void *data, int type, void *event);
 static Eina_Bool    _tasks_cb_window_focus_out(void *data, int type, void *event);
 static Eina_Bool    _tasks_cb_event_desk_show(void *data, int type, void *event);
+static Eina_Bool    _tasks_cb_timer_del(void *data);
 
 static E_Config_DD *conf_edd = NULL;
 static E_Config_DD *conf_item_edd = NULL;
@@ -112,11 +123,14 @@ e_modapi_init(E_Module *m)
 #define T Config_Item
 #define D conf_item_edd
    E_CONFIG_VAL(D, T, id, STR);
-   E_CONFIG_VAL(D, T, show_all, INT);
+   E_CONFIG_VAL(D, T, show_all_desktops, INT);
+   E_CONFIG_VAL(D, T, show_all_screens, INT);
    E_CONFIG_VAL(D, T, minw, INT);
    E_CONFIG_VAL(D, T, minh, INT);
+   E_CONFIG_VAL(D, T, preview_size, INT);
    E_CONFIG_VAL(D, T, icon_only, UCHAR);
    E_CONFIG_VAL(D, T, text_only, UCHAR);
+   E_CONFIG_VAL(D, T, preview, UCHAR);
 
    conf_edd = E_CONFIG_DD_NEW("Tasks_Config", Config);
 
@@ -135,9 +149,12 @@ e_modapi_init(E_Module *m)
 
         config = E_NEW(Config_Item, 1);
         config->id = eina_stringshare_add("0");
-        config->show_all = 0;
+        config->show_all_desktops = 0;
+        config->show_all_screens = 0;
         config->minw = 100;
         config->minh = 32;
+        config->preview = 0;
+        config->preview_size = 32;
         tasks_config->items = eina_list_append(tasks_config->items, config);
      }
 
@@ -318,12 +335,20 @@ _gc_id_new(const E_Gadcon_Client_Class *client_class EINA_UNUSED)
 static void
 _tasks_cb_iconify_end_cb(void *data, Evas_Object *obj EINA_UNUSED, const char *sig EINA_UNUSED, const char *src EINA_UNUSED)
 {
+   Tasks_Item *item;
    E_Client *ec = data;
 
+   if (!ec) return;
+
    evas_object_layer_set(ec->frame, ec->layer);
    ec->layer_block = 0;
-   if (ec->iconic)
-     evas_object_hide(ec->frame);
+   if (ec->iconic) evas_object_hide(ec->frame);
+
+   item = _tasks_all_item_find(ec);
+   if (!item) return;
+
+   item->iconifying = 0;
+   if (item->delete_me) free(item);
 }
 
 static Eina_Bool
@@ -344,7 +369,7 @@ _tasks_cb_iconify_provider(void *data, Evas_Object *obj, const char *signal)
         ec2 = e_client_stack_bottom_get(item->client);
         for (; ec2; ec2 = ec2->stack.next)
           {
-             if (ec2 == ec)
+             if ((ec2 == ec) && (!item->iconifying))
                {
                   evas_object_geometry_get(item->o_item, &ox, &oy, &ow, &oh);
                   ec->layer_block = 1;
@@ -352,6 +377,7 @@ _tasks_cb_iconify_provider(void *data, Evas_Object *obj, const char *signal)
                   e_comp_object_effect_set(ec->frame, "iconify/tasks");
                   e_comp_object_effect_params_set(ec->frame, 1, (int[]){ec->x, ec->y, ec->w, ec->h, ox, oy, ow, oh}, 8);
                   e_comp_object_effect_params_set(ec->frame, 0, (int[]){!!strcmp(signal, "e,action,iconify")}, 1);
+                  item->iconifying = 1;
                   e_comp_object_effect_start(ec->frame, _tasks_cb_iconify_end_cb, ec);
                   return EINA_TRUE;
                }
@@ -360,6 +386,26 @@ _tasks_cb_iconify_provider(void *data, Evas_Object *obj, const char *signal)
    return EINA_FALSE;
 }
 
+static void
+_taks_client_remove(E_Client *ec)
+{
+   Tasks *tasks;
+   Eina_List *l;
+
+   if (!tasks_config) return;
+   EINA_LIST_FOREACH(tasks_config->tasks, l, tasks)
+     {
+        tasks->clients = eina_list_remove(tasks->clients, ec);
+     }
+   _tasks_refill_all();
+}
+
+static void
+_tasks_cb_client_del(void *data EINA_UNUSED, void *obj)
+{
+   _taks_client_remove(obj);
+}
+
 static Tasks *
 _tasks_new(Evas *e, E_Zone *zone, const char *id)
 {
@@ -370,12 +416,17 @@ _tasks_new(Evas *e, E_Zone *zone, const char *id)
    tasks = E_NEW(Tasks, 1);
    tasks->config = _tasks_config_item_get(id);
    tasks->o_items = elm_box_add(e_win_evas_win_get(e));
+   E_EXPAND(tasks->o_items);
+   E_FILL(tasks->o_items);
    tasks->horizontal = 1;
    EINA_LIST_FOREACH(e_comp->clients, l, ec)
      {
         if ((!e_client_util_ignored_get(ec)) && (!e_object_is_del(E_OBJECT(ec))) &&
             (!e_client_util_is_popup(ec)))
-          tasks->clients = eina_list_append(tasks->clients, ec);
+          {
+             e_object_delfn_add(E_OBJECT(ec), _tasks_cb_client_del, NULL);
+             tasks->clients = eina_list_append(tasks->clients, ec);
+          }
      }
 
    elm_box_homogeneous_set(tasks->o_items, 1);
@@ -393,7 +444,7 @@ _tasks_free(Tasks *tasks)
    e_comp_object_effect_mover_del(tasks->iconify_provider);
    EINA_LIST_FREE(tasks->items, item)
      _tasks_item_free(item);
-   eina_list_free(tasks->clients);
+   tasks->clients = eina_list_free(tasks->clients);
    evas_object_del(tasks->o_items);
    free(tasks);
 }
@@ -427,10 +478,12 @@ _tasks_refill(Tasks *tasks)
         if (tasks->horizontal)
           {
              if (w < tasks->config->minw) w = tasks->config->minw;
+             if (tasks->config->icon_only) w = h;
           }
         else
           {
              if (h < tasks->config->minh) h = tasks->config->minh;
+             if (tasks->config->icon_only) h = w;
           }
         if (!tasks->gcc->resizable)
           {
@@ -443,6 +496,20 @@ _tasks_refill(Tasks *tasks)
                                             w,
                                             h * eina_list_count(tasks->items));
           }
+        if (tasks->horizontal)
+          {
+             EINA_LIST_FOREACH(tasks->items, l, item)
+               {
+                  evas_object_size_hint_min_set(item->o_item, w, th);
+               }
+          }
+        else
+          {
+             EINA_LIST_FOREACH(tasks->items, l, item)
+               {
+                  evas_object_size_hint_min_set(item->o_item, tw, h);
+               }
+          }
      }
    else
      e_gadcon_client_min_size_set(tasks->gcc, 0, 0);
@@ -529,6 +596,21 @@ _tasks_item_find(Tasks *tasks, E_Client *ec)
    return NULL;
 }
 
+static Tasks_Item *
+_tasks_all_item_find(E_Client *ec)
+{
+   const Eina_List *l;
+   Tasks *tasks;
+   Tasks_Item *item = NULL;
+
+   EINA_LIST_FOREACH(tasks_config->tasks, l, tasks)
+     {
+        item = _tasks_item_find(tasks, ec);
+        if (item) break;
+     }
+   return item;
+}
+
 static Tasks_Item *
 _tasks_item_new(Tasks *tasks, E_Client *ec)
 {
@@ -569,6 +651,12 @@ _tasks_item_new(Tasks *tasks, E_Client *ec)
                                   _tasks_cb_item_mouse_up, item);
    evas_object_event_callback_add(item->o_item, EVAS_CALLBACK_MOUSE_WHEEL,
                                   _tasks_cb_item_mouse_wheel, item);
+   evas_object_event_callback_add(item->o_item, EVAS_CALLBACK_MOUSE_IN,
+                                  _tasks_cb_item_mouse_in, item);
+   evas_object_event_callback_add(item->o_item, EVAS_CALLBACK_MOUSE_OUT,
+                                  _tasks_cb_item_mouse_out, item);
+   evas_object_event_callback_add(item->o_item, EVAS_CALLBACK_DEL,
+                                  _tasks_cb_item_del, item);
    evas_object_show(item->o_item);
 
    _tasks_item_fill(item);
@@ -583,13 +671,19 @@ _tasks_item_check_add(Tasks *tasks, E_Client *ec)
    if (ec->stack.prev) return 1;
    if (_tasks_item_find(tasks, ec)) return 1;
    if (!tasks->config) return 1;
-   if (!(tasks->config->show_all))
+
+   if (!(tasks->config->show_all_desktops))
      {
-        if (ec->zone != tasks->zone) return 1;
         if ((ec->desk != e_desk_current_get(ec->zone)) &&
             (!ec->sticky))
           return 1;
      }
+
+   if (!(tasks->config->show_all_screens))
+     {
+        if (ec->zone != tasks->zone) return 1;
+     }
+
    _tasks_item_add(tasks, ec);
    return 0;
 }
@@ -614,15 +708,35 @@ _tasks_item_remove(Tasks_Item *item)
    _tasks_item_free(item);
 }
 
+static void
+_tasks_item_preview_del(Tasks_Item *item)
+{
+   _tasks_cb_timer_del(item);
+}
+
 static void
 _tasks_item_free(Tasks_Item *item)
 {
-   if (item->o_icon) evas_object_del(item->o_icon);
+   if (item->o_icon)
+     {
+        evas_object_del(item->o_icon);
+        item->o_icon = NULL;
+     }
    if (e_object_is_del(E_OBJECT(item->client)))
      item->tasks->clients = eina_list_remove(item->tasks->clients, item->client);
    e_object_unref(E_OBJECT(item->client));
-   evas_object_del(item->o_item);
-   free(item);
+   _tasks_item_preview_del(item);
+   if (item->o_item)
+     {
+        evas_object_del(item->o_item);
+        item->o_item = NULL;
+     }
+   item->client = NULL;
+   item->tasks = NULL;
+   if (item->iconifying)
+     item->delete_me = EINA_TRUE;
+   else
+     free(item);
 }
 
 static void
@@ -633,7 +747,11 @@ _tasks_item_refill(Tasks_Item *item)
         _tasks_refill(item->tasks);
         return;
      }
-   if (item->o_icon) evas_object_del(item->o_icon);
+   if (item->o_icon)
+     {
+        evas_object_del(item->o_icon);
+        item->o_icon = NULL;
+     }
    _tasks_item_fill(item);
 }
 
@@ -690,9 +808,7 @@ _tasks_item_fill(Tasks_Item *item)
 
    ec = item->client;
 
-   if (item->tasks->config->text_only)
-     item->o_icon = NULL;
-   else
+   if (!item->tasks->config->text_only)
      {
         item->o_icon = e_client_icon_add(ec, evas_object_evas_get(item->tasks->o_items));
         edje_object_part_swallow(item->o_item, "e.swallow.icon", item->o_icon);
@@ -753,7 +869,8 @@ _tasks_config_item_get(const char *id)
 
    config = E_NEW(Config_Item, 1);
    config->id = eina_stringshare_add(id);
-   config->show_all = 0;
+   config->show_all_desktops = 0;
+   config->show_all_screens = 0;
    config->minw = 100;
    config->minh = 32;
 
@@ -784,6 +901,14 @@ _tasks_cb_menu_configure(void *data, E_Menu *m EINA_UNUSED, E_Menu_Item *mi EINA
    _config_tasks_module(tasks->config);
 }
 
+static void
+_tasks_cb_item_del(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   Tasks_Item *item = data;
+
+   _tasks_item_preview_del(item);
+}
+
 static void
 _tasks_cb_item_mouse_down(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info)
 {
@@ -792,6 +917,9 @@ _tasks_cb_item_mouse_down(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA
 
    item = (Tasks_Item *)data;
    ev = event_info;
+
+   _tasks_item_preview_del(item);
+
    if (ev->button == 3)
      {
         E_Menu *m;
@@ -831,6 +959,138 @@ _tasks_cb_item_mouse_down(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA
      }
 }
 
+static Eina_Bool
+_tasks_cb_timer_del(void *data)
+{
+   Tasks_Item *item = data;
+
+   if (item->o_preview)
+     evas_object_del(item->o_preview);
+   if (item->timer)
+     ecore_timer_del(item->timer);
+   item->timer = item->o_preview = NULL;
+
+   return ECORE_CALLBACK_CANCEL;
+}
+
+static void
+_tasks_item_preview_add(Tasks_Item *item)
+{
+   E_Client *ec;
+   Evas_Object *o, *ot, *or, *img;
+   Evas_Coord ox, oy, ow, oh, size;
+   double n;
+
+   if (item->o_preview) _tasks_cb_timer_del(item);
+
+   evas_object_geometry_get(item->o_item, &ox, &oy, &ow, &oh);
+
+   item->o_preview = o = elm_ctxpopup_add(e_comp->elm);
+   evas_object_pass_events_set(o, EINA_TRUE);
+   elm_object_focus_allow_set(o, EINA_FALSE);
+   elm_object_tree_focus_allow_set(o, EINA_FALSE);
+   elm_object_style_set(o, "noblock");
+   evas_object_layer_set(o, E_LAYER_POPUP);
+
+   ot = elm_table_add(o);
+   evas_object_size_hint_align_set(ot, 0, 0);
+   elm_object_content_set(o, ot);
+   evas_object_show(ot);
+
+   ec = item->client;
+   img = e_comp_object_util_mirror_add(ec->frame);
+   evas_object_size_hint_aspect_set(img, EVAS_ASPECT_CONTROL_BOTH, ec->client.w, ec->client.h);
+   evas_object_show(img);
+
+   or = evas_object_rectangle_add(evas_object_evas_get(o));
+
+   size = item->tasks->config->preview_size;
+   if (ec->client.w > ec->client.h)
+     {
+        n = size * (1.0 / ec->client.w);
+        evas_object_size_hint_min_set(img, size, n * ec->client.h);
+        evas_object_size_hint_max_set(img, size, n * ec->client.h);
+        evas_object_size_hint_min_set(or,  size + 1, (n * ec->client.h) + 1);
+     }
+   else
+     {
+        n = size * (1.0 / ec->client.h);
+        evas_object_size_hint_min_set(img, n * ec->client.w, size);
+        evas_object_size_hint_max_set(img, n * ec->client.w, size);
+        evas_object_size_hint_min_set(or, (n * ec->client.w) + 1, size + 1);
+     }
+
+   elm_table_pack(ot, or, 0, 0, 1, 1);
+   elm_table_pack(ot, img, 0, 0, 1, 1);
+
+   switch (item->tasks->gcc->gadcon->orient)
+     {
+        case E_GADCON_ORIENT_BOTTOM:
+        case E_GADCON_ORIENT_CORNER_BR:
+        case E_GADCON_ORIENT_CORNER_BL:
+          evas_object_move(o, ox + (ow / 2), oy);
+          elm_ctxpopup_direction_priority_set(o, ELM_CTXPOPUP_DIRECTION_UP, ELM_CTXPOPUP_DIRECTION_DOWN,
+                                              ELM_CTXPOPUP_DIRECTION_LEFT, ELM_CTXPOPUP_DIRECTION_RIGHT);
+          break;
+        case E_GADCON_ORIENT_TOP:
+        case E_GADCON_ORIENT_CORNER_TL:
+        case E_GADCON_ORIENT_CORNER_TR:
+          evas_object_move(o, ox + (ow / 2), oy + oh);
+          elm_ctxpopup_direction_priority_set(o, ELM_CTXPOPUP_DIRECTION_DOWN, ELM_CTXPOPUP_DIRECTION_UP,
+                                              ELM_CTXPOPUP_DIRECTION_LEFT, ELM_CTXPOPUP_DIRECTION_RIGHT);
+          break;
+        case E_GADCON_ORIENT_LEFT:
+        case E_GADCON_ORIENT_CORNER_LB:
+        case E_GADCON_ORIENT_CORNER_LT:
+          evas_object_move(o, ox + ow, oy + (oh / 2));
+          elm_ctxpopup_direction_priority_set(o, ELM_CTXPOPUP_DIRECTION_RIGHT, ELM_CTXPOPUP_DIRECTION_DOWN,
+                                              ELM_CTXPOPUP_DIRECTION_LEFT, ELM_CTXPOPUP_DIRECTION_UP);
+          break;
+        case E_GADCON_ORIENT_RIGHT:
+        case E_GADCON_ORIENT_CORNER_RB:
+        case E_GADCON_ORIENT_CORNER_RT:
+          evas_object_move(o, ox, oy + (oh / 2));
+          elm_ctxpopup_direction_priority_set(o, ELM_CTXPOPUP_DIRECTION_LEFT, ELM_CTXPOPUP_DIRECTION_DOWN,
+                                              ELM_CTXPOPUP_DIRECTION_RIGHT, ELM_CTXPOPUP_DIRECTION_UP);
+          break;
+        default:
+          evas_object_move(o, ox + (ow / 2), oy);
+          elm_ctxpopup_direction_priority_set(o, ELM_CTXPOPUP_DIRECTION_UP, ELM_CTXPOPUP_DIRECTION_DOWN,
+                                              ELM_CTXPOPUP_DIRECTION_LEFT, ELM_CTXPOPUP_DIRECTION_RIGHT);
+          break;
+     }
+   evas_object_pass_events_set(o, 1);
+   evas_object_show(o);
+}
+
+static void
+_tasks_cb_item_mouse_in(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   Tasks_Item *item = data;
+   Eina_Bool show;
+
+   if (!item->tasks->config->preview) return;
+
+   show = !item->iconifying;
+   if ((item->tasks->gcc->gadcon->shelf) &&
+       (item->tasks->gcc->gadcon->shelf->hide_animator))
+     show = 0;
+
+   if (show) _tasks_item_preview_add(item);
+}
+
+static void
+_tasks_cb_item_mouse_out(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info EINA_UNUSED)
+{
+   Tasks_Item *item = data;
+
+   if (item->o_preview)
+     {
+        elm_ctxpopup_dismiss(item->o_preview);
+        item->timer = ecore_timer_add(1.0, _tasks_cb_timer_del, item);
+     }
+}
+
 static void
 _tasks_cb_item_mouse_wheel(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event_info)
 {
@@ -860,9 +1120,12 @@ _tasks_cb_item_mouse_up(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_U
 
    ev = event_info;
    item = data;
+
+   _tasks_item_preview_del(item);
+
    if (ev->button == 1)
      {
-        if (!item->client->sticky && item->tasks->config->show_all)
+        if (!item->client->sticky && item->tasks->config->show_all_desktops)
           e_desk_show(item->client->desk);
         if (evas_key_modifier_is_set(ev->modifiers, "Alt"))
           {
@@ -889,7 +1152,7 @@ _tasks_cb_item_mouse_up(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_U
           {
              e_client_act_close_begin(item->client);
           }
-        else
+        else if (!item->iconifying)
           {
              if (item->client->iconic)
                {
@@ -912,7 +1175,7 @@ _tasks_cb_item_mouse_up(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_U
      }
    else if (ev->button == 2)
      {
-        if (!item->client->sticky && item->tasks->config->show_all)
+        if (!item->client->sticky && item->tasks->config->show_all_desktops)
           e_desk_show(item->client->desk);
         evas_object_raise(item->client->frame);
         evas_object_focus_set(item->client->frame, 1);
@@ -938,7 +1201,10 @@ _tasks_cb_event_client_add(void *data EINA_UNUSED, int type EINA_UNUSED, void *e
    EINA_LIST_FOREACH(tasks_config->tasks, l, tasks)
      {
         if ((!tasks->clients) || (!eina_list_data_find(tasks->clients, ev->ec)))
-          tasks->clients = eina_list_append(tasks->clients, ev->ec);
+          {
+             e_object_delfn_add(E_OBJECT(ev->ec), _tasks_cb_client_del, NULL);
+             tasks->clients = eina_list_append(tasks->clients, ev->ec);
+          }
      }
    _tasks_refill_all();
    return EINA_TRUE;
@@ -948,14 +1214,8 @@ static Eina_Bool
 _tasks_cb_event_client_remove(void *data EINA_UNUSED, int type EINA_UNUSED, void *event)
 {
    E_Event_Client *ev = event;
-   Tasks *tasks;
-   Eina_List *l;
 
-   EINA_LIST_FOREACH(tasks_config->tasks, l, tasks)
-     {
-        tasks->clients = eina_list_remove(tasks->clients, ev->ec);
-     }
-   _tasks_refill_all();
+   _taks_client_remove(ev->ec);
    return EINA_TRUE;
 }
 
