@@ -4,7 +4,6 @@
 
 #include <Ecore_Drm2.h>
 #include <Elput.h>
-#include "e_drm2.x"
 
 static Ecore_Event_Handler *seat_handler;
 
@@ -56,6 +55,7 @@ _e_mod_drm_cb_activate(void *data EINA_UNUSED, int type EINA_UNUSED, void *event
      {
         if (session_state) goto end;
         session_state = EINA_TRUE;
+        e_backlight_suspend_set(EINA_FALSE);
 
         ecore_evas_show(e_comp->ee);
         evas_damage_rectangle_add(e_comp->evas, 0, 0, e_comp->w, e_comp->h);
@@ -73,6 +73,8 @@ _e_mod_drm_cb_activate(void *data EINA_UNUSED, int type EINA_UNUSED, void *event
    else
      {
         session_state = EINA_FALSE;
+        e_backlight_suspend_set(EINA_TRUE);
+
         ecore_evas_hide(e_comp->ee);
         edje_file_cache_flush();
         edje_collection_cache_flush();
@@ -402,12 +404,12 @@ _drm2_randr_create(void)
                {
                   unsigned int refresh;
 
-                  e_drm2_output_info_get(output,
-                                         &s->config.geom.x,
-                                         &s->config.geom.y,
-                                         &s->config.geom.w,
-                                         &s->config.geom.h,
-                                         &refresh);
+                  ecore_drm2_output_info_get(output,
+                                             &s->config.geom.x,
+                                             &s->config.geom.y,
+                                             &s->config.geom.w,
+                                             &s->config.geom.h,
+                                             &refresh);
                   s->config.mode.w = s->config.geom.w;
                   s->config.mode.h = s->config.geom.h;
                   s->config.mode.refresh = refresh;
@@ -419,7 +421,7 @@ _drm2_randr_create(void)
                          s->config.geom.w, s->config.geom.h);
                }
 
-             outrot = e_drm2_output_rotation_get(output);
+             outrot = ecore_drm2_output_rotation_get(output);
              if (outrot & ECORE_DRM2_ROTATION_NORMAL)
                s->config.rotation = 0;
              else if (outrot & ECORE_DRM2_ROTATION_90)
@@ -665,6 +667,7 @@ _drm2_randr_apply(void)
                     {
                        Ecore_Drm2_Output_Mode *mode;
                        Ecore_Drm2_Rotation orient = ECORE_DRM2_ROTATION_NORMAL;
+                       Ecore_Drm2_Relative_Mode relmode;
 
                        mode = _drm2_mode_screen_find(screenconf[i], outconf[i]);
                        if (screenconf[i]->config.rotation == 0)
@@ -690,8 +693,19 @@ _drm2_randr_apply(void)
 
                        ecore_drm2_output_relative_to_set(outconf[i],
                                                          screenconf[i]->config.relative.to);
-                       ecore_drm2_output_relative_mode_set(outconf[i],
-                                                           screenconf[i]->config.relative.mode);
+                       relmode = ECORE_DRM2_RELATIVE_MODE_NONE;
+                       switch (screenconf[i]->config.relative.mode)
+                         {
+                          case E_RANDR2_RELATIVE_UNKNOWN:  relmode = ECORE_DRM2_RELATIVE_MODE_UNKNOWN; break;
+                          case E_RANDR2_RELATIVE_NONE:     relmode = ECORE_DRM2_RELATIVE_MODE_NONE; break;
+                          case E_RANDR2_RELATIVE_CLONE:    relmode = ECORE_DRM2_RELATIVE_MODE_CLONE; break;
+                          case E_RANDR2_RELATIVE_TO_LEFT:  relmode = ECORE_DRM2_RELATIVE_MODE_TO_LEFT; break;
+                          case E_RANDR2_RELATIVE_TO_RIGHT: relmode = ECORE_DRM2_RELATIVE_MODE_TO_RIGHT; break;
+                          case E_RANDR2_RELATIVE_TO_ABOVE: relmode = ECORE_DRM2_RELATIVE_MODE_TO_ABOVE; break;
+                          case E_RANDR2_RELATIVE_TO_BELOW: relmode = ECORE_DRM2_RELATIVE_MODE_TO_BELOW; break;
+                          default: break;
+                         }
+                       ecore_drm2_output_relative_mode_set(outconf[i], relmode);
 
                        if (screenconf[i]->config.priority == top_priority)
                          {
@@ -702,7 +716,7 @@ _drm2_randr_apply(void)
                        ecore_drm2_output_enabled_set(outconf[i],
                                                      screenconf[i]->config.enabled);
 
-                       e_drm2_output_rotation_set(outconf[i], orient);
+                       ecore_drm2_output_rotation_set(outconf[i], orient);
 
                        ecore_evas_rotation_with_resize_set(e_comp->ee,
                                                            screenconf[i]->config.rotation);
@@ -889,16 +903,10 @@ _drm_device_del(void *data EINA_UNUSED, const Efl_Event *event)
    if (efl_input_device_type_get(event->info) == EFL_INPUT_DEVICE_TYPE_SEAT) return;
    seat = efl_input_device_seat_get(event->info);
 
-   if (seat != evas_default_device_get(e_comp->evas, EFL_INPUT_DEVICE_TYPE_SEAT)) return;
-#ifdef EFL_VERSION_1_23
+   if (seat != evas_default_device_get(e_comp->evas, EVAS_DEVICE_CLASS_SEAT)) return;
    if (!efl_input_device_is_pointer_type_get(event->info)) return;
    if (efl_input_device_pointer_device_count_get(seat) == 1)
      ecore_evas_cursor_device_unset(e_comp->ee, event->info);
-#else
-   if (!efl_input_device_has_pointer_caps(event->info)) return;
-   if (efl_input_device_has_pointer_caps(seat) == 1)
-     ecore_evas_cursor_device_unset(e_comp->ee, event->info);
-#endif
 }
 
 E_API void *
@@ -917,8 +925,6 @@ e_modapi_init(E_Module *m)
    /*      return NULL; */
    /*   } */
 
-   if (!e_drm2_compat_init()) return NULL;
-
    if (e_comp_config_get()->engine == E_COMP_ENGINE_GL)
      {
         e_comp->ee = ecore_evas_new("gl_drm", 0, 0, 1, 1, NULL);
@@ -938,7 +944,7 @@ e_modapi_init(E_Module *m)
           }
         else
           {
-             fprintf(stderr, "Could not create ecore_evas_drm canvas");
+             fprintf(stderr, "Could not create ecore_evas_drm canvas\n");
              return NULL;
           }
      }
@@ -953,14 +959,7 @@ e_modapi_init(E_Module *m)
    ecore_evas_screen_geometry_get(e_comp->ee, NULL, NULL, &w, &h);
    if (!e_comp_canvas_init(w, h)) return NULL;
 
-#ifdef EFL_VERSION_1_21
    arr[0].desc = EFL_CANVAS_SCENE_EVENT_DEVICE_REMOVED;
-#else
-   if (E_EFL_VERSION_MINIMUM(1, 20, 99))
-     arr[0].desc = dlsym(NULL, "_EFL_CANVAS_SCENE_EVENT_DEVICE_REMOVED");
-   if (!arr[0].desc)
-     arr[0].desc = dlsym(NULL, "_EFL_CANVAS_EVENT_DEVICE_REMOVED");
-#endif
    ecore_evas_pointer_xy_get(e_comp->ee, &e_comp_wl->ptr.x,
                              &e_comp_wl->ptr.y);
    evas_event_feed_mouse_in(e_comp->evas, 0, NULL);
@@ -1000,6 +999,5 @@ e_modapi_shutdown(E_Module *m EINA_UNUSED)
    activate_handler = NULL;
 
    E_FREE_FUNC(input_handler, ecore_event_handler_del);
-   e_drm2_compat_shutdown();
    return 1;
 }
